@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.c,v 1.228 2003/12/23 19:30:51 miguelfreitas Exp $
+ * $Id: actions.c,v 1.244 2004/07/02 21:40:53 f1rmb Exp $
  *
  * implementation of all the various actions for the gui (play, stop, open, pause...)
  *
@@ -43,19 +43,43 @@
 
 extern gGui_t          *gGui;
 extern _panel_t        *panel;
+filebrowser_t          *load_stream = NULL, *load_sub = NULL;
 
 static pthread_mutex_t new_pos_mutex =  PTHREAD_MUTEX_INITIALIZER;
 
+void reparent_all_windows(void) {
+  int                    i;
+  typedef struct {
+    int                 (*visible)(void);
+    void                (*reparent)(void);
+  } reparent_t;
+  reparent_t _reparent[] = {
+    { panel_is_visible,         panel_reparent },
+    { mrl_browser_is_visible,   mrl_browser_reparent },
+    { playlist_is_visible,      playlist_reparent },
+    { control_is_visible,       control_reparent },
+    { setup_is_visible,         setup_reparent },
+    { viewlog_is_visible,       viewlog_reparent },
+    { kbedit_is_visible,        kbedit_reparent },
+    { event_sender_is_visible,  event_sender_reparent },
+    { stream_infos_is_visible,  stream_infos_reparent },
+    { tvset_is_visible,         tvset_reparent },
+    { pplugin_is_visible,       pplugin_reparent },
+    { help_is_visible,          help_reparent },
+    { NULL,                     NULL}
+  };
+  
+  for(i = 0; _reparent[i].visible; i++) {
+    if(_reparent[i].visible())
+      _reparent[i].reparent();
+  }
+
+}
+
 void raise_window(Window window, int visible, int running) {
   if(window) {
     if(visible && running) {
-      XLockDisplay(gGui->display);
-      XUnmapWindow(gGui->display, window);
-      XRaiseWindow(gGui->display, window);
-      XMapWindow(gGui->display, window);
-      if(!gGui->use_root_window)
-        XSetTransientForHint (gGui->display, window, gGui->video_window);
-      XUnlockDisplay(gGui->display);
+      reparent_window(window);
       layer_above_video(window);
     }
   }
@@ -84,7 +108,7 @@ void toggle_window(Window window, xitk_widget_list_t *widget_list, int *visible,
       XLockDisplay(gGui->display);
       XRaiseWindow(gGui->display, window);
       XMapWindow(gGui->display, window);
-      if(!gGui->use_root_window)
+      if(!gGui->use_root_window && gGui->video_display == gGui->display)
         XSetTransientForHint (gGui->display, window, gGui->video_window);
       XUnlockDisplay(gGui->display);
       layer_above_video(window);
@@ -158,7 +182,7 @@ void gui_display_logo(void) {
     visual_anim_stop();
 
   if(gGui->display_logo)
-    (void) gui_xine_open_and_play((char *)gGui->logo_mrl, NULL, 0, 0, 0, 0);
+    (void) gui_xine_open_and_play((char *)gGui->logo_mrl, NULL, 0, 0, 0, 0, 1);
 
   gGui->logo_mode = 1;
   
@@ -199,12 +223,11 @@ static int _gui_xine_play(xine_stream_t *stream,
 
   }
 
-  if((ret = xine_play(stream, start_pos, start_time_in_secs)) == 0) {
+  if((ret = xine_play(stream, start_pos, start_time_in_secs)) == 0)
     gui_handle_xine_error(stream, NULL);
-  }
   else {
     char *ident;
-    
+
     if(gGui->logo_mode != 2)
       gGui->logo_mode = 0;
     
@@ -214,7 +237,6 @@ static int _gui_xine_play(xine_stream_t *stream,
 	stream_infos_update_infos();
       
       if(update_mmk && ((ident = stream_infos_get_ident_from_stream(stream)) != NULL)) {
-	
 	if(gGui->mmk.ident)
 	  free(gGui->mmk.ident);
 	if(gGui->playlist.mmk[gGui->playlist.cur]->ident)
@@ -323,12 +345,12 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
     memset(&buffer, 0, sizeof(buffer));
 
     if(v_unhandled && a_unhandled) {
-      sprintf(buffer, _("The stream '%s' isn't supported by xine:\n\n"),
-	      (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current]);
+      snprintf(buffer, sizeof(buffer), _("The stream '%s' isn't supported by xine:\n\n"),
+	       (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current]);
     }
     else {
-      sprintf(buffer, _("The stream '%s' use an unsupported codec:\n\n"),
-	      (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current]);
+      snprintf(buffer, sizeof(buffer), _("The stream '%s' use an unsupported codec:\n\n"),
+	       (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current]);
     }
     
     if(v_unhandled) {
@@ -348,7 +370,7 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
       
       minfo = xine_get_meta_info(stream, XINE_META_INFO_AUDIOCODEC);
       afcc = xine_get_stream_info(stream, XINE_STREAM_INFO_AUDIO_FOURCC);
-      sprintf(buffer, _("%sAudio Codec: %s (%s)\n"), buffer,
+      sprintf(buffer,  _("%sAudio Codec: %s (%s)\n"), buffer,
 	      (minfo && strlen(minfo)) ? (char *) minfo : _("Unavailable"), 
 	      (get_fourcc_string(afcc)));
     }
@@ -375,7 +397,7 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
 					       NULL, 400, ALIGN_CENTER,
 					       buffer);
       XLockDisplay(gGui->display);
-      if(!gGui->use_root_window)
+      if(!gGui->use_root_window && gGui->video_display == gGui->display)
 	XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
       XSync(gGui->display, False);
       XUnlockDisplay(gGui->display);
@@ -394,7 +416,7 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
 }
 
 int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos, 
-			   int start_time, int av_offset, int spu_offset) {
+			   int start_time, int av_offset, int spu_offset, int report_error) {
   char *mrl = _mrl;
   
   if(gGui->verbosity)
@@ -428,7 +450,7 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
 	
 	filename++;
 	memset(&fullfilename, 0, sizeof(fullfilename));
-	sprintf(fullfilename, "%s/%s", xine_get_homedir(), filename);
+	snprintf(fullfilename, sizeof(fullfilename), "%s/%s", xine_get_homedir(), filename);
 	
 	if((fd = fopen(fullfilename, "w+b")) != NULL) {
 	  char  *newmrl, *ident, *sub = NULL;
@@ -447,7 +469,7 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
 
 	  mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur], 
 				  newmrl, ident, sub, start, end, 0, 0);
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	  mrl = gGui->mmk.mrl;
 	}
       }
@@ -462,18 +484,23 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
 
   if(mrl_look_like_playlist(mrl)) {
     if(mediamark_concat_mediamarks(mrl)) {
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-      mrl = gGui->mmk.mrl;
+      gui_set_current_mmk(mediamark_get_current_mmk());
+      mrl        = gGui->mmk.mrl;
+      start_pos  = 0;
+      start_time = gGui->mmk.start;
+      av_offset  = gGui->mmk.av_offset;
+      spu_offset = gGui->mmk.spu_offset;
       playlist_update_playlist();
     }
   }
-  
+
   if(!xine_open(gGui->stream, (const char *) mrl)) {
     
     if(!strcmp(mrl, gGui->mmk.mrl))
       gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
     
-    gui_handle_xine_error(gGui->stream, mrl);
+    if(report_error)
+      gui_handle_xine_error(gGui->stream, mrl);
     return 0;
   }
   
@@ -488,13 +515,13 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
   
   xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
   xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, spu_offset);
-
+  
   if(!gui_xine_play(gGui->stream, start_pos, start_time, 1)) {
     if(!strcmp(mrl, gGui->mmk.mrl))
       gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
     return 0;
   }
-  
+
   if(!strcmp(mrl, gGui->mmk.mrl))
     gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
 
@@ -503,6 +530,21 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
   return 1;
 }
 
+int gui_open_and_play_alternates(mediamark_t *mmk, const char *sub) {
+  char *alt;
+  
+  if(!(alt = mediamark_get_current_alternate_mrl(mmk)))
+    alt = mediamark_get_first_alternate_mrl(mmk);
+  
+  do {
+
+    if(gui_xine_open_and_play(alt, gGui->mmk.sub, 0, 0, 0, 0, 0))
+      return 1;
+
+  } while((alt = mediamark_get_next_alternate_mrl(&(gGui->mmk))));
+  return 0;
+}
+
 /*
  * Callback-functions for widget-button click
  */
@@ -517,7 +559,7 @@ void gui_exit (xitk_widget_t *w, void *data) {
       xine_stop(gGui->visual_anim.stream);
 
       while(xine_get_status(gGui->visual_anim.stream) != XINE_STATUS_STOP)
-	xine_usec_sleep(50000);
+      	xine_usec_sleep(50000);
       
       audio_source = xine_get_audio_source(gGui->stream);
       (void) xine_post_wire_audio_port(audio_source, gGui->ao_port);
@@ -528,11 +570,27 @@ void gui_exit (xitk_widget_t *w, void *data) {
   }
   
   gGui->on_quit = 1;
-  
+
   panel_deinit();
   playlist_deinit();
-  control_deinit();
   mrl_browser_deinit();
+  control_deinit();
+  
+  setup_end();
+  viewlog_end();
+  kbedit_end();
+  event_sender_end();
+  stream_infos_end();
+  tvset_end();
+  pplugin_end();
+  help_end();
+  download_skin_end();
+  
+  if(load_stream)
+    filebrowser_end(load_stream);
+  if(load_sub)
+    filebrowser_end(load_sub);
+
   gui_deinit();
 
   if(video_window_is_visible())
@@ -551,10 +609,6 @@ void gui_exit (xitk_widget_t *w, void *data) {
   //     gui_set_fullscreen_mode(NULL,NULL);
 #endif
    
-  destroy_mrl_browser();
-  control_exit(NULL, NULL);
-  playlist_exit(NULL, NULL);
-
   osd_deinit();
 
   config_update_num("gui.amp_level", gGui->mixer.amp_level);
@@ -579,9 +633,13 @@ void gui_exit (xitk_widget_t *w, void *data) {
 
   if(gGui->vo_port)
     xine_close_video_driver(gGui->xine, gGui->vo_port);
+  if(gGui->vo_none)
+    xine_close_video_driver(gGui->xine, gGui->vo_none);
 
   if(gGui->ao_port)
     xine_close_audio_driver(gGui->xine, gGui->ao_port);
+  if(gGui->ao_none)
+    xine_close_audio_driver(gGui->xine, gGui->ao_none);
 
   xine_exit(gGui->xine); 
 
@@ -598,7 +656,8 @@ void gui_exit (xitk_widget_t *w, void *data) {
    * This prevent xine waiting till the end of time for an
    * XEvent when lirc (and futur other control ways) is used to quit .
    */
-  gui_send_expose_to_window(gGui->video_window);
+  if( gGui->video_display == gGui->display )
+    gui_send_expose_to_window(gGui->video_window);
   xitk_skin_unload_config(gGui->skin_config);
 }
 
@@ -625,11 +684,16 @@ void gui_play (xitk_widget_t *w, void *data) {
     }
     
     if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-			       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
-      
-      if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
-	gui_exit(NULL, NULL);
-      gui_display_logo();
+			       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+			       !mediamark_have_alternates(&(gGui->mmk)))) {
+
+      if(!mediamark_have_alternates(&(gGui->mmk)) ||
+	 !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+	
+	if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+	  gui_exit(NULL, NULL);
+	gui_display_logo();
+      }
     }
   }
   else {
@@ -663,11 +727,11 @@ void gui_stop (xitk_widget_t *w, void *data) {
 
   if(is_playback_widgets_enabled()) {
     if(!gGui->playlist.num) {
-      gui_set_current_mrl(NULL);
+      gui_set_current_mmk(NULL);
       enable_playback_controls(0);
     }
     else if(gGui->playlist.num && (strcmp((mediamark_get_current_mrl()), gGui->mmk.mrl))) {
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
     }
   }
   gui_display_logo();
@@ -718,13 +782,7 @@ void gui_eject(xitk_widget_t *w, void *data) {
 	for(i = 0; i < gGui->playlist.num; i++) {
 	  if(strncasecmp(gGui->playlist.mmk[i]->mrl, tok, strlen(tok))) {
 	    
-	    if(!new_num)
-	      mmk = (mediamark_t **) xine_xmalloc(sizeof(mediamark_t *) * 2);
-	    else { 
-	      if(new_num > 1) {
-		mmk = (mediamark_t **) realloc(mmk, sizeof(mediamark_t *) * (new_num + 2));
-	      }
-	    }
+	    mmk = (mediamark_t **) realloc(mmk, sizeof(mediamark_t *) * (new_num + 2));
 	    
 	    (void) mediamark_store_mmk(&mmk[new_num], 
 				       gGui->playlist.mmk[i]->mrl,
@@ -760,8 +818,7 @@ void gui_eject(xitk_widget_t *w, void *data) {
 	for(i = gGui->playlist.cur; i < gGui->playlist.num; i++)
 	  gGui->playlist.mmk[i] = gGui->playlist.mmk[i + 1];
 	
-	gGui->playlist.mmk = (mediamark_t **) 
-	  realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
+	gGui->playlist.mmk = (mediamark_t **) realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
 
 	gGui->playlist.mmk[gGui->playlist.num] = NULL;
 
@@ -775,7 +832,7 @@ void gui_eject(xitk_widget_t *w, void *data) {
 
     }
     
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+    gui_set_current_mmk(mediamark_get_current_mmk());
     playlist_update_playlist();
   }
   else {
@@ -787,20 +844,22 @@ void gui_eject(xitk_widget_t *w, void *data) {
 
 void gui_toggle_visibility(xitk_widget_t *w, void *data) {
 
-  if(panel_is_visible()) {
+  if(panel_is_visible() && (!gGui->use_root_window)) {
+
     video_window_set_visibility(!(video_window_is_visible()));
-    XLockDisplay(gGui->display);
-    XRaiseWindow(gGui->display, gGui->panel_window); 
-    XMapWindow(gGui->display, gGui->panel_window); 
-    XUnlockDisplay(gGui->display);
-    layer_above_video(gGui->panel_window);
+
+    /* We need to reparent all visible windows because of redirect tweaking */
+    reparent_all_windows();
 
     /* (re)start/stop visual animation */
     if(video_window_is_visible()) {
+      layer_above_video(gGui->panel_window);
+      
       if(gGui->visual_anim.enabled && (gGui->visual_anim.running == 2))
 	visual_anim_play();
     }
     else {
+
       if(gGui->visual_anim.running) {
 	visual_anim_stop();
 	gGui->visual_anim.running = 2;
@@ -824,7 +883,7 @@ static void set_fullscreen_mode(int fullscreen_mode) {
   int pplugin      = pplugin_is_visible();
   int help         = help_is_visible();
 
-  if(!(video_window_is_visible()))
+  if((!(video_window_is_visible())) || gGui->use_root_window)
     return;
 
   if(panel)
@@ -920,7 +979,7 @@ void gui_toggle_aspect(int aspect) {
   if (panel_is_visible())  {
     XLockDisplay(gGui->display);
     XRaiseWindow(gGui->display, gGui->panel_window);
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
     XUnlockDisplay(gGui->display);
     
@@ -935,7 +994,7 @@ void gui_toggle_interlaced(void) {
   if (panel_is_visible())  {
     XLockDisplay(gGui->display);
     XRaiseWindow(gGui->display, gGui->panel_window);
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
     XUnlockDisplay(gGui->display);
   }
@@ -1004,7 +1063,7 @@ void gui_change_speed_playback(xitk_widget_t *w, void *data) {
   osd_update_status();
 }
 
-static void *gui_set_current_position_thread(void *data) {
+static void *_gui_set_current_position(void *data) {
   int  pos = (int) data;
   int  update_mmk = 0;
   
@@ -1079,7 +1138,7 @@ static void *gui_set_current_position_thread(void *data) {
   return NULL;
 }
 
-static void *gui_seek_relative_thread(void *data) {
+static void *_gui_seek_relative(void *data) {
   int off_sec = (int)data;
   int sec, pos;
   
@@ -1137,7 +1196,7 @@ static void *gui_seek_relative_thread(void *data) {
 
 void gui_set_current_position (int pos) {
   int        err;
-  pthread_t  sthread;
+  pthread_t  pth;
 
   if(gGui->new_pos == -1) {
     
@@ -1147,7 +1206,7 @@ void gui_set_current_position (int pos) {
     gGui->new_pos = pos;
     pthread_mutex_unlock(&new_pos_mutex);
     
-    if((err = pthread_create(&sthread, NULL, gui_set_current_position_thread, (void *)pos)) != 0) {
+    if((err = pthread_create(&pth, NULL, _gui_set_current_position, (void *)pos)) != 0) {
       printf(_("%s(): can't create new thread (%s)\n"), __XINE_FUNCTION__, strerror(err));
       abort();
     }
@@ -1162,16 +1221,17 @@ void gui_set_current_position (int pos) {
 
 void gui_seek_relative (int off_sec) {
   int        err;
-  pthread_t  sthread;
+  pthread_t  pth;
   
-  if((err = pthread_create(&sthread, NULL, gui_seek_relative_thread, (void *)off_sec)) != 0) {
+  if((err = pthread_create(&pth, NULL, _gui_seek_relative, (void *)off_sec)) != 0) {
     printf(_("%s(): can't create new thread (%s)\n"), __XINE_FUNCTION__, strerror(err));
     abort();
   }
 }
 
-void gui_dndcallback(char *filename) {
-  int more_than_one = -1;
+static void *_gui_dndcallback(void *data) {
+  int   more_than_one = -2;
+  char *filename = (char *) data;
 
   if(filename) {
     char  buffer[strlen(filename) + 10];
@@ -1204,14 +1264,14 @@ void gui_dndcallback(char *filename) {
 	    goto __do_play;
 	  }
 	  else
-	    sprintf(buffer, "file:/%s", p);
+	    snprintf(buffer, sizeof(buffer), "file:/%s", p);
 	}
 	else {
-	  sprintf(buffer2, "/%s", p);
+	  snprintf(buffer2, sizeof(buffer2), "/%s", p);
 	  
 	  /* file don't exist, add it anyway */
 	  if((stat(buffer2, &pstat)) == -1)
-	    sprintf(buffer, "%s", filename);
+	    snprintf(buffer, sizeof(buffer), "%s", filename);
 	  else {
 	    if(is_a_dir(buffer2)) {
 	      
@@ -1223,7 +1283,7 @@ void gui_dndcallback(char *filename) {
 	      goto __do_play;
 	    }
 	    else
-	      sprintf(buffer, "file:/%s", buffer2);
+	      snprintf(buffer, sizeof(buffer), "file:/%s", buffer2);
 	  }
 	  
 	}
@@ -1234,7 +1294,7 @@ void gui_dndcallback(char *filename) {
       }
     }
     else
-      sprintf(buffer, "%s", filename);
+      snprintf(buffer, sizeof(buffer), "%s", filename);
     
     if(is_a_dir(buffer)) {
       if(buffer[strlen(buffer) - 1] == '/')
@@ -1243,37 +1303,68 @@ void gui_dndcallback(char *filename) {
       mediamark_collect_from_directory(buffer);
       more_than_one = gGui->playlist.cur;
     }
-    else
-      mediamark_add_entry(buffer, buffer, NULL, 0, -1, 0, 0);
+    else {
+      
+      if(mrl_look_like_playlist(buffer)) {
+	int cur = gGui->playlist.cur;
+	
+	more_than_one = (gGui->playlist.cur - 1);
+	if(mediamark_concat_mediamarks(buffer))
+	  gGui->playlist.cur = cur;
+	else
+	  mediamark_append_entry(buffer, buffer, NULL, 0, -1, 0, 0);
+      }
+      else
+	mediamark_append_entry(buffer, buffer, NULL, 0, -1, 0, 0);
+
+    }
     
   __do_play:
-    
+
     playlist_update_playlist();
 
     if(!(gGui->playlist.control & PLAYLIST_CONTROL_IGNORE)) {
       if((xine_get_status(gGui->stream) == XINE_STATUS_STOP) || gGui->logo_mode) {
-	if((more_than_one > -1) && ((more_than_one + 1) < gGui->playlist.num))
+
+	if((more_than_one > -2) && ((more_than_one + 1) < gGui->playlist.num))
 	  gGui->playlist.cur = more_than_one + 1;
 	else
 	  gGui->playlist.cur = gGui->playlist.num - 1;
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+
+	gui_set_current_mmk(mediamark_get_current_mmk());
 	if(gGui->smart_mode)
 	  gui_play(NULL, NULL);
+
       }
     }
     
     if((!is_playback_widgets_enabled()) && gGui->playlist.num)
       enable_playback_controls(1);
   }
+
+  pthread_exit(NULL);
+}
+
+void gui_dndcallback(char *filename) {
+  int        err;
+  pthread_t  pth;
+
+  if((err = pthread_create(&pth, NULL, _gui_dndcallback, (void *)filename)) != 0) {
+    printf(_("%s(): can't create new thread (%s)\n"), __XINE_FUNCTION__, strerror(err));
+    abort();
+  }
 }
 
 void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
-  int i;
-  int by_chapter;
+  int            i, by_chapter;
+  mediamark_t   *mmk = mediamark_get_current_mmk();
 
   by_chapter = (gGui->skip_by_chapter &&
 		(xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_HAS_CHAPTERS))) ? 1 : 0;
   
+  if(mmk && mediamark_got_alternate(mmk))
+    mediamark_unset_got_alternate(mmk);
+
   if(((int)data) == GUI_NEXT) {
 
     if(by_chapter) {
@@ -1312,9 +1403,9 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  
 	  gGui->ignore_next = 1;
 	  gGui->playlist.cur = newcur;
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	  if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				     gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
+				     gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1))
 	    gui_display_logo();
 
 	  gGui->ignore_next = 0;
@@ -1353,9 +1444,9 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  gGui->playlist.cur -= value;
 	  
 	  if((gGui->playlist.cur < gGui->playlist.num)) {
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
+				       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1))
 	      gui_display_logo();
 
 	  }
@@ -1374,9 +1465,9 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  gGui->playlist.cur -= value;
 	  
 	  if((gGui->playlist.cur < gGui->playlist.num)) {
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
+				       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1))
 	      gui_display_logo();
 	  }
 	  else
@@ -1387,9 +1478,9 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 	  int newcur = (gGui->playlist.cur - value) + gGui->playlist.num;
 	  
 	  gGui->playlist.cur = newcur;
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	  if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				     gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
+				     gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1))
 	    gui_display_logo();
 	}
 
@@ -1446,7 +1537,7 @@ void gui_mrlbrowser_show(xitk_widget_t *w, void *data) {
 
 }
 
-void gui_set_current_mrl(mediamark_t *mmk) {
+void gui_set_current_mmk(mediamark_t *mmk) {
   
   if(gGui->mmk.mrl)
     free(gGui->mmk.mrl);
@@ -1454,35 +1545,44 @@ void gui_set_current_mrl(mediamark_t *mmk) {
     free(gGui->mmk.ident);
   if(gGui->mmk.sub)
     free(gGui->mmk.sub);
+  if(mediamark_have_alternates(&(gGui->mmk)))
+    mediamark_free_alternates(&(gGui->mmk));
   
   if(mmk) {
-    gGui->mmk.mrl        = strdup(mmk->mrl);
-    gGui->mmk.ident      = strdup(((mmk->ident) ? mmk->ident : mmk->mrl));
-    gGui->mmk.sub        = mmk->sub ? strdup(mmk->sub): NULL;
-    gGui->mmk.start      = mmk->start;
-    gGui->mmk.end        = mmk->end;
-    gGui->mmk.av_offset  = mmk->av_offset;
-    gGui->mmk.spu_offset = mmk->spu_offset;
+    gGui->mmk.mrl           = strdup(mmk->mrl);
+    gGui->mmk.ident         = strdup(((mmk->ident) ? mmk->ident : mmk->mrl));
+    gGui->mmk.sub           = mmk->sub ? strdup(mmk->sub): NULL;
+    gGui->mmk.start         = mmk->start;
+    gGui->mmk.end           = mmk->end;
+    gGui->mmk.av_offset     = mmk->av_offset;
+    gGui->mmk.spu_offset    = mmk->spu_offset;
+    gGui->mmk.got_alternate = mmk->got_alternate;
+    mediamark_duplicate_alternates(mmk, &(gGui->mmk));
   }
   else {
     char buffer[1024];
     
     memset(&buffer, 0, sizeof(buffer));
-    sprintf(buffer, "xine-ui version %s", VERSION);
+    snprintf(buffer, sizeof(buffer), "xine-ui version %s", VERSION);
     
-    gGui->mmk.mrl        = strdup(_("There is no mrl."));
-    gGui->mmk.ident      = strdup(buffer);
-    gGui->mmk.sub        = NULL;
-    gGui->mmk.start      = 0;
-    gGui->mmk.end        = -1;
-    gGui->mmk.av_offset  = 0;
-    gGui->mmk.spu_offset = 0;
+    gGui->mmk.mrl           = strdup(_("There is no mrl."));
+    gGui->mmk.ident         = strdup(buffer);
+    gGui->mmk.sub           = NULL;
+    gGui->mmk.start         = 0;
+    gGui->mmk.end           = -1;
+    gGui->mmk.av_offset     = 0;
+    gGui->mmk.spu_offset    = 0;
+    gGui->mmk.got_alternate = 0;
+    gGui->mmk.alternates    = NULL;
+    gGui->mmk.cur_alt       = NULL;
   }
 
   video_window_set_mrl(gGui->mmk.ident);
   event_sender_update_menu_buttons();
-  panel_update_mrl_display ();
+  panel_update_mrl_display();
   playlist_update_focused_entry();
+  
+  gGui->playlist.ref_append = gGui->playlist.cur;
 }
 
 void gui_control_show(xitk_widget_t *w, void *data) {
@@ -1509,7 +1609,7 @@ void gui_setup_show(xitk_widget_t *w, void *data) {
     if(gGui->use_root_window)
       setup_toggle_visibility(NULL, NULL);
     else
-      setup_exit(NULL, NULL);
+      setup_end();
   }
 }
 
@@ -1523,7 +1623,7 @@ void gui_event_sender_show(xitk_widget_t *w, void *data) {
     if(gGui->use_root_window)
       event_sender_toggle_visibility(NULL, NULL);
     else
-      event_sender_exit(NULL, NULL);
+      event_sender_end();
   }
 }
 
@@ -1592,7 +1692,7 @@ void gui_viewlog_show(xitk_widget_t *w, void *data) {
     if(gGui->use_root_window)
       viewlog_toggle_visibility(NULL, NULL);
     else
-      viewlog_exit(NULL, NULL);
+      viewlog_end();
   }
 }
 
@@ -1606,7 +1706,7 @@ void gui_kbedit_show(xitk_widget_t *w, void *data) {
     if(gGui->use_root_window)
       kbedit_toggle_visibility(NULL, NULL);
     else
-      kbedit_exit(NULL, NULL);
+      kbedit_end();
   }
 }
 
@@ -1620,7 +1720,7 @@ void gui_help_show(xitk_widget_t *w, void *data) {
     if(gGui->use_root_window)
       help_toggle_visibility(NULL, NULL);
     else
-      help_exit(NULL, NULL);
+      help_end();
   }
 }
 
@@ -1710,7 +1810,7 @@ void gui_change_zoom(int zoom_dx, int zoom_dy) {
   if (panel_is_visible())  {
     XLockDisplay(gGui->display);
     XRaiseWindow(gGui->display, gGui->panel_window);
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
     XUnlockDisplay(gGui->display);
   }
@@ -1727,7 +1827,7 @@ void gui_reset_zoom(void) {
   if (panel_is_visible())  {
     XLockDisplay(gGui->display);
     XRaiseWindow(gGui->display, gGui->panel_window);
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
     XUnlockDisplay(gGui->display);
   }
@@ -1777,8 +1877,8 @@ void gui_add_mediamark(void) {
     if(gui_xine_get_pos_length(gGui->stream, NULL, &secs, NULL)) {
       secs /= 1000;
       
-      mediamark_add_entry(gGui->mmk.mrl, gGui->mmk.ident, 
-			  gGui->mmk.sub, secs, -1, gGui->mmk.av_offset, gGui->mmk.spu_offset);
+      mediamark_append_entry(gGui->mmk.mrl, gGui->mmk.ident, 
+			     gGui->mmk.sub, secs, -1, gGui->mmk.av_offset, gGui->mmk.spu_offset);
       playlist_update_playlist();
     }
   }
@@ -1787,10 +1887,15 @@ void gui_add_mediamark(void) {
 static void fileselector_cancel_callback(filebrowser_t *fb) {
   char *cur_dir = filebrowser_get_current_dir(fb);
 
-  if(cur_dir && strlen(cur_dir)) {
-    sprintf(gGui->curdir, "%s", cur_dir);
-    config_update_string("input.file_origin_path", gGui->curdir);
+  if(fb == load_stream) {
+    if(cur_dir && strlen(cur_dir)) {
+      snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", cur_dir);
+      config_update_string("input.file_origin_path", gGui->curdir);
+    }
+    load_stream = NULL;
   }
+  else if(fb == load_sub)
+    load_sub = NULL;
 }
 
 static void fileselector_callback(filebrowser_t *fb) {
@@ -1798,7 +1903,7 @@ static void fileselector_callback(filebrowser_t *fb) {
   char *cur_dir = filebrowser_get_current_dir(fb);
   
   if(cur_dir && strlen(cur_dir)) {
-    sprintf(gGui->curdir, "%s", cur_dir);
+    snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", cur_dir);
     config_update_string("input.file_origin_path", gGui->curdir);
   }
   
@@ -1820,10 +1925,12 @@ static void fileselector_callback(filebrowser_t *fb) {
       }
       
       gGui->playlist.cur = first;
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
       gui_play(NULL, NULL);
     }
   }
+
+  load_stream = NULL;
 }
 
 static void fileselector_all_callback(filebrowser_t *fb) {
@@ -1831,7 +1938,7 @@ static void fileselector_all_callback(filebrowser_t *fb) {
   char  *path = filebrowser_get_current_dir(fb);
   
   if(path && strlen(path)) {
-    sprintf(gGui->curdir, "%s", path);
+    snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", path);
     config_update_string("input.file_origin_path", gGui->curdir);
   }
   
@@ -1844,13 +1951,13 @@ static void fileselector_all_callback(filebrowser_t *fb) {
       int  first = gGui->playlist.num;
 
       if(strcasecmp(path, "/"))
-	sprintf(pathname, "%s/", path);
+	snprintf(pathname, sizeof(pathname), "%s/", path);
       else
-	sprintf(pathname, "%s", path);
+	snprintf(pathname, sizeof(pathname), "%s", path);
       
       gGui->playlist.control |= PLAYLIST_CONTROL_IGNORE;
       while(files[i]) {
-	sprintf(fullfilename, "%s%s", pathname, files[i++]);
+	snprintf(fullfilename, sizeof(fullfilename), "%s%s", pathname, files[i++]);
 	gui_dndcallback(fullfilename);
       }
       gGui->playlist.control &= !PLAYLIST_CONTROL_IGNORE;
@@ -1864,7 +1971,7 @@ static void fileselector_all_callback(filebrowser_t *fb) {
 	  gGui->ignore_next = 0;
 	}
 	gGui->playlist.cur = first;
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	gui_set_current_mmk(mediamark_get_current_mmk());
 	gui_play(NULL, NULL);
       }
     }
@@ -1875,20 +1982,26 @@ static void fileselector_all_callback(filebrowser_t *fb) {
     
     free(files);
   }
+
+  load_stream = NULL;
 }
 
 void gui_file_selector(void) {
   filebrowser_callback_button_t  cbb[3];
-  
-  cbb[0].label = _("Select");
-  cbb[0].callback = fileselector_callback;
-  cbb[0].need_a_file = 0;
-  cbb[1].label = _("Select all");
-  cbb[1].callback = fileselector_all_callback;
-  cbb[1].need_a_file = 0;
-  cbb[2].callback = fileselector_cancel_callback;
-  cbb[2].need_a_file = 0;
-  (void *) create_filebrowser(_("Stream(s) loading"), gGui->curdir, &cbb[0], &cbb[1], &cbb[2]);
+
+  if(load_stream)
+    filebrowser_raise_window(load_stream);
+  else {
+    cbb[0].label = _("Select");
+    cbb[0].callback = fileselector_callback;
+    cbb[0].need_a_file = 0;
+    cbb[1].label = _("Select all");
+    cbb[1].callback = fileselector_all_callback;
+    cbb[1].need_a_file = 0;
+    cbb[2].callback = fileselector_cancel_callback;
+    cbb[2].need_a_file = 0;
+    load_stream = create_filebrowser(_("Stream(s) loading"), gGui->curdir, hidden_file_cb, &cbb[0], &cbb[1], &cbb[2]);
+  }
 }
 
 static void subselector_callback(filebrowser_t *fb) {
@@ -1896,15 +2009,15 @@ static void subselector_callback(filebrowser_t *fb) {
 
   if((file = filebrowser_get_full_filename(fb)) != NULL) {
     if(file) {
-      mediamark_t *mmk = mediamark_clone_mmk((mediamark_t *) mediamark_get_current_mmk());
+      mediamark_t *mmk = mediamark_clone_mmk(mediamark_get_current_mmk());
       
       if(mmk) {
 	mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur], mmk->mrl, mmk->ident, 
 				file, mmk->start, mmk->end, mmk->av_offset, mmk->spu_offset);
 	mediamark_free_mmk(&mmk);
 
-	mmk = (mediamark_t *) mediamark_get_current_mmk();
-	gui_set_current_mrl(mmk);
+	mmk = mediamark_get_current_mmk();
+	gui_set_current_mmk(mmk);
 	
 	playlist_mrlident_toggle();
 	
@@ -1925,41 +2038,49 @@ static void subselector_callback(filebrowser_t *fb) {
     }
     free(file);
   }
+
+  load_sub = NULL;
 }
 
 void gui_select_sub(void) {
   
   if(gGui->playlist.num) {
-    filebrowser_callback_button_t  cbb;
-    const mediamark_t *mmk;
-    
-    mmk = mediamark_get_current_mmk();
-
-    if(mmk) {
-      char *path, *open_path;
+    if(load_sub)
+      filebrowser_raise_window(load_sub);
+    else {
+      filebrowser_callback_button_t  cbb[2];
+      mediamark_t *mmk;
       
-      cbb.label = _("Select");
-      cbb.callback = subselector_callback;
-      cbb.need_a_file = 1;
+      mmk = mediamark_get_current_mmk();
       
-      path = mmk->sub ? mmk->sub : mmk->mrl;
-      
-      if(mrl_look_like_file(path)) {
-	char *p;
+      if(mmk) {
+	char *path, *open_path;
 	
-	xine_strdupa(open_path, path);
+	cbb[0].label = _("Select");
+	cbb[0].callback = subselector_callback;
+	cbb[0].need_a_file = 1;
+	cbb[1].callback = fileselector_cancel_callback;
+	cbb[1].need_a_file = 0;
+    	
+	path = mmk->sub ? mmk->sub : mmk->mrl;
 	
-	if(!strncasecmp(path, "file:", 5))
-	  path += 5;
-
-	p = strrchr(open_path, '/');
-	if (p && strlen(p))
-	  *p = '\0';
+	if(mrl_look_like_file(path)) {
+	  char *p;
+	  
+	  xine_strdupa(open_path, path);
+	  
+	  if(!strncasecmp(path, "file:", 5))
+	    path += 5;
+	  
+	  p = strrchr(open_path, '/');
+	  if (p && strlen(p))
+	    *p = '\0';
+	}
+	else
+	  open_path = gGui->curdir;
+	
+	load_sub = create_filebrowser(_("Pick a subtitle file"), open_path, hidden_file_cb, &cbb[0], NULL, &cbb[1]);
       }
-      else
-	open_path = gGui->curdir;
-      
-      (void *) create_filebrowser(_("Pick a subtitle file"), open_path, &cbb, NULL, NULL);
     }
   }
 }
@@ -1971,7 +2092,7 @@ void visual_anim_init(void) {
   char buffer[4096];
 
   memset(&buffer, 0, sizeof(buffer));
-  sprintf(buffer, "%s/%s", XINE_VISDIR, "default.avi");
+  snprintf(buffer, sizeof(buffer), "%s/%s", XINE_VISDIR, "default.avi");
   
   gGui->visual_anim.mrls = (char **) xine_xmalloc(sizeof(char *) * 3);
   gGui->visual_anim.num_mrls = 0;
