@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.h,v 1.56 2003/11/07 21:33:56 f1rmb Exp $
+ * $Id: actions.h,v 1.59 2004/04/26 21:31:44 f1rmb Exp $
  *
  * all the various actions for the gui (play, stop, open, pause...)
  *
@@ -32,6 +32,8 @@
 #define GUI_PREV     2
 #define GUI_RESET    3
 
+void reparent_window(Window window);
+void reparent_all_windows(void);
 void raise_window(Window window, int visible, int running);
 void toggle_window(Window window, xitk_widget_list_t *widget_list, int *visible, int running);
 
@@ -39,8 +41,9 @@ int gui_xine_get_pos_length(xine_stream_t *stream, int *pos, int *time, int *len
 void try_to_set_input_focus(Window window);
 void gui_display_logo(void);
 int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time, int update_mmk);
+int gui_open_and_play_alternates(mediamark_t *mmk, const char *sub);
 int gui_xine_open_and_play(char *mrl, char *sub, int start_pos, 
-			   int start_time, int av_offset, int spu_offset);
+			   int start_time, int av_offset, int spu_offset, int report_error);
 void gui_exit (xitk_widget_t *w, void *data);
 void gui_play (xitk_widget_t *w, void *data);
 void gui_stop (xitk_widget_t *w, void *data);
@@ -66,7 +69,7 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value);
 void gui_nextprev(xitk_widget_t *w, void *data);
 void gui_playlist_show(xitk_widget_t *w, void *data);
 void gui_mrlbrowser_show(xitk_widget_t *w, void *data);
-void gui_set_current_mrl(mediamark_t *mmk);
+void gui_set_current_mmk(mediamark_t *mmk);
 void gui_control_show(xitk_widget_t *w, void *data);
 void gui_mrl_browser_show(xitk_widget_t *w, void *data);
 void gui_setup_show(xitk_widget_t *w, void *data);
