@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.c,v 1.93 2003/10/29 12:32:15 f1rmb Exp $
+ * $Id: control.c,v 1.104 2004/07/02 09:33:37 f1rmb Exp $
  *
  *
  */
@@ -355,6 +355,8 @@ void control_exit(xitk_widget_t *w, void *data) {
 
     free(control);
     control = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -404,15 +406,19 @@ void control_toggle_visibility (xitk_widget_t *w, void *data) {
 /*
  * Handle X events here.
  */
-void control_handle_event(XEvent *event, void *data) {
+static void control_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
     
   case KeyPress:
-    gui_handle_event(event, data);
-    break;      
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      control_exit(NULL, NULL);
+    else
+      gui_handle_event(event, data);
+    break;
+    
   }
-
+  
 }
 
 /*
@@ -458,7 +464,7 @@ void control_change_skins(void) {
     control->bg_image = new_img;
     
     XLockDisplay(gGui->display);
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, control->window, gGui->video_window);
     
     Imlib_destroy_image(gGui->imlib_data, old_img);
@@ -466,6 +472,9 @@ void control_change_skins(void) {
     
     XUnlockDisplay(gGui->display);
 
+    if(control_is_visible())
+      control_raise_window();
+
     xitk_skin_unlock(gGui->skin_config);
     
     xitk_change_skins_widget_list(control->widget_list, gGui->skin_config);
@@ -487,10 +496,15 @@ void control_deinit(void) {
   if(control) {
     if(control_is_visible())
       control_toggle_visibility(NULL, NULL);
-    xitk_unregister_event_handler(&control->widget_key);
+    control_exit(NULL, NULL);
   }
 }
 
+void control_reparent(void) {
+  if(control)
+    reparent_window(control->window);
+}
+
 /*
  * Create control panel window
  */
@@ -573,11 +587,16 @@ void control_panel(void) {
 				   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				   &attr);
   
-  XSetStandardProperties(gGui->display, control->window, title, title,
- 			 None, NULL, 0, &hint);
+  XmbSetWMProperties(gGui->display, control->window, title, title, NULL, 0, 
+                     &hint, NULL, NULL);
   
   XSelectInput(gGui->display, control->window, INPUT_MOTION | KeymapStateMask);
   XUnlockDisplay(gGui->display);
+
+  if(!video_window_is_visible())
+    xitk_set_wm_window_type(control->window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(control->window, WINDOW_TYPE_NORMAL);
   
   if(is_layer_above())
     xitk_set_layer_above(control->window);
@@ -601,13 +620,16 @@ void control_panel(void) {
     xclasshint->res_name = title;
     xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, control->window, xclasshint);
+    XFree(xclasshint);
+    
   }
 
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
     wm_hint->input         = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags         = InputHint | StateHint;
+    wm_hint->icon_pixmap   = gGui->icon;
+    wm_hint->flags         = InputHint | StateHint | IconPixmapHint;
     XSetWMHints(gGui->display, control->window, wm_hint);
     XFree(wm_hint);
   }
@@ -762,7 +784,7 @@ void control_panel(void) {
 			    xitk_browser_create(control->widget_list, gGui->skin_config, &br)));
 
   xitk_browser_update_list(control->skinlist, 
-			   control->skins, control->skins_num, 0);
+			   control->skins, NULL, control->skins_num, 0);
 
 
   lb.skin_element_name = "CtlDismiss";
@@ -788,6 +810,6 @@ void control_panel(void) {
   control->visible = 1;
   control->running = 1;
   control_raise_window();
-
+  
   try_to_set_input_focus(control->window);
 }
