@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.c,v 1.243 2003/12/18 00:30:12 f1rmb Exp $
+ * $Id: event.c,v 1.256 2004/07/02 21:41:02 f1rmb Exp $
  *
  * gui initialization and top-level event handling stuff
  *
@@ -55,7 +55,7 @@ extern _panel_t        *panel;
 static pid_t            xine_pid;
 
 /* Icon data */
-static unsigned char xine_bits[] = {
+static unsigned char icon_datas[] = {
    0x11, 0x00, 0x00, 0x00, 0x88, 0x11, 0x00, 0x00, 0x00, 0x88, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xff, 0xff, 0xff,
    0x8f, 0xf1, 0xff, 0xff, 0xff, 0x8f, 0x1f, 0x00, 0x00, 0x00, 0xf8, 0x1f,
@@ -101,6 +101,26 @@ static char *mixer_control_method[] = {
   NULL
 };
 
+static char *shortcut_style[] = {
+  "Windows style",
+  "Emacs style",
+  NULL
+};
+
+int hidden_file_cb(int action, int value) {
+  xine_cfg_entry_t  cfg_entry;
+  int               retval = 0;
+  
+  if(xine_config_lookup_entry(gGui->xine, "input.file_hidden_files", &cfg_entry)) {
+    if(action)
+      config_update_bool("input.file_hidden_files", value);
+    else
+      retval = cfg_entry.num_value;
+  }
+
+  return retval;
+}
+
 void dummy_config_cb(void *data, xine_cfg_entry_t *cfg) {
   /* It exist to avoid "restart" window message in setup window */
 }
@@ -199,6 +219,9 @@ static void audio_mixer_method_cb(void *data, xine_cfg_entry_t *cfg) {
   xitk_slider_set_max(panel->mixer.slider, max);
   xitk_slider_set_pos(panel->mixer.slider, vol);
 }
+static void shortcut_style_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->shortcut_style = cfg->num_value;
+}
 
 int wm_not_ewmh_only(void) {
   int wm_type = xitk_get_wm_type();
@@ -272,6 +295,31 @@ static void gui_signal_handler (int sig, void *data) {
 
 }
 
+
+/*
+ * convert pts value to string
+ */
+static char *pts2str(int64_t pts) {
+  static char  buf[40];
+  int64_t      min;
+  double       sec;
+  int          ds;
+  int          sign;
+
+  if((sign = (pts < 0)) != 0)
+    pts = -pts;
+
+  min = pts / (90000 * 60);
+  sec = (double)pts / 90000 - 60 * min;
+  ds  = sec / 10;
+  sec -= 10 * ds;
+
+  snprintf(buf, sizeof(buf), "%s%02" PRIi64 ":%d%.2f (%" PRIi64 " pts)", sign ? "-" : "", min, ds, sec, pts);
+
+  return buf;
+}
+
+
 /*
  *
  */
@@ -563,7 +611,7 @@ void gui_execute_action_id(action_id_t action) {
       if (gGui->playlist.cur < gGui->playlist.num)
         gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
       xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
-      osd_display_info(_("A/V offset: %d"), av_offset);
+      osd_display_info(_("A/V offset: %s"), pts2str(av_offset));
     }
     break;
     
@@ -575,7 +623,7 @@ void gui_execute_action_id(action_id_t action) {
       if (gGui->playlist.cur < gGui->playlist.num)
         gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
       xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
-      osd_display_info(_("A/V offset: %d"), av_offset);
+      osd_display_info(_("A/V offset: %s"), pts2str(av_offset));
     }
     break;
 
@@ -589,7 +637,8 @@ void gui_execute_action_id(action_id_t action) {
       int spu_offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) + 3600;
       
       xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, spu_offset);
-      osd_display_info(_("SPU Offset: %d"), spu_offset);
+      
+      osd_display_info(_("SPU Offset: %s"), pts2str(spu_offset));
     }
     break;
 
@@ -598,10 +647,16 @@ void gui_execute_action_id(action_id_t action) {
       int spu_offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) - 3600;
       
       xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, spu_offset);
-      osd_display_info(_("SPU Offset: %d"), spu_offset);
+      
+      osd_display_info(_("SPU Offset: %s"), pts2str(spu_offset));
     }
     break;
     
+  case ACTID_SV_SYNC_RESET:
+    xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, 0);
+    osd_display_info(_("SPU Offset: reset."));
+    break;
+
   case ACTID_SPEED_FAST:
     gui_change_speed_playback(NULL, (void*)GUI_PREV);
     break;
@@ -669,10 +724,10 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_GRAB_POINTER:
     if(!gGui->cursor_grabbed) {
       if(!panel_is_visible()) {
-	XLockDisplay(gGui->display);
-	XGrabPointer(gGui->display, gGui->video_window, 1, None, 
+	XLockDisplay(gGui->video_display);
+	XGrabPointer(gGui->video_display, gGui->video_window, 1, None, 
 		     GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
-	XUnlockDisplay(gGui->display);
+	XUnlockDisplay(gGui->video_display);
       }
       
       gGui->cursor_grabbed = 1;
@@ -922,21 +977,27 @@ void gui_playlist_start_next(void) {
   }
   
   switch(gGui->playlist.loop) {
-    
+
   case PLAYLIST_LOOP_NO_LOOP:
   case PLAYLIST_LOOP_LOOP:
     gGui->playlist.cur++;
     
     if(gGui->playlist.cur < gGui->playlist.num) {
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
+      
       if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
-
-	if((gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) && 
-	   mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
-	  gui_exit(NULL, NULL);
-
-	gui_display_logo();
+                                 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 
+                                 !mediamark_have_alternates(&(gGui->mmk)))) {
+      
+        if(!mediamark_have_alternates(&(gGui->mmk)) || 
+	   !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+        
+          if((gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) && 
+              mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+            gui_exit(NULL, NULL);
+          
+          gui_display_logo();
+        }
       }
     }
     else {
@@ -951,23 +1012,36 @@ void gui_playlist_start_next(void) {
       }
       else if(gGui->playlist.loop == PLAYLIST_LOOP_LOOP) {
 	gGui->playlist.cur = 0;
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-	if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0,
-				   gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
-	  if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
-	    gui_exit(NULL, NULL);
-	  gui_display_logo();
-	}
-      }
+	gui_set_current_mmk(mediamark_get_current_mmk());
 
+        if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0,
+                                   gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 
+                                   !mediamark_have_alternates(&(gGui->mmk)))) {
+          
+          if(!mediamark_have_alternates(&(gGui->mmk)) ||
+             !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+              gui_display_logo();
+            if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+              gui_exit(NULL, NULL);
+            
+            gui_display_logo();
+          }
+        }
+      }
     }
     break;
     
   case PLAYLIST_LOOP_REPEAT:
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+    gui_set_current_mmk(mediamark_get_current_mmk());
     if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-			       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
-      gui_display_logo();
+                               gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+                               !mediamark_have_alternates(&(gGui->mmk)))) {
+      
+      if(mediamark_have_alternates(&(gGui->mmk))) {
+        if(!gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub))
+          gui_display_logo();
+      }
+    }
     break;
 
   case PLAYLIST_LOOP_SHUFFLE:
@@ -977,10 +1051,20 @@ void gui_playlist_start_next(void) {
     __shuffle_restart:
       gGui->playlist.cur = mediamark_get_shuffle_next();
       
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
       if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
-	gui_display_logo();
+                                 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+                                 !mediamark_have_alternates(&(gGui->mmk)))) {
+        
+        if(!mediamark_have_alternates(&(gGui->mmk)) ||
+           !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+        
+          if(!mediamark_all_played())
+            goto __shuffle_restart;
+          else
+            gui_display_logo();
+        }
+      }
     }
     else {
       mediamark_reset_played_state();
@@ -997,7 +1081,7 @@ void gui_playlist_start_next(void) {
   }
   
   if(is_playback_widgets_enabled() && (!gGui->playlist.num)) {
-    gui_set_current_mrl(NULL);
+    gui_set_current_mmk(NULL);
     enable_playback_controls(0);
   }
 
@@ -1100,23 +1184,26 @@ void gui_deinit(void) {
 void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attribute) {
   int    i;
   char  *server;
+  char  *video_display_name;
 
   /*
    * init playlist
    */
   for (i = 0; i < nfiles; i++) {
     char *sub = NULL;
-    
+    char *file = atoa(filenames[i]);
+
     /* grab recursively all files from dir */
-    if(is_a_dir(filenames[i])) {
-      if(filenames[i][strlen(filenames[i]) - 1] == '/')
-	filenames[i][strlen(filenames[i]) - 1] = '\0'; 
+    if(is_a_dir(file)) {
       
-      mediamark_collect_from_directory(filenames[i]);
+      if(file[strlen(file) - 1] == '/')
+	file[strlen(file) - 1] = '\0'; 
+      
+      mediamark_collect_from_directory(file);
     }
     else {
-      if((sub = (char *)get_last_double_semicolon(filenames[i])) != NULL) {
-	if(is_ipv6_last_double_semicolon(filenames[i]))
+      if((sub = (char *)get_last_double_semicolon(file)) != NULL) {
+	if(is_ipv6_last_double_semicolon(file))
 	  sub = NULL;
 	else {
 	  *sub = 0;
@@ -1124,8 +1211,7 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 	}
       }
       
-      mediamark_add_entry((const char *)filenames[i], 
-			  (const char *)filenames[i], sub, 0, -1, 0, 0);
+      mediamark_append_entry((const char *)file, (const char *)file, sub, 0, -1, 0, 0);
     }
   }
   
@@ -1152,6 +1238,31 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     exit(1);
   }
 
+  video_display_name = 
+    (char *)xine_config_register_string (gGui->xine, "gui.video_display", 
+					 "",
+					 _("Name of video display"),
+					 _("Use this setting to configure to which "
+                                           "display xine will show videos. When left blank, "
+                                           "the main display will be used. Setting this "                  "option to something like ':0.1' or ':1' makes "
+                                           "possible to display video and control on different "
+                                           "screens (very useful for TV presentations)."),
+					 CONFIG_LEVEL_ADV,
+					 NULL,
+					 CONFIG_NO_DATA);
+  
+  gGui->video_display = NULL;
+  if ( strlen(video_display_name) ) {
+  
+    if ((gGui->video_display = XOpenDisplay(video_display_name)) == NULL )
+      fprintf(stderr, _("Cannot open display '%s' for video. Falling back to primary display.\n"),
+              video_display_name);
+  }
+  
+  if ( gGui->video_display == NULL ) {
+    gGui->video_display = gGui->display;
+  }
+
   if (xine_config_register_bool (gGui->xine, "gui.xsynchronize", 
 				 0,
 				 _("Synchronized X protocol (debug)"), 
@@ -1163,7 +1274,7 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     XLockDisplay(gGui->display);
     XSynchronize (gGui->display, True);
     XUnlockDisplay(gGui->display);
-    fprintf (stderr, _("Warning! Synchronized X activated - this is way slow...\n"));
+    fprintf (stderr, _("Warning! Synchronized X activated - this is very slow...\n"));
   }
 
   gGui->layer_above = 
@@ -1352,12 +1463,25 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 			      audio_mixer_method_cb,
 			      CONFIG_NO_DATA);
 
+  gGui->shortcut_style = 
+    xine_config_register_enum(gGui->xine, "gui.shortcut_style", 
+			      0, shortcut_style,
+			      _("Menu shortcut style"),
+			      _("Shortcut representation in menu, 'Ctrl,Alt' or 'C,M'."), 
+			      CONFIG_LEVEL_ADV,
+			      shortcut_style_cb,
+			      CONFIG_NO_DATA);
+
   gGui->numeric.set = 0;
   gGui->numeric.arg = 0;
   
   XLockDisplay (gGui->display);
 
   gGui->screen = DefaultScreen(gGui->display);
+  
+  XLockDisplay (gGui->video_display);
+  gGui->video_screen = DefaultScreen(gGui->video_display);
+  XUnlockDisplay (gGui->video_display);
 
   /* Some infos */
   if(gGui->verbosity) {
@@ -1397,7 +1521,7 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    * when done before the video_window_init(). */
   gGui->kbindings = kbindings_init_kbinding();
 
-  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+  gui_set_current_mmk(mediamark_get_current_mmk());
 
   panel_init ();
 }
@@ -1446,7 +1570,7 @@ void gui_init_imlib (Visual *vis) {
   
   gGui->icon = XCreateBitmapFromData (gGui->display, 
 				      gGui->imlib_data->x.root,
-				      xine_bits, 40, 40);
+				      icon_datas, 40, 40);
 }
 
 /*
@@ -1532,11 +1656,11 @@ void gui_run(char **session_opts) {
 	  
 	  if(autoplay_mrls) {
 	    for (j = 0; j < num_mrls; j++)
-	      mediamark_add_entry((const char *)autoplay_mrls[j],
-				  (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
+	      mediamark_append_entry((const char *)autoplay_mrls[j],
+				     (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
 	   
 	    gGui->playlist.cur = 0;
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	  }
 	}    
       }
@@ -1569,7 +1693,9 @@ void gui_run(char **session_opts) {
 						 gui_dndcallback, 
 						 NULL, NULL);
   
+#ifdef HAVE_READLINE
   start_remote_server();
+#endif
   init_session();
 
   if(gGui->tvout) {
@@ -1597,7 +1723,7 @@ void gui_run(char **session_opts) {
       if(video_window_is_visible())
 	video_window_set_visibility(!(video_window_is_visible()));
       else
-	xine_gui_send_vo_data(gGui->stream,
+	xine_port_send_gui_data(gGui->vo_port,
 			      XINE_GUI_SEND_VIDEOWIN_VISIBLE,
 			      (int *)0);
       
@@ -1616,7 +1742,7 @@ void gui_run(char **session_opts) {
     /* User load a playlist on startup */
     if(actions_on_start(gGui->actions_on_start, ACTID_PLAYLIST)) {
       mediamark_load_mediamarks(gGui->playlist.on_start);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
       if((!is_playback_widgets_enabled()) && gGui->playlist.num)
 	enable_playback_controls(1);
     }
@@ -1647,10 +1773,17 @@ void gui_run(char **session_opts) {
   if(gGui->playlist.mmk && gGui->playlist.num) {
     char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
-    sprintf(buffer, "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
+    snprintf(buffer, sizeof(buffer), "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
     mediamark_save_mediamarks(buffer);
   }
 
+  { /* Give focus to the ROOT window, to get C-Tab working */
+    Window rootwindow, wparent;
+    
+    if((rootwindow = xitk_get_desktop_root_window(gGui->video_display, gGui->video_screen, &wparent)) != None)
+      try_to_set_input_focus(rootwindow);
+  }
+
   gGui->running = 0;
   deinit_session();
 
@@ -1658,4 +1791,6 @@ void gui_run(char **session_opts) {
   kbindings_free_kbinding(&gGui->kbindings);
 
   XCloseDisplay(gGui->display);
+  if( gGui->video_display != gGui->display )
+    XCloseDisplay(gGui->video_display);
 }
