@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event_sender.c,v 1.21 2003/10/29 12:32:15 f1rmb Exp $
+ * $Id: event_sender.c,v 1.26 2004/07/02 09:33:37 f1rmb Exp $
  *
  *
  */
@@ -291,7 +291,7 @@ void event_sender_update_menu_buttons(void) {
   }
 }
 
-void event_sender_exit(xitk_widget_t *w, void *data) {
+static void event_sender_exit(xitk_widget_t *w, void *data) {
 
   if(eventer) {
     window_info_t wi;
@@ -321,6 +321,8 @@ void event_sender_exit(xitk_widget_t *w, void *data) {
     
     free(eventer);
     eventer = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -368,10 +370,15 @@ void event_sender_move(int x, int y) {
   }
 }
 
-static void event_sender_end(xitk_widget_t *w, void *data) {
+void event_sender_end(void) {
   event_sender_exit(NULL, NULL);
 }
 
+void event_sender_reparent(void) {
+  if(eventer)
+    reparent_window((xitk_window_get_window(eventer->xwin)));
+}
+
 void event_sender_panel(void) {
   GC                         gc;
   xitk_labelbutton_widget_t  lb;
@@ -403,10 +410,12 @@ void event_sender_panel(void) {
   }
 
   /* Create window */
-  eventer->xwin = xitk_window_create_simple_window(gGui->imlib_data, eventer->x, eventer->y,
-						   WINDOW_WIDTH, WINDOW_HEIGHT);
-  
+  eventer->xwin = xitk_window_create_simple_window(gGui->imlib_data, eventer->x, eventer->y, WINDOW_WIDTH, WINDOW_HEIGHT);
+
+  set_window_states_start((xitk_window_get_window(eventer->xwin)));
+
   XLockDisplay (gGui->display);
+  XStoreName(gGui->display, (xitk_window_get_window(eventer->xwin)), _("xine Event Sender"));
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(eventer->xwin)), None, None);
   XUnlockDisplay (gGui->display);
@@ -516,7 +525,7 @@ void event_sender_panel(void) {
     char number[2];
     
     memset(&number, 0, sizeof(number));
-    sprintf(number, "%d", i);
+    snprintf(number, sizeof(number), "%d", i);
 
     lb.button_type       = CLICK_BUTTON;
     lb.label             = number;
@@ -553,7 +562,7 @@ void event_sender_panel(void) {
     i = 10;
     
     memset(&number, 0, sizeof(number));
-    sprintf(number, "+ %d", i);
+    snprintf(number, sizeof(number), "+ %d", i);
     
     lb.button_type       = CLICK_BUTTON;
     lb.label             = number;
@@ -659,7 +668,7 @@ void event_sender_panel(void) {
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = ALIGN_CENTER;
-  lb.callback          = event_sender_end; 
+  lb.callback          = event_sender_exit; 
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
