@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: file_browser.c,v 1.44 2003/11/09 13:52:31 f1rmb Exp $
+ * $Id: file_browser.c,v 1.55 2004/07/02 21:41:02 f1rmb Exp $
  *
  */
 
@@ -85,7 +85,7 @@ static filebrowser_filter_t __fb_filters[] = {
  /* subtitles */
   { NULL, ".sub .srt .asc .smi .ssa "                                              },
   /* playlists */
-  { NULL, ".pls .m3u .sfv .tox .asx .smil "                                        },
+  { NULL, ".pls .m3u .sfv .tox .asx .smil .smi .xml .fxd "                         },
   { "*.4xm", ".4xm "                                                               },
   { "*.ac3", ".ac3 "                                                               },
   { "*.aif, *.aiff,", ".aif .aiff "                                                },
@@ -182,6 +182,8 @@ struct filebrowser_s {
   
   xitk_widget_t                  *close;
 
+  hidden_file_toggle_t            hidden_cb;
+
   filebrowser_callback_button_t   cbb[3];
   xitk_widget_t                  *cb_buttons[2];
   
@@ -284,6 +286,15 @@ static void fne_cancel_cb(xitk_widget_t *w, void *data) {
   fb_reactivate(fne->fb);
   fne_destroy(fne);
 }
+static void fne_handle_event(XEvent *event, void *data) {
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      fne_cancel_cb(NULL, data);
+    break;
+  }
+}
 static void fb_create_input_window(char *title, char *text,
 				   xitk_string_callback_t cb, filebrowser_t *fb) {
   filename_editor_t          *fne;
@@ -306,6 +317,10 @@ static void fb_create_input_window(char *title, char *text,
 
   fne->xwin = xitk_window_create_dialog_window(gGui->imlib_data, title, x, y, width, height);
 
+  xitk_set_wm_window_type((xitk_window_get_window(fne->xwin)), WINDOW_TYPE_NORMAL);
+  change_class_name((xitk_window_get_window(fne->xwin)));
+  change_icon((xitk_window_get_window(fne->xwin)));
+
   XLockDisplay(gGui->display);
   gc = XCreateGC(gGui->display, (xitk_window_get_window(fne->xwin)), None, None);
   XUnlockDisplay(gGui->display);
@@ -371,21 +386,21 @@ static void fb_create_input_window(char *title, char *text,
 
   {
     char buffer[256];
-    sprintf(buffer, "filenameed%d", (unsigned int) time(NULL));
+    snprintf(buffer, sizeof(buffer), "filenameed%d", (unsigned int) time(NULL));
     
     fne->widget_key = xitk_register_event_handler(buffer, 
 						  (xitk_window_get_window(fne->xwin)),
-						  NULL,
+						  fne_handle_event,
 						  NULL,
 						  NULL,
 						  fne->widget_list,
-						  NULL);
+						  (void *)fne);
   }
   
   XLockDisplay(gGui->display);
   XRaiseWindow(gGui->display, xitk_window_get_window(fne->xwin));
   XMapWindow(gGui->display, xitk_window_get_window(fne->xwin));
-  if(!gGui->use_root_window)
+  if(!gGui->use_root_window && gGui->video_display == gGui->display)
     XSetTransientForHint(gGui->display, 
 			 xitk_window_get_window(fne->xwin), gGui->video_window);
   XUnlockDisplay(gGui->display);
@@ -428,15 +443,15 @@ static void fb_update_origin(filebrowser_t *fb) {
   
   if(strcasecmp(fb->current_dir, "/")) {
     if(strlen(fb->filename))
-      sprintf(buf, "%s/%s", fb->current_dir, fb->filename);
+      snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->filename);
     else
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   }
   else {
     if(strlen(fb->filename))
-      sprintf(buf, "%s%s", fb->current_dir, fb->filename);
+      snprintf(buf, sizeof(buf), "%s%s", fb->current_dir, fb->filename);
     else
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   }
   
   xitk_inputtext_change_text(fb->origin, buf); 
@@ -468,15 +483,15 @@ static void fb_extract_path_and_file(filebrowser_t *fb, char *filepathname) {
     
     if(filename && (strlen(filename) > 1)) {
       *filename++ = '\0';
-      sprintf(fb->filename, "%s", filename);
+      snprintf(fb->filename, sizeof(fb->filename), "%s", filename);
       
       if(is_a_dir(_filepathname))
-	sprintf(fb->current_dir, "%s", _filepathname);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", _filepathname);
 
     }
     else {
       if(is_a_dir((char *)_filepathname)) {
-	sprintf(fb->current_dir, "%s", _filepathname);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", _filepathname);
 	memset(&fb->filename, 0, sizeof(fb->filename));
       }
     }
@@ -589,7 +604,7 @@ static void sort_files(filebrowser_t *fb) {
     fb->files[i] = NULL;
   }
   
-  xitk_browser_update_list(fb->files_browser, (const char *const *)fb->files, fb->files_num, 0);
+  xitk_browser_update_list(fb->files_browser, (const char *const *)fb->files, NULL, fb->files_num, 0);
 }
 
 static void sort_directories(filebrowser_t *fb) {
@@ -624,7 +639,7 @@ static void sort_directories(filebrowser_t *fb) {
   }
 
   xitk_browser_update_list(fb->directories_browser, 
-			   (const char *const *)fb->directories, fb->directories_num, 0);
+			   (const char *const *)fb->directories, NULL, fb->directories_num, 0);
 }
 
 static void fb_getdir(filebrowser_t *fb) {
@@ -669,7 +684,7 @@ static void fb_getdir(filebrowser_t *fb) {
   while((pdirent = readdir(pdir)) != NULL) {
     
     memset(&fullfilename, 0, sizeof(fullfilename));
-    sprintf(fullfilename, "%s/%s", fb->current_dir, pdirent->d_name);
+    snprintf(fullfilename, sizeof(fullfilename), "%s/%s", fb->current_dir, pdirent->d_name);
     
     if(is_a_dir(fullfilename)) {
       
@@ -730,7 +745,7 @@ static void fb_select(xitk_widget_t *w, void *data, int selected) {
   filebrowser_t *fb = (filebrowser_t *) data;
   
   if(w == fb->files_browser) {
-    sprintf(fb->filename, "%s", fb->norm_files[selected].name);
+    snprintf(fb->filename, sizeof(fb->filename), "%s", fb->norm_files[selected].name);
     fb_update_origin(fb);
   }
 }
@@ -767,7 +782,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
       char *p;
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
       if(strlen(buf) > 1) { /* not '/' directory */
 	
 	p = &buf[strlen(buf)-1];
@@ -780,7 +795,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
 	if((strlen(buf) > 1) && *p == '/') 
 	  *p = '\0';
 	
-	sprintf(fb->current_dir, "%s", buf);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", buf);
       }
     }
     else {
@@ -788,15 +803,15 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
       /* not '/' directory */
       if(strcasecmp(fb->current_dir, "/")) {
 	memset(&buf, 0, sizeof(buf));
-	sprintf(buf, "%s/%s", fb->current_dir, fb->dir_files[selected].name);
+	snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->dir_files[selected].name);
       }
       else {
 	memset(&buf, 0, sizeof(buf));
-	sprintf(buf, "/%s", fb->dir_files[selected].name);
+	snprintf(buf, sizeof(buf), "/%s", fb->dir_files[selected].name);
       }
       
       if(is_a_dir(buf))
-	sprintf(fb->current_dir, "%s", buf);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", buf);
 
     }
     
@@ -804,7 +819,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
     fb_getdir(fb);
   }
   else if(w == fb->files_browser) {
-    sprintf(fb->filename, "%s", fb->norm_files[selected].name);
+    snprintf(fb->filename, sizeof(fb->filename), "%s", fb->norm_files[selected].name);
     fb_callback_button_cb(fb->cb_buttons[0], (void *)data);
   }
 
@@ -816,7 +831,7 @@ static void fb_change_origin(xitk_widget_t *w, void *data, char *currenttext) {
   char          *p;
 
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s", currenttext);
+  snprintf(buf, sizeof(buf), "%s", currenttext);
 
   p = &buf[strlen(buf) - 1];
   while((strlen(buf) > 1) && (*p == '/'))
@@ -962,9 +977,9 @@ static void fb_delete_file_cb(xitk_widget_t *w, void *data, int button) {
       int sel = xitk_browser_get_current_selected(fb->files_browser);
 
       memset(&buf, 0, sizeof(buf));
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
       if(strlen(fb->current_dir) > 1)
-	sprintf(buf, "%s/", buf);
+	strcat(buf, "/");
       sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
       
       if((unlink(buf)) == -1)
@@ -985,9 +1000,9 @@ static void fb_delete_file(xitk_widget_t *w, void *data) {
   if((sel = xitk_browser_get_current_selected(fb->files_browser)) >= 0) {
     char buf[256 + XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
-    sprintf(buf, "Do you really want to delete the file: '%s", fb->current_dir);
+    snprintf(buf, sizeof(buf), _("Do you really want to delete the file: '%s"), fb->current_dir);
     if(strlen(fb->current_dir) > 1)
-      sprintf(buf, "%s/", buf);
+      strcat(buf, "/");
     sprintf(buf, "%s%s' ?.", buf, fb->norm_files[sel].name);
     
     fb_deactivate(fb);
@@ -1004,9 +1019,9 @@ static void fb_rename_file_cb(xitk_widget_t *w, void *data, char *newname) {
   int sel = xitk_browser_get_current_selected(fb->files_browser);
   
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s", fb->current_dir);
+  snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   if(strlen(fb->current_dir) > 1)
-    sprintf(buf, "%s/", buf);
+    strcat(buf, "/");
   sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
   
   if((rename(buf, newname)) == -1)
@@ -1023,9 +1038,9 @@ static void fb_rename_file(xitk_widget_t *w, void *data) {
     char buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
     memset(&buf, 0, sizeof(buf));
-    sprintf(buf, "%s", fb->current_dir);
+    snprintf(buf, sizeof(buf), "%s", fb->current_dir);
     if(strlen(fb->current_dir) > 1)
-      sprintf(buf, "%s/", buf);
+      strcat(buf, "/");
     sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
     
     fb_deactivate(fb);
@@ -1046,9 +1061,9 @@ static void fb_create_directory(xitk_widget_t *w, void *data) {
   char           buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s", fb->current_dir);
+  snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   if(strlen(fb->current_dir) > 1)
-    sprintf(buf, "%s/", buf);
+    strcat(buf, "/");
   
   fb_deactivate(fb);
   fb_create_input_window(_("Create a new directory"), buf, fb_create_directory_cb, fb);
@@ -1065,6 +1080,7 @@ static void fb_hidden_files(xitk_widget_t *w, void *data, int state) {
   filebrowser_t *fb = (filebrowser_t *) data;
   
   fb->show_hidden_files = state;
+  fb->hidden_cb(1, state);
   fb_getdir(fb);
 }
 static void fb_lbl_hidden_files(xitk_widget_t *w, void *data) {
@@ -1080,7 +1096,9 @@ static void fb_handle_events(XEvent *event, void *data) {
   switch(event->type) {
 
   case KeyPress:
-    {
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      _fb_exit(NULL, data);
+    else {
       int sel = xitk_browser_get_current_selected(fb->files_browser);
       
       if(sel >= 0)
@@ -1142,7 +1160,7 @@ char **filebrowser_get_all_files(filebrowser_t *fb) {
   return files;
 }
 
-filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
+filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden_file_toggle_t hidden_cb,
 				  filebrowser_callback_button_t *cbb1,
 				  filebrowser_callback_button_t *cbb2,
 				  filebrowser_callback_button_t *cbb_close) {
@@ -1195,15 +1213,20 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 					      (window_title) ? window_title : _("File Browser"), 
 					      x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  xitk_set_wm_window_type((xitk_window_get_window(fb->xwin)), WINDOW_TYPE_NORMAL);
+  change_class_name((xitk_window_get_window(fb->xwin)));
+  change_icon((xitk_window_get_window(fb->xwin)));
+
   fb->directories                = NULL;
   fb->directories_num            = 0;
   fb->files                      = NULL;
   fb->files_num                  = 0;
   fb->directories_sort_direction = DEFAULT_SORT;
   fb->files_sort_direction       = DEFAULT_SORT;
-  fb->show_hidden_files          = 1;
+  fb->hidden_cb                  = hidden_cb;
+  fb->show_hidden_files          = hidden_cb(0, 0);
 
-  sprintf(fb->current_dir, "%s", xine_get_homedir());
+  snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", xine_get_homedir());
   memset(&fb->filename, 0, sizeof(fb->filename));
   fb_extract_path_and_file(fb, filepathname);
 
@@ -1613,7 +1636,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 
   {
     char buffer[256];
-    sprintf(buffer, "filebrowser%d", (unsigned int) time(NULL));
+    snprintf(buffer, sizeof(buffer), "filebrowser%d", (unsigned int) time(NULL));
     fb->widget_key = xitk_register_event_handler(buffer, 
 						 (xitk_window_get_window(fb->xwin)),
 						 fb_handle_events,
@@ -1629,7 +1652,8 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
     
   fb_getdir(fb);
 
+  layer_above_video(xitk_window_get_window(fb->xwin));
   try_to_set_input_focus(xitk_window_get_window(fb->xwin));
-
+  
   return fb;
 }
