@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: kbindings.h,v 1.47 2003/11/18 22:15:44 f1rmb Exp $
+ * $Id: kbindings.h,v 1.53 2004/06/13 11:44:17 f1rmb Exp $
  *
  */
 #ifndef KBINDINGS_H
@@ -128,6 +128,7 @@ typedef enum {
   ACTID_SUBSELECT,
   ACTID_SV_SYNC_p,
   ACTID_SV_SYNC_m,
+  ACTID_SV_SYNC_RESET,
   ACTID_HUECONTROLp,
   ACTID_HUECONTROLm,
   ACTID_SATURATIONCONTROLp,
@@ -178,7 +179,7 @@ typedef enum {
   
 kbinding_t *kbindings_init_kbinding(void);
 void kbindings_save_kbinding(kbinding_t *);
-void kbindings_reset_kbinding(kbinding_t **);
+void kbindings_reset_kbinding(kbinding_t *);
 void kbindings_free_kbinding(kbinding_t **);
 void kbindings_display_current_bindings(kbinding_t *);
 void kbindings_display_default_lirc_bindings(void);
@@ -186,12 +187,14 @@ void kbindings_display_default_bindings(void);
 kbinding_entry_t *kbindings_lookup_action(kbinding_t *, const char *);
 void kbindings_handle_kbinding(kbinding_t *, XEvent *);
 action_id_t kbindings_get_action_id(kbinding_entry_t *);
+char *kbindings_get_shortcut(kbinding_t *, char *);
 
 void kbedit_window(void);
-void kbedit_exit(xitk_widget_t *, void *);
+void kbedit_end(void);
 int kbedit_is_visible(void);
 int kbedit_is_running(void);
 void kbedit_toggle_visibility(xitk_widget_t *, void *);
 void kbedit_raise_window(void);
+void kbedit_reparent(void);
 
 #endif
