@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.283 2004/04/15 19:07:42 f1rmb Exp $
+ * $Id: main.c,v 1.293 2004/07/02 21:41:03 f1rmb Exp $
  *
  * xine main for X11
  *
@@ -112,7 +112,7 @@ static const char *short_options = "?hHgfvn"
 #ifdef HAVE_XF86VIDMODE
  "F"
 #endif
- "u:a:V:A:p::s:RG:BN:P:l::S:ZD::r:c:ET:";
+ "u:a:V:A:p::s:RG:BN:P:l::S:ZD::r:c:ET:I";
 
 static struct option long_options[] = {
   {"help"           , no_argument      , 0, 'h'                      },
@@ -161,6 +161,7 @@ static struct option long_options[] = {
   {"tvout"          , required_argument, 0, 'T'                      },
   {"list-plugins"   , optional_argument, 0, OPTION_LIST_PLUGINS      },
   {"bug-report"     , optional_argument, 0, OPTION_BUG_REPORT        },
+  {"no-gui"         , no_argument      , 0, 'I'                      },
   {0                , no_argument      , 0, 0                        }
 };
 
@@ -353,7 +354,7 @@ static void xrm_parse(void) {
   memset(&wide_dbname, 0, sizeof(wide_dbname));
   memset(str_type, 0, sizeof(str_type));
 
-  sprintf(wide_dbname, "%s%s", "/usr/lib/X11/app-defaults/", classname);
+  snprintf(wide_dbname, sizeof(wide_dbname), "%s%s", "/usr/lib/X11/app-defaults/", classname);
   
   if((display = XOpenDisplay((getenv("DISPLAY")))) == NULL)
     return;
@@ -364,7 +365,7 @@ static void xrm_parse(void) {
   if(XResourceManagerString(display) != NULL)
     server_rmdb = XrmGetStringDatabase(XResourceManagerString(display));
   else {
-    sprintf(user_dbname, "%s%s", (xine_get_homedir()), "/.Xdefaults");
+    snprintf(user_dbname, sizeof(user_dbname), "%s%s", (xine_get_homedir()), "/.Xdefaults");
     server_rmdb = XrmGetFileDatabase(user_dbname);
   }
   
@@ -374,7 +375,7 @@ static void xrm_parse(void) {
     int len;
     
     environment = environement_buf;
-    sprintf(environement_buf, "%s%s", (xine_get_homedir()), "/.Xdefaults-");
+    snprintf(environement_buf, sizeof(environement_buf), "%s%s", (xine_get_homedir()), "/.Xdefaults-");
     len = strlen(environment);
     (void) gethostname(environment + len, (XITK_PATH_MAX + XITK_NAME_MAX) - len);
   }
@@ -422,7 +423,7 @@ static void show_version(void) {
 #ifdef DEBUG
   printf("-[DEBUG]");
 #endif
-  printf(_(".\n(c) 2000-2003 The xine Team.\n"));
+  printf(_(".\n(c) 2000-2004 The xine Team.\n"));
 }
 
 /*
@@ -460,13 +461,12 @@ static void print_formatted(char *title, const char *const *plugins) {
     len = strlen(buffer);
     
     if((len + (strlen(plugin) + 3)) < 80) {
-      sprintf(buffer, "%s%s%s", buffer, 
-	      (strlen(buffer) == strlen(blanks)) ? "" : ", ", plugin);
+      sprintf(buffer, "%s%s%s", buffer, (strlen(buffer) == strlen(blanks)) ? "" : ", ", plugin);
     }
     else {
       printf(buffer);
       printf(",\n");
-      sprintf(buffer, "%s%s", blanks, plugin);
+      snprintf(buffer, sizeof(buffer), "%s%s", blanks, plugin);
     }
     
     plugin = *plugins++;
@@ -621,6 +621,7 @@ static void show_usage (void) {
   printf(_("  -F, --xineramafull           start in xinerama fullscreen (display on several screens),\n"));
 #endif
   printf(_("  -g, --hide-gui               hide GUI (panel, etc.)\n"));
+  printf(_("  -I, --no-gui                 disable GUI\n"));
   printf(_("  -H, --hide-video             hide video window\n"));
 #ifdef HAVE_LIRC
   printf(_("  -L, --no-lirc                Turn off LIRC support.\n"));
@@ -731,15 +732,15 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
   x11_visual_t            vis;
   int                     driver_num;
 
-  vis.display           = gGui->display;
-  vis.screen            = gGui->screen;
+  vis.display           = gGui->video_display;
+  vis.screen            = gGui->video_screen;
   vis.d                 = gGui->video_window;
-  XLockDisplay(gGui->display);
-  res_h                 = (DisplayWidth  (gGui->display, gGui->screen)*1000 
-			   / DisplayWidthMM (gGui->display, gGui->screen));
-  res_v                 = (DisplayHeight (gGui->display, gGui->screen)*1000
-			   / DisplayHeightMM (gGui->display, gGui->screen));
-  XUnlockDisplay(gGui->display);
+  XLockDisplay(gGui->video_display);
+  res_h                 = (DisplayWidth  (gGui->video_display, gGui->video_screen)*1000 
+			   / DisplayWidthMM (gGui->video_display, gGui->video_screen));
+  res_v                 = (DisplayHeight (gGui->video_display, gGui->video_screen)*1000
+			   / DisplayHeightMM (gGui->video_display, gGui->video_screen));
+  XUnlockDisplay(gGui->video_display);
   gGui->pixel_aspect    = res_v / res_h;
   
 #ifdef DEBUG
@@ -1006,10 +1007,6 @@ static void event_listener(void *user_data, const xine_event_t *event) {
   case XINE_EVENT_UI_PLAYBACK_FINISHED:
     if(event->stream == gGui->stream) {
 
-      /* There a reference stream waiting to be played, just wait for */
-      if(gGui->got_reference_stream)
-	return;
-
       gui_playlist_start_next();
     }
     else if(event->stream == gGui->visual_anim.stream) {
@@ -1098,10 +1095,9 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 		 "%s:", _("*drum roll*, xine lib wants to take your attention "
 			  "to deliver an important message to you ;-):"));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s %s %s", buffer,
-		   (char *) data + data->explanation, (char *) data + data->parameters);
+	  sprintf(buffer, "%s %s %s", buffer, (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  snprintf(buffer, sizeof(buffer), "%s %s", buffer, _("No Information available."));
+	  sprintf(buffer, "%s %s", buffer, _("No Information available."));
 	  
 	break;
 
@@ -1110,14 +1106,14 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	snprintf(buffer, sizeof(buffer), "%s", _("The host you're trying to connect is unknown.\n"
 						 "Check the validity of the specified hostname."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 	
 	/* (device name) */
       case XINE_MSG_UNKNOWN_DEVICE:
 	snprintf(buffer, sizeof(buffer), "%s", _("The device name you specified seems invalid."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* none */
@@ -1125,21 +1121,21 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	snprintf(buffer, sizeof(buffer), "%s", _("The network looks unreachable.\nCheck your network "
 						 "setup and/or the server name."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (host name) */
       case XINE_MSG_CONNECTION_REFUSED:
 	snprintf(buffer, sizeof(buffer), "%s", _("The connection was refused.\nCheck the host name."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (file name or mrl) */
       case XINE_MSG_FILE_NOT_FOUND:
 	snprintf(buffer, sizeof(buffer), "%s", _("The specified file or mrl is not found. Please check it twice."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (device/file/mrl) */
@@ -1148,35 +1144,34 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 						 "rights for this, or source doesn't contain data "
 						 "(e.g: not disc in drive)."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 	
 	/* (library/decoder) */
       case XINE_MSG_LIBRARY_LOAD_ERROR:
 	snprintf(buffer, sizeof(buffer), "%s", _("A problem occur while loading a library or a decoder"));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s: %s", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s: %s", buffer, (char *) data + data->parameters);
 	break;
 
 	/* none */
       case XINE_MSG_ENCRYPTED_SOURCE:
 	snprintf(buffer, sizeof(buffer), "%s", _("The source seems encrypted, and can't be read."));
 	if(!strncasecmp(gGui->mmk.mrl, "dvd:/", 5)) {
-	  snprintf(buffer, sizeof(buffer), "%s", _("\nYou DVD is probably crypted. "
-						   "According to your country laws, you can or can't "
-						   "install/use libdvdcss to be able to read this disc, "
-						   "which you bought."));
+	  sprintf(buffer, "%s%s", buffer, _("\nYour DVD is probably crypted. "
+					    "According to your country laws, you can or can't "
+					    "install/use libdvdcss to be able to read this disc, "
+					    "which you bought."));
 	}
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (warning message) */
       case XINE_MSG_SECURITY:
 	report = xine_info;
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s %s",
-		   (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s %s", (char *) data + data->explanation, (char *) data + data->parameters);
 	else
 	  snprintf(buffer, sizeof(buffer), "%s", _("No Informations available."));
 	break;
@@ -1190,19 +1185,17 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       default:
 	snprintf(buffer, sizeof(buffer), "%s", _("*sight*, unkown error."));
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s (%s %s)", buffer,
-		   (char *) data + data->explanation, (char *) data + data->parameters);
+	  sprintf(buffer, "%s (%s %s)", buffer, (char *) data + data->explanation, (char *) data + data->parameters);
 	break;
       }
       
       if(gGui->verbosity >= XINE_VERBOSITY_DEBUG) {
-	snprintf(buffer, sizeof(buffer), "%s\n\n[", buffer);
+	sprintf(buffer, "%s\n\n[", buffer);
 	
 	if(data->explanation)
-	  snprintf(buffer, sizeof(buffer), "%s'%s' '%s'", buffer, 
-		   (char *) data + data->explanation, (char *) data + data->parameters);
+	  sprintf(buffer, "%s'%s' '%s'", buffer, (char *) data + data->explanation, (char *) data + data->parameters);
 	
-	snprintf(buffer, sizeof(buffer), "%s]", buffer);
+	sprintf(buffer, "%s]", buffer);
       }
       
       if(strlen(buffer))
@@ -1241,7 +1234,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       char                  buffer[1024];
       
       memset(&buffer, 0, sizeof(buffer));
-      sprintf(buffer, "%s [%d%%]\n", pevent->description, pevent->percent);
+      snprintf(buffer, sizeof(buffer), "%s [%d%%]\n", pevent->description, pevent->percent);
       gGui->mrl_overrided = 3;
       panel_set_title(buffer);
       osd_display_info(buffer);
@@ -1252,29 +1245,22 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     if((event->stream == gGui->stream) && gGui->playlist.num) {
       xine_mrl_reference_data_t *ref = (xine_mrl_reference_data_t *) event->data;
 
-      if(ref->alternative == 0) {
-	mediamark_t *mmk = mediamark_clone_mmk((mediamark_t *) mediamark_get_current_mmk());
+      if(gGui->verbosity)
+	printf("XINE_EVENT_MRL_REFERENCE got mrl [%s] (alternative=%d)\n",
+               ref->mrl, ref->alternative);
 
+      if(ref->alternative == 0) {
+        gGui->playlist.ref_append++;
+        mediamark_insert_entry(gGui->playlist.ref_append, ref->mrl, ref->mrl, NULL, 0, -1, 0, 0);
+      } else {
+	mediamark_t *mmk = mediamark_get_mmk_by_index(gGui->playlist.ref_append);
+	
 	if(mmk) {
-	  mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur],
-				  ref->mrl, 
-				  ((mmk->ident && (strcmp(mmk->mrl, mmk->ident) == 0)) ?
-				   ref->mrl : mmk->ident), 
-				  mmk->sub, mmk->start, mmk->end, mmk->av_offset, mmk->spu_offset);
-	  mediamark_free_mmk(&mmk);
-	  
-	  mmk = (mediamark_t *) mediamark_get_current_mmk();
-	  gui_set_current_mrl(mmk);
-	  
-	  playlist_update_playlist();
-	  panel_update_mrl_display();
-	  
-	  /* We can't do anything more here, otherwise deadlock occur */
-	  gGui->got_reference_stream++;
+	  mediamark_append_alternate_mrl(mmk, ref->mrl);
+	  mediamark_set_got_alternate(mmk);
 	}
+
       }
-      else
-	mediamark_add_entry(ref->mrl, ref->mrl, NULL, 0, -1, 0, 0);
 
     }
     break;
@@ -1294,12 +1280,8 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     break;
 
   case XINE_EVENT_DROPPED_FRAMES:
-    {
-      char buffer[2048];
-      
-      memset(&buffer, 0, sizeof(buffer));
-      snprintf(buffer, sizeof(buffer), "%s", _("Your system seems too slow, the amoung of dropped frames is too high."));
-      xine_error_with_more(buffer);
+    if (xine_get_param(gGui->stream, XINE_PARAM_SPEED) <= XINE_SPEED_NORMAL) {
+      too_slow_window();
     }
     break;
 
@@ -1391,6 +1373,8 @@ int main(int argc, char *argv[]) {
 #endif
   gGui->deinterlace_enable     = 0;
   gGui->report                 = stdout;
+  gGui->ssaver_enabled         = 1;
+  gGui->no_gui                 = 0;
 
   window_attribute.x     = window_attribute.y      = -8192;
   window_attribute.width = window_attribute.height = -1;
@@ -1672,10 +1656,7 @@ int main(int argc, char *argv[]) {
 	session_handle_subopt(optarg, &session);
       else {
 
-	if(!session_argv_num)
-	  session_argv = (char **) xine_xmalloc(sizeof(char *) * 2);
-	else
-	  session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
+	session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
 
 	session_argv[session_argv_num++] = strdup(optarg);
 	session_argv[session_argv_num]   = NULL;
@@ -1702,10 +1683,7 @@ int main(int argc, char *argv[]) {
       break;
 
     case OPTION_POST:
-      if(!pplugins_num)
-	pplugins = (char **) xine_xmalloc(sizeof(char *) * 2);
-      else
-	pplugins = (char **) realloc(pplugins, sizeof(char *) * (pplugins_num + 2));
+      pplugins = (char **) realloc(pplugins, sizeof(char *) * (pplugins_num + 2));
       
       pplugins[pplugins_num++] = optarg;
       pplugins[pplugins_num] = NULL;
@@ -1780,10 +1758,7 @@ int main(int argc, char *argv[]) {
 	if(optarg) {
 	  char *p = xine_chomp(optarg);
 
-	  if(!session_argv_num)
-	    session_argv = (char **) xine_xmalloc(sizeof(char *) * 2);
-	  else
-	    session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
+	  session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
 	  
 	  session_argv[session_argv_num] = (char *) xine_xmalloc(strlen(p) + 5);
 	  sprintf(session_argv[session_argv_num], "mrl=%s", p);
@@ -1797,6 +1772,10 @@ int main(int argc, char *argv[]) {
       }
       break;
 
+    case 'I':
+      gGui->no_gui = 1;
+      break;
+
     default:
       show_usage();
       fprintf (stderr, _("invalid argument %d => exit\n"), c);
@@ -1897,7 +1876,7 @@ int main(int argc, char *argv[]) {
   if(old_playlist_cfg && (!(_argc - optind)) && (!no_old_playlist)) {
     char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
-    sprintf(buffer, "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
+    snprintf(buffer, sizeof(buffer), "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
     mediamark_load_mediamarks(buffer);
   }
 
@@ -1950,7 +1929,7 @@ int main(int argc, char *argv[]) {
     xine_cfg_entry_t  cfg_entry;
     
     if(xine_config_lookup_entry(gGui->xine, "input.file_origin_path", &cfg_entry))
-      sprintf(gGui->curdir, "%s", cfg_entry.str_value);
+      snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", cfg_entry.str_value);
     else
       (void *) getcwd(&(gGui->curdir[0]), XITK_PATH_MAX);
 
