@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menus.c,v 1.38 2003/11/18 22:15:44 f1rmb Exp $
+ * $Id: menus.c,v 1.55 2004/07/02 21:41:03 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@ extern _panel_t                *panel;
 #define PLAYB_PREV              5
 #define PLAYB_SPEEDM            6
 #define PLAYB_SPEEDL            7
+#define PLAYB_ADDMMK            8
 
 #define PLAYL_LOAD              10
 #define PLAYL_SAVE              11
@@ -60,6 +61,7 @@ extern _panel_t                *panel;
 #define VIDEO_INTERLEAVE        34
 #define VIDEO_PPROCESS          35
 #define VIDEO_PPROCESS_ENABLE   36
+#define VIDEO_TOGGLE            37
 
 #define SETS_SETUP              40
 #define SETS_KEYMAP             41
@@ -73,9 +75,26 @@ extern _panel_t                *panel;
 
 #define IS_CHANNEL_CHECKED(C, N) (C == N) ? "<checked>" : "<check>"
 
+static char *menu_get_shortcut(char *action) {
+  char *shortcut = kbindings_get_shortcut(gGui->kbindings, action);
+#ifdef DEBUG
+  if(!shortcut) {
+    fprintf(stderr, "Action '%s' is invalid\n", action);
+    abort();
+  }
+#endif
+  return (shortcut ? strdup(shortcut) : NULL);
+}
+
 static void menu_open_mrlbrowser(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   open_mrlbrowser_from_playlist(w, data);
 }
+static void menu_scan_infos(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_scan_for_infos();
+}
+static void menu_scan_infos_selected(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_scan_for_infos_selected();
+}
 static void menu_playlist_mmk_editor(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   playlist_mmk_editor();
 }
@@ -149,6 +168,10 @@ static void menu_playback_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *da
     gui_execute_action_id(ACTID_SPEED_SLOW);
     break;
 
+  case PLAYB_ADDMMK:
+    gui_execute_action_id(ACTID_ADDMEDIAMARK);
+    break;
+
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -223,10 +246,10 @@ static void menu_playlist_from(xitk_widget_t *w, xitk_menu_entry_t *me, void *da
       gGui->playlist.cur = 0;
     
     for (j = 0; j < num_mrls; j++)
-      mediamark_add_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0, 0);
+      mediamark_append_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0, 0);
     
     if(gGui->playlist.cur == 0)
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
     
     /* 
      * If we're in newbie mode, start playback immediately
@@ -349,6 +372,10 @@ static void menu_video_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *data)
     gui_execute_action_id(ACTID_VPP_ENABLE);
     break;
 
+  case VIDEO_TOGGLE:
+    gui_execute_action_id(ACTID_TOGGLE_WINOUT_VISIBLITY);
+    break;
+    
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -400,6 +427,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
   int                  x, y;
   xitk_menu_widget_t   menu;
   char                 buffer[2048];
+  char                *sh[255];
+  int                  shc = 0;
   xitk_widget_t       *w;
 #ifdef HAVE_XINERAMA
   int                  fullscr_mode = (FULLSCR_MODE | FULLSCR_XI_MODE);
@@ -408,266 +437,350 @@ void video_window_menu(xitk_widget_list_t *wl) {
 #endif
   xitk_menu_entry_t    menu_entries[] = {
     { NULL ,
+      NULL,
       "<title>",      
       NULL, NULL                                                                             },
-    { _("GUI visiblity"),
+    { _("Show controls"),
+      (sh[shc++] = menu_get_shortcut("ToggleVisibility")),
       panel_is_visible() ? "<checked>" : "<check>",  
       menu_panel_visibility, NULL                                                            },
+    { _("Show video window"),
+      (sh[shc++] = menu_get_shortcut("ToggleWindowVisibility")),
+      video_window_is_visible() ? "<checked>" : "<check>",  
+      menu_video_ctrl, (void *) VIDEO_TOGGLE                                                 },
+    { _("Fullscreen"),
+      (sh[shc++] = menu_get_shortcut("ToggleFullscreen")),
+      (video_window_get_fullscreen_mode() & fullscr_mode) ? "<checked>" : "<check>",
+      menu_video_ctrl, (void *) VIDEO_FULLSCR                                                },
     { "SEP",  
+      NULL,
       "<separator>",  
       NULL,  NULL                                                                            },
     { _("Open"),
+      NULL,
       "<branch>",   
       NULL, NULL                                                                             },
     { _("Open/File..."),
+      (sh[shc++] = menu_get_shortcut("FileSelector")),
       NULL,
       menu_file_selector,        NULL                                                        },
     { _("Open/Playlist..."),
+      NULL,
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_LOAD                                                },
     { _("Open/Location..."),
+      (sh[shc++] = menu_get_shortcut("MrlBrowser")),
       NULL,
       menu_mrl_browser, NULL                                                                 },
     { _("Playback"),
+      NULL,
       "<Branch>",
       NULL, NULL                                                                             },
     { _("Playback/Play"),
+      (sh[shc++] = menu_get_shortcut("Play")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_PLAY                                                },
     { _("Playback/Stop"),
+      (sh[shc++] = menu_get_shortcut("Stop")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_STOP                                                },
     { _("Playback/Pause"),
+      (sh[shc++] = menu_get_shortcut("Pause")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_PAUSE                                               },
     { _("Playback/SEP"),
+      NULL,
       "<separator>",  
       NULL,  NULL                                                                            },
     { _("Playback/Next MRL"),
+      (sh[shc++] = menu_get_shortcut("NextMrl")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_NEXT                                                },
     { _("Playback/Previous MRL"),
+      (sh[shc++] = menu_get_shortcut("PriorMrl")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_PREV                                                },
     { _("Playback/SEP"),
+      NULL,
       "<separator>",  
       NULL,  NULL                                                                            },
     { _("Playback/Increase Speed"),
+      (sh[shc++] = menu_get_shortcut("SpeedFaster")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_SPEEDM                                              },
     { _("Playback/Decrease Speed"),
+      (sh[shc++] = menu_get_shortcut("SpeedSlower")),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_SPEEDL                                              },
     { _("Playlist"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Playlist/Get from"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Playlist/Load..."),
+      NULL,
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_LOAD                                                },
     { _("Playlist/Editor..."),
+      (sh[shc++] = menu_get_shortcut("PlaylistEditor")),
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_EDIT                                                },
     { _("Playlist/SEP"),  
+      NULL,
       "<separator>",  
       NULL, NULL                                                                             },
     { _("Playlist/Loop modes"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Playlist/Loop modes/Disabled"),
+      NULL,
       (gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) ? "<checked>" : "<check>",
       menu_playlist_ctrl, (void *) PLAYL_NO_LOOP                                             },
     { _("Playlist/Loop modes/Loop"),
+      NULL,
       (gGui->playlist.loop == PLAYLIST_LOOP_LOOP) ? "<checked>" : "<check>",
       menu_playlist_ctrl, (void *) PLAYL_LOOP                                                },
     { _("Playlist/Loop modes/Repeat Selection"),
+      NULL,
       (gGui->playlist.loop == PLAYLIST_LOOP_REPEAT) ? "<checked>" : "<check>",
       menu_playlist_ctrl, (void *) PLAYL_REPEAT                                              },
     { _("Playlist/Loop modes/Shuffle"),
+      NULL,
       (gGui->playlist.loop == PLAYLIST_LOOP_SHUFFLE) ? "<checked>" : "<check>",
       menu_playlist_ctrl, (void *) PLAYL_SHUFFLE                                             },
     { _("Playlist/Loop modes/Non-stop Shuffle"),
+      NULL,
       (gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS) ? "<checked>" : "<check>",
       menu_playlist_ctrl, (void *) PLAYL_SHUF_PLUS                                           },
-    { _("Playlist/SEP"),  
-      "<separator>",
-      NULL, NULL                                                                             },
-    { _("Playlist/Continue"),
+    { _("Playlist/Continue Playback"),
+      (sh[shc++] = menu_get_shortcut("PlaylistStop")),
       (gGui->playlist.control & PLAYLIST_CONTROL_STOP) ? "<check>" : "<checked>",
       menu_playlist_ctrl, (void *) PLAYL_CTRL_STOP                                           },
     { "SEP",  
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Menus"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Menus/Navigation..."),
+      (sh[shc++] = menu_get_shortcut("EventSenderShow")),
       NULL,
       menu_event_sender, NULL                                                                },
     { _("Menus/SEP"),  
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { "SEP",  
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Stream"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Stream/Information..."),
+      (sh[shc++] = menu_get_shortcut("StreamInfosShow")),
       NULL,
       menu_stream, (void *) STREAM_WINI                                                      },
     { _("Stream/Information (OSD)"),
+      (sh[shc++] = menu_get_shortcut("OSDStreamInfos")),
       NULL,
       menu_stream, (void *) STREAM_OSDI                                                      },
-    { _("Fullscreen\\/Window"),
-      (video_window_get_fullscreen_mode() & fullscr_mode) ? "<checked>" : "<check>",
-      menu_video_ctrl, (void *) VIDEO_FULLSCR                                                },
     { _("Video"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Video/Deinterlace"),
+      (sh[shc++] = menu_get_shortcut("ToggleInterleave")),
       (xine_get_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE)) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_INTERLEAVE                                             },
     { _("Video/SEP"),
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Video/Aspect ratio"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Video/Aspect ratio/Automatic"),
+      NULL,
       (aspect == XINE_VO_ASPECT_AUTO) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_AUTO                                              },
     { _("Video/Aspect ratio/Square"),
+      NULL,
       (aspect == XINE_VO_ASPECT_SQUARE) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_SQUARE                                            },
     { _("Video/Aspect ratio/4:3"),
+      NULL,
       (aspect == XINE_VO_ASPECT_4_3) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_4_3                                               },
     { _("Video/Aspect ratio/Anamorphic"),
+      NULL,
       (aspect == XINE_VO_ASPECT_ANAMORPHIC) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_ANAMORPHIC                                        },
     { _("Video/Aspect ratio/DVB"),
+      NULL,
       (aspect == XINE_VO_ASPECT_DVB) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_DVB                                               },
     { _("Video/200%"),
+      (sh[shc++] = menu_get_shortcut("Window200")),
       (video_window_get_mag() == 2.0) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_2X                                                     },
     { _("Video/100%"),
+      (sh[shc++] = menu_get_shortcut("Window100")),
       (video_window_get_mag() == 1.0) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_1X                                                     },
     { _("Video/50%"),
+      (sh[shc++] = menu_get_shortcut("Window50")),
       (video_window_get_mag() ==  .5) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO__5X                                                    },
     { _("Video/SEP"),
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Video/Postprocess"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Video/Postprocess/Chain Reaction..."),
+      (sh[shc++] = menu_get_shortcut("VPProcessShow")),
       NULL,
       menu_video_ctrl, (void *) VIDEO_PPROCESS                                               },
     { _("Video/Postprocess/Enable Postprocessing"),
+      (sh[shc++] = menu_get_shortcut("VPProcessEnable")),
       gGui->post_enable ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_PPROCESS_ENABLE                                        },
     { _("Audio"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Audio/Volume"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Audio/Volume/Mute"),
+      (sh[shc++] = menu_get_shortcut("Mute")),
       gGui->mixer.mute ? "<checked>" : "<check>",
       menu_audio_ctrl, (void *) AUDIO_MUTE                                                   },
     { _("Audio/Volume/Increase 10%"),
+      NULL,
       NULL,
       menu_audio_ctrl, (void *) AUDIO_INCRE_VOL                                              },
     { _("Audio/Volume/Decrease 10%"),
+      NULL,
       NULL,
       menu_audio_ctrl, (void *) AUDIO_DECRE_VOL                                              },
     { _("Audio/Channel"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Audio/Visualization"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Subtitle"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Subtitle/Channel"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { "SEP",  
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Settings"),
+      NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Settings/Setup..."),
+      (sh[shc++] = menu_get_shortcut("SetupShow")),
       NULL,
       menu_settings, (void *) SETS_SETUP                                                     },
 #ifdef HAVE_CURL
     { _("Settings/Skin Downloader..."),
+      (sh[shc++] = menu_get_shortcut("SkinDownload")),
       NULL,
       menu_settings, (void *) SETS_SKINDL                                                    },
 #endif
     { _("Settings/Keymap Editor..."),
+      (sh[shc++] = menu_get_shortcut("KeyBindingEditor")),
       NULL,
       menu_settings, (void *) SETS_KEYMAP                                                    },
     { _("Settings/Video..."),
+      (sh[shc++] = menu_get_shortcut("ControlShow")),
       NULL,
       menu_settings, (void *) SETS_VIDEO                                                     },
     { _("Settings/TV Analog..."),
+      (sh[shc++] = menu_get_shortcut("TVAnalogShow")),
       NULL,
       menu_settings, (void *) SETS_TVANALOG                                                  },
     { "SEP",
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Help..."),
+      (sh[shc++] = menu_get_shortcut("HelpShow")),
       NULL,
       menu_help, NULL                                                                        },
     { _("Logs..."),
+      (sh[shc++] = menu_get_shortcut("ViewlogShow")),
       NULL,
       menu_settings, (void *) SETS_LOGS                                                      },
     { "SEP",
+      NULL,
       "<separator>",
       NULL, NULL                                                                             },
     { _("Quit"),
+      (sh[shc++] = menu_get_shortcut("Quit")),
       NULL,
       menu_quit, NULL                                                                        },
     { NULL,
+      NULL,
       NULL,
       NULL, NULL                                                                             }
   };
 
-  sprintf(buffer, _("xine %s"), VERSION);
+  if(gGui->no_gui)
+    return;
+  
+  snprintf(buffer, sizeof(buffer), _("xine %s"), VERSION);
   menu_entries[0].menu = buffer;
 
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
-  (void) xitk_get_mouse_coords(gGui->display, 
-			       gGui->video_window, NULL, NULL, &x, &y);
   
+  XLockDisplay(gGui->video_display);
+  (void) xitk_get_mouse_coords(gGui->video_display, 
+			       gGui->video_window, NULL, NULL, &x, &y);
+  XUnlockDisplay(gGui->video_display);
+
   menu.menu_tree         = &menu_entries[0];
   menu.parent_wlist      = wl;
   menu.skin_element_name = NULL;
   
   w = xitk_noskin_menu_create(wl, &menu, x + 1, y + 1);
 
+  while(shc > 0)
+    free(sh[--shc]);
+
   /* Subtitle loader */
   if(gGui->playlist.num) {
     xitk_menu_entry_t   menu_entry;
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Open/Subtitle...");
-    menu_entry.type      = NULL;
     menu_entry.cb        = menu_subtitle_selector;
-    menu_entry.user_data = NULL;
     xitk_menu_add_entry(w, &menu_entry);
 
     /* Save Playlist */
     menu_entry.menu      = _("Playlist/Save...");
-    menu_entry.type      = NULL;
     menu_entry.cb        = menu_playlist_ctrl;
     menu_entry.user_data = (void *) PLAYL_SAVE;
     xitk_menu_add_entry(w, &menu_entry);
@@ -684,12 +797,11 @@ void video_window_menu(xitk_widget_list_t *wl) {
     while(plug_id) {
 
       memset(&buffer, 0, sizeof(buffer));
-      sprintf(buffer, "%s/%s", location, plug_id);
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, plug_id);
       
       menu_entry.menu      = buffer;
-      menu_entry.type      = NULL;
       menu_entry.cb        = menu_playlist_from;
-      menu_entry.user_data = NULL;
       xitk_menu_add_entry(w, &menu_entry);
       
       plug_id = *plug_ids++;
@@ -706,7 +818,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
       char                buffer[2048];
 
       while(viz_names[i]) {
-	sprintf(buffer, "%s/%s", location, viz_names[i]);
+	memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+	snprintf(buffer, sizeof(buffer), "%s/%s", location, viz_names[i]);
 	menu_entry.menu      = buffer;
 	menu_entry.type      = IS_CHANNEL_CHECKED(i, gGui->visual_anim.post_plugin_num);
 	menu_entry.cb        = menu_audio_viz;
@@ -717,10 +830,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
 
     }
     else {
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
       menu_entry.menu      = _("Audio/Visualization/None");
-      menu_entry.type      = NULL;
-      menu_entry.cb        = NULL;
-      menu_entry.user_data = NULL;
       xitk_menu_add_entry(w, &menu_entry);
     }
     
@@ -733,12 +844,14 @@ void video_window_menu(xitk_widget_list_t *wl) {
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Audio/Channel/Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_audio_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Audio/Channel/Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_audio_chan;
@@ -754,7 +867,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
 
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
-	    sprintf(buffer, "%s/%d", location, i);
+	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+	    snprintf(buffer, sizeof(buffer), "%s/%d", location, i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_audio_chan;
@@ -766,7 +880,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
 	break;
       }
       
-      sprintf(buffer, "%s/%s", location, (get_language_from_iso639_1(langbuf)));
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, (get_language_from_iso639_1(langbuf)));
       menu_entry.menu      = buffer;
       menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
       menu_entry.cb        = menu_audio_chan;
@@ -783,12 +898,14 @@ void video_window_menu(xitk_widget_list_t *wl) {
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_SPU_CHANNEL);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Subtitle/Channel/Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_spu_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Subtitle/Channel/Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_spu_chan;
@@ -804,7 +921,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
 	
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
-	    sprintf(buffer, "%s/%d", location, i);
+	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+	    snprintf(buffer, sizeof(buffer), "%s/%d", location, i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_spu_chan;
@@ -816,7 +934,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
 	break;
       }
       
-      sprintf(buffer, "%s/%s", location, (get_language_from_iso639_1(langbuf)));
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, (get_language_from_iso639_1(langbuf)));
       menu_entry.menu      = buffer;
       menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
       menu_entry.cb        = menu_spu_chan;
@@ -845,16 +964,41 @@ void video_window_menu(xitk_widget_list_t *wl) {
       menu = dvd_menu;
     
     for(i = 0; i < 7; i++) {
-      sprintf(buffer, "%s/%s", location, menu[i]);
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, menu[i]);
       
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
       menu_entry.menu      = buffer;
-      menu_entry.type      = NULL;
       menu_entry.cb        = menu_menus_selection;
       menu_entry.user_data = (void *)i;
       xitk_menu_add_entry(w, &menu_entry);
     }
   }
 
+  /* Mediamark */
+  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+    xitk_menu_entry_t   menu_entry;
+    char                buffer[2048];
+    char               *location = _("Playback");
+
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+    menu_entry.menu      = _("Playback/SEP");
+    menu_entry.type      = "<separator>";
+    xitk_menu_add_entry(w, &menu_entry);
+    
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+    memset(&buffer, 0, sizeof(buffer));
+    
+    snprintf(buffer, sizeof(buffer), "%s/%s", location, _("Add Mediamark"));
+    menu_entry.menu      = buffer;
+    menu_entry.shortcut  = menu_get_shortcut("AddMediamark");
+    menu_entry.cb        = menu_playback_ctrl;
+    menu_entry.user_data = (void *) PLAYB_ADDMMK;
+    xitk_menu_add_entry(w, &menu_entry);
+    
+    if(menu_entry.shortcut)
+      free(menu_entry.shortcut);
+  }
+
   xitk_menu_show_menu(w);
 }
 
@@ -863,12 +1007,12 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
   char                 buffer[2048];
   xitk_widget_t       *w;
   xitk_menu_entry_t    menu_entries[] = {
-    { NULL , "<title>",      NULL, NULL },
-    { "SEP", "<separator>",  NULL, NULL },
-    { NULL,  NULL,           NULL, NULL }
+    { NULL , NULL, "<title>",      NULL, NULL },
+    { "SEP", NULL, "<separator>",  NULL, NULL },
+    { NULL,  NULL, NULL,           NULL, NULL }
   };
 
-  sprintf(buffer, "%s", _("Audio"));
+  snprintf(buffer, sizeof(buffer), "%s", _("Audio"));
   menu_entries[0].menu = buffer;
   
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
@@ -885,12 +1029,14 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_audio_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_audio_chan;
@@ -906,7 +1052,8 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
-	    sprintf(buffer, "%d", i);
+	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+	    snprintf(buffer, sizeof(buffer), "%d", i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_audio_chan;
@@ -918,6 +1065,7 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 	break;
       }
       
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
       menu_entry.menu      = (char *) get_language_from_iso639_1(langbuf);
       menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
       menu_entry.cb        = menu_audio_chan;
@@ -935,12 +1083,12 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
   char                 buffer[2048];
   xitk_widget_t       *w;
   xitk_menu_entry_t    menu_entries[] = {
-    { NULL , "<title>",      NULL, NULL },
-    { "SEP", "<separator>",  NULL, NULL },
-    { NULL,  NULL,           NULL, NULL }
+    { NULL , NULL, "<title>",      NULL, NULL },
+    { "SEP", NULL, "<separator>",  NULL, NULL },
+    { NULL,  NULL, NULL,           NULL, NULL }
   };
 
-  sprintf(buffer, "%s", _("Subtitle"));
+  snprintf(buffer, sizeof(buffer), "%s", _("Subtitle"));
   menu_entries[0].menu = buffer;
   
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
@@ -957,6 +1105,7 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_SPU_CHANNEL);
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_spu_chan;
@@ -978,7 +1127,8 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 	
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
-	    sprintf(buffer, "%d", i);
+	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+	    snprintf(buffer, sizeof(buffer), "%d", i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_spu_chan;
@@ -990,6 +1140,7 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 	break;
       }
       
+      memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
       menu_entry.menu      = (char *) get_language_from_iso639_1(langbuf);
       menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
       menu_entry.cb        = menu_spu_chan;
@@ -1006,31 +1157,37 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
   xitk_menu_widget_t   menu;
   xitk_widget_t       *w = NULL;
   char                 buffer[2048];
+  char                *sh[50];
+  int                  shc = 0;
   xitk_menu_entry_t    menu_entries_nosel[] = {
-    { NULL ,           "<title>",     NULL,                         NULL                    },
-    { "SEP",           "<separator>", NULL,                         NULL                    },
-    { _("Add"),        NULL,          menu_open_mrlbrowser,         NULL                    },
-    { NULL,            NULL,          NULL,                         NULL                    }
+    { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
+    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
+    { _("Scan"),       (sh[shc] = menu_get_shortcut("ScanPlaylistInfo")),
+                                      NULL,          menu_scan_infos,              NULL                    },
+    { _("Add"),        NULL,          NULL,          menu_open_mrlbrowser,         NULL                    },
+    { NULL,            NULL,          NULL,          NULL,                         NULL                    }
   };
   xitk_menu_entry_t    menu_entries_sel[] = {
-    { NULL ,           "<title>",     NULL,                         NULL                    },
-    { "SEP",           "<separator>", NULL,                         NULL                    },
-    { _("Play"),       NULL,          menu_playlist_play_current,   NULL                    },
-    { "SEP",           "<separator>", NULL,                         NULL                    },
-    { _("Add"),        NULL,          menu_open_mrlbrowser,         NULL                    },
-    { _("Edit"),       NULL,          menu_playlist_mmk_editor,     NULL                    },
-    { _("Delete"),     NULL,          menu_playlist_delete_current, NULL                    },
-    { _("Delete All"), NULL,          menu_playlist_delete_all,     NULL                    },
-    { "SEP",           "<separator>", NULL,                         NULL                    },
-    { _("Move Up"),    NULL,          menu_playlist_move_updown,    (void *) DIRECTION_UP   },
-    { _("Move Down"),  NULL,          menu_playlist_move_updown,    (void *) DIRECTION_DOWN },
-    { NULL,            NULL,          NULL,                         NULL                    }
+    { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
+    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
+    { _("Play"),       NULL,          NULL,          menu_playlist_play_current,   NULL                    },
+    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
+    { _("Scan"),       NULL,          NULL,          menu_scan_infos_selected,     NULL                    },
+    { _("Add"),        NULL,          NULL,          menu_open_mrlbrowser,         NULL                    },
+    { _("Edit"),       (sh[shc] = menu_get_shortcut("MediamarkEditor")),
+                                      NULL,          menu_playlist_mmk_editor,     NULL                    },
+    { _("Delete"),     NULL,          NULL,          menu_playlist_delete_current, NULL                    },
+    { _("Delete All"), NULL,          NULL,          menu_playlist_delete_all,     NULL                    },
+    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
+    { _("Move Up"),    NULL,          NULL,          menu_playlist_move_updown,    (void *) DIRECTION_UP   },
+    { _("Move Down"),  NULL,          NULL,          menu_playlist_move_updown,    (void *) DIRECTION_DOWN },
+    { NULL,            NULL,          NULL,          NULL,                         NULL                    }
   };
 
   
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
   
-  sprintf(buffer, "%s", _("Playlist"));
+  snprintf(buffer, sizeof(buffer), "%s", _("Playlist"));
   if(selected) {
     menu_entries_sel[0].menu = buffer;
     menu.menu_tree           = &menu_entries_sel[0];
@@ -1045,13 +1202,15 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
 
   w = xitk_noskin_menu_create(wl, &menu, x, y);
 
+  while(shc > 0)
+    free(sh[--shc]);
+  
   if(!selected && gGui->playlist.num) {
     xitk_menu_entry_t   menu_entry;
     
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
     menu_entry.menu      = _("Delete All");
-    menu_entry.type      = NULL;
     menu_entry.cb        = menu_playlist_delete_all;
-    menu_entry.user_data = NULL;
     xitk_menu_add_entry(w, &menu_entry);
   }
 
