@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrl_browser.c,v 1.68 2003/10/21 19:17:57 f1rmb Exp $
+ * $Id: mrl_browser.c,v 1.73 2004/07/02 09:33:37 f1rmb Exp $
  *
  * MRL Browser
  *
@@ -52,7 +52,7 @@ void mrl_browser_show_tips(int enabled, unsigned long timeout) {
 void set_mrl_browser_transient(void) {
 
   if(mrlb) {
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       xitk_mrlbrowser_set_transient(mrlb, gGui->video_window);
   }
 }
@@ -65,6 +65,8 @@ void mrl_browser_change_skins(void) {
   if(mrlb) {
     xitk_mrlbrowser_change_skins(mrlb, gGui->skin_config);
     set_mrl_browser_transient();
+    if(mrl_browser_is_visible())
+      raise_window((xitk_mrlbrowser_get_window_id(mrlb)), 1, 1);
   }
 }
 
@@ -140,6 +142,8 @@ void destroy_mrl_browser(void) {
     }
     xitk_mrlbrowser_destroy(mrlb);
     mrlb = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -155,13 +159,15 @@ static void mrl_browser_kill(xitk_widget_t *w, void *data) {
       config_update_num ("gui.mrl_browser_y", wi.y);
       WINDOW_INFO_ZERO(&wi);
     }
+
+    try_to_set_input_focus(gGui->video_window);
   }
 
   /* FIXME */
   mrlb = NULL;
 }
 
-xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
+static xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
   xitk_mrlbrowser_filter_t **filters = NULL;
   int                        num_endings = 0;
   char                      *mrl_exts, *p, *pp;
@@ -240,8 +246,10 @@ void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t play_cb,
 
   XITK_WIDGET_INIT(&mb, gGui->imlib_data);
 
-  mb.window_trans                   = (gGui->use_root_window) ? None : gGui->video_window;
+  mb.window_trans                   = (gGui->use_root_window || gGui->video_display != gGui->display) ? None : gGui->video_window;
   mb.layer_above                    = (is_layer_above());
+  mb.icon                           = &gGui->icon;
+  mb.set_wm_window_normal           = !video_window_is_visible();
 
   mb.x                              = xine_config_register_num (gGui->xine, "gui.mrl_browser_x", 
 								200,
@@ -345,11 +353,11 @@ static void mrl_add_noautoplay(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
 	playlist_toggle_visibility(NULL, NULL);
     }
     
-    mediamark_add_entry((char *)mrl->mrl, (char *)mrl->mrl, NULL, 0, -1, 0, 0);
+    mediamark_append_entry((char *)mrl->mrl, (char *)mrl->mrl, NULL, 0, -1, 0, 0);
     
     if((!num) && ((xine_get_status(gGui->stream) == XINE_STATUS_STOP) || gGui->logo_mode)) {
       gGui->playlist.cur = (gGui->playlist.num - 1);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
     }   
     
     playlist_update_playlist();
@@ -394,7 +402,7 @@ static void mrl_play(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
 
     if(mrl_look_like_playlist(_mrl)) {
       if(mediamark_concat_mediamarks(_mrl)) {
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	gui_set_current_mmk(mediamark_get_current_mmk());
 	_mrl = (char *) mediamark_get_current_mrl();
 	playlist_update_playlist();
       }
@@ -418,7 +426,7 @@ static void mrl_play(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
     mmk.sub   = NULL;
     mmk.start = 0;
     mmk.end   = -1;
-    gui_set_current_mrl(&mmk);
+    gui_set_current_mmk(&mmk);
   }
 }
 
@@ -429,6 +437,11 @@ void mrl_browser_deinit(void) {
   }
 }
 
+void mrl_browser_reparent(void) {
+  if(mrlb)
+    reparent_window((xitk_mrlbrowser_get_window_id(mrlb)));
+}
+
 /*
  * Create a new mrl browser.
  */
