@@ -1,6 +1,6 @@
 #warning IMPLEMENT POST SUPPORT
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: network.c,v 1.56 2003/12/13 01:09:39 f1rmb Exp $
+ * $Id: network.c,v 1.63 2004/05/23 22:31:13 f1rmb Exp $
  *
  * xine network related stuff
  *
@@ -37,6 +37,8 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_READLINE
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -64,10 +66,10 @@
 #include <netdb.h>
 #include <pthread.h>
 
-#include "common.h"
+#include <readline.h>
+#include <history.h>
 
-#include "readline.h"
-#include "history.h"
+#include "common.h"
 
 #ifndef	MSG_NOSIGNAL
 #define	MSG_NOSIGNAL	     0
@@ -160,7 +162,6 @@ typedef struct client_info_s client_info_t;
 typedef struct passwds_s passwds_t;
 typedef void (*cmd_func_t)(commands_t *, client_info_t *);
 
-static pthread_t     thread_server;
 static passwds_t   **passwds = NULL;
 
 static void do_commands(commands_t *, client_info_t *);
@@ -431,7 +432,7 @@ extern gGui_t *gGui;
 
 #endif
 
-void sock_err(const char *error_msg, ...) {
+static void sock_err(const char *error_msg, ...) {
   va_list args;
   
   va_start(args, error_msg);
@@ -439,7 +440,7 @@ void sock_err(const char *error_msg, ...) {
   va_end(args);
 }
 
-int sock_create(const char *service, const char *transport, struct sockaddr_in *sin) {
+static int sock_create(const char *service, const char *transport, struct sockaddr_in *sin) {
   struct servent    *iservice;
   struct protoent *itransport;
   int                sock;
@@ -482,57 +483,10 @@ int sock_create(const char *service, const char *transport, struct sockaddr_in *
   return sock;
 }
 
-int sock_client(const char *host, const char *service, const char *transport) {
-  struct sockaddr_in   sin;
-  struct hostent      *ihost;
-  int                  sock;
-  
-  sock = sock_create(service, transport, &sin);
-
-  if ((sin.sin_addr.s_addr = inet_addr(host)) == -1) {
-    ihost = gethostbyname(host);
-    
-    if(!ihost) {
-      sock_err("Unknown host: %s\n", host);
-      return -1;
-    }
-    memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
-  }
-  
-  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-    int err = errno;
-
-    close(sock);
-    errno = err;
-    sock_err("Unable to connect %s[%s]: %s\n", host, service, strerror(errno));
-    return -1;
-  }
-
-  return sock;
-}
-
-int sock_serv(const char *service, const char *transport, int queue_length) {
-  struct sockaddr_in  sin;
-  int                 sock;
-  int                 on = 1;
-
-  sock = sock_create(service, transport, &sin);
-  
-  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
-  
-  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
-    sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
-  
-  if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
-    sock_err("Passive mode impossible on %s: %s\n", service, strerror(errno));
-  
-  return sock;
-}
-
 /*
  * Check for socket validity.
  */
-int sock_check_opened(int socket) {
+static int sock_check_opened(int socket) {
   fd_set   readfds, writefds, exceptfds;
   int      retval;
   struct   timeval timeout;
@@ -558,46 +512,6 @@ int sock_check_opened(int socket) {
   return 0;
 }
 
-/*
- * Read from socket.
- */
-int sock_read(int socket, char *buf, int len) {
-  char    *pbuf;
-  int      r, rr;
-  void    *nl;
-  
-  if((socket < 0) || (buf == NULL))
-    return -1;
-
-  if(!sock_check_opened(socket))
-    return -1;
-  
-  if (--len < 1)
-    return(-1);
-  
-  pbuf = buf;
-  
-  do {
-    
-    if((r = recv(socket, pbuf, len, MSG_PEEK)) <= 0)
-      return -1;
-
-    if((nl = memchr(pbuf, '\n', r)) != NULL)
-      r = ((char *) nl) - pbuf + 1;
-    
-    if((rr = read(socket, pbuf, r)) < 0)
-      return -1;
-    
-    pbuf += rr;
-    len -= rr;
-
-  } while((nl == NULL) && len);
-  
-  *pbuf = '\0';
-  
-  return (pbuf - buf);
-}
-
 /*
  * Write to socket.
  */
@@ -677,6 +591,34 @@ static char *_atoa(char *str) {
 }
 
 #ifdef NETWORK_CLIENT
+static int sock_client(const char *host, const char *service, const char *transport) {
+  struct sockaddr_in   sin;
+  struct hostent      *ihost;
+  int                  sock;
+  
+  sock = sock_create(service, transport, &sin);
+
+  if ((sin.sin_addr.s_addr = inet_addr(host)) == -1) {
+    ihost = gethostbyname(host);
+    
+    if(!ihost) {
+      sock_err("Unknown host: %s\n", host);
+      return -1;
+    }
+    memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
+  }
+  
+  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
+    int err = errno;
+
+    close(sock);
+    errno = err;
+    sock_err("Unable to connect %s[%s]: %s\n", host, service, strerror(errno));
+    return -1;
+  }
+
+  return sock;
+}
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 91)
 #define write_to_console_unlocked(session, msg, args...)  __sock_write(session->console, 1, msg, ##args)
@@ -798,7 +740,7 @@ static void client_help(session_t *session, session_commands_t *command, const c
   maxlen++; 
   i = 0;  
   
-  sprintf(buf, "%s", "Available commands are:\n       ");
+  snprintf(buf, sizeof(buf), "%s", "Available commands are:\n       ");
   curpos += 7;
   
   while(session_commands[i]->command != NULL) {
@@ -862,7 +804,7 @@ static void client_open(session_t *session, session_commands_t *command, const c
   }
   
   if(cmd) {
-    sprintf(buf, "%s", cmd);
+    snprintf(buf, sizeof(buf), "%s", cmd);
     pbuf = buf;
     if((p = strchr(pbuf, ' ')) != NULL) {
       host = _atoa(p);
@@ -1093,7 +1035,7 @@ static void client_handle_command(session_t *session, const char *command) {
   
   /* Get only first arg of command */
   memset(&cmd, 0, sizeof(cmd));
-  sprintf(cmd, "%s", command);
+  snprintf(cmd, sizeof(cmd), "%s", command);
   if((p = strchr(cmd, ' ')) != NULL)
     *p = '\0';
   
@@ -1172,7 +1114,7 @@ static void session_single_shot(session_t *session, int num_commands, char *comm
     if(strlen(buf))
       sprintf(buf, "%s %s", buf, commands[i]);
     else
-      sprintf(buf, "%s", commands[i]);
+      snprintf(buf, sizeof(buf), "%s", commands[i]);
   }
 
   client_handle_command(session, buf);
@@ -1383,6 +1325,64 @@ int main(int argc, char **argv) {
 
 #else
 
+static int sock_serv(const char *service, const char *transport, int queue_length) {
+  struct sockaddr_in  sin;
+  int                 sock;
+  int                 on = 1;
+
+  sock = sock_create(service, transport, &sin);
+  
+  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
+  
+  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
+    sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
+  
+  if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
+    sock_err("Passive mode impossible on %s: %s\n", service, strerror(errno));
+  
+  return sock;
+}
+
+/*
+ * Read from socket.
+ */
+static int sock_read(int socket, char *buf, int len) {
+  char    *pbuf;
+  int      r, rr;
+  void    *nl;
+  
+  if((socket < 0) || (buf == NULL))
+    return -1;
+
+  if(!sock_check_opened(socket))
+    return -1;
+  
+  if (--len < 1)
+    return(-1);
+  
+  pbuf = buf;
+  
+  do {
+    
+    if((r = recv(socket, pbuf, len, MSG_PEEK)) <= 0)
+      return -1;
+
+    if((nl = memchr(pbuf, '\n', r)) != NULL)
+      r = ((char *) nl) - pbuf + 1;
+    
+    if((rr = read(socket, pbuf, r)) < 0)
+      return -1;
+    
+    pbuf += rr;
+    len -= rr;
+
+  } while((nl == NULL) && len);
+  
+  *pbuf = '\0';
+  
+  return (pbuf - buf);
+}
+
 /*
  * Password related
  */
@@ -1430,7 +1430,7 @@ static int _passwdfile_is_entry_valid(char *entry, char *name, char *passwd) {
   char buf[_BUFSIZ];
   char *n, *p;
   
-  sprintf(buf, "%s", entry);
+  snprintf(buf, sizeof(buf), "%s", entry);
   n = buf;
   if((p = strrchr(buf, ':')) != NULL) {
     if(strlen(p) > 1) {
@@ -1643,7 +1643,7 @@ static const char *get_arg(client_info_t *client_info, int num) {
 /*
  * return 1 if *arg match with argument <pos>
  */
-int is_arg_contain(client_info_t *client_info, int pos, const char *arg) {
+static int is_arg_contain(client_info_t *client_info, int pos, const char *arg) {
   
   if(client_info && pos && ((arg != NULL) && (strlen(arg))) && (client_info->command.num_args >= pos)) {
     if(!strncmp(client_info->command.args[pos - 1], arg, strlen(client_info->command.args[pos - 1])))
@@ -1660,10 +1660,7 @@ static void set_command_line(client_info_t *client_info, char *line) {
 
   if(client_info && line && strlen(line)) {
 
-    if(client_info->command.line)
-      client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
-    else
-      client_info->command.line = (char *) xine_xmalloc(strlen(line) + 1);
+    client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
     
     memset(client_info->command.line, 0, sizeof(client_info->command.line));
     sprintf(client_info->command.line, "%s", line);
@@ -1708,7 +1705,7 @@ static void do_commands(commands_t *cmd, client_info_t *client_info) {
   char buf[_BUFSIZ];
 
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, COMMANDS_PREFIX);
+  snprintf(buf, sizeof(buf), "%s", COMMANDS_PREFIX);
   
   while(commands[i].command != NULL) {
     if(commands[i].public) {
@@ -1741,7 +1738,7 @@ static void do_help(commands_t *cmd, client_info_t *client_info) {
     maxlen++; 
     i = 0;  
 
-    sprintf(buf, "Available commands are:\n       ");
+    snprintf(buf, sizeof(buf), "%s", "Available commands are:\n       ");
     curpos += 7;
     
     while(commands[i].command != NULL) {
@@ -1802,7 +1799,7 @@ static void do_auth(commands_t *cmd, client_info_t *client_info) {
       char *name;
       char *passwd;
       
-      sprintf(buf, "%s", get_arg(client_info, 1));
+      snprintf(buf, sizeof(buf), "%s", get_arg(client_info, 1));
       name = buf;
       if((passwd = strrchr(buf, ':')) != NULL) {
 	if(strlen(passwd) > 1) {
@@ -1846,9 +1843,9 @@ static void do_mrl(commands_t *cmd, client_info_t *client_info) {
 	xine_stop (gGui->stream);
 	gGui->playlist.cur--;
 	if ((gGui->playlist.cur >= 0) && (gGui->playlist.cur < gGui->playlist.num)) {
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	  (void) gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-					gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset);
+					gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1);
   
 	} 
 	else {
@@ -1876,7 +1873,7 @@ static void do_mrl(commands_t *cmd, client_info_t *client_info) {
 	  xine_stop(gGui->stream);
 	  gGui->ignore_next = 0;
 	}
-	gui_set_current_mrl(gGui->playlist.mmk[gGui->playlist.num - 1]);
+	gui_set_current_mmk(gGui->playlist.mmk[gGui->playlist.num - 1]);
 	if(!(xine_open(gGui->stream, gGui->mmk.mrl) 
 	     && xine_play (gGui->stream, 0, gGui->mmk.start))) {
 	  handle_xine_error(client_info);
@@ -1938,7 +1935,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	    }
 
 	    gGui->playlist.cur = entry;
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	    gui_play(NULL, NULL);
 	  }
 	}
@@ -1962,7 +1959,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	
 	if((j >= 0) && (j <= gGui->playlist.num)) {
 	  gGui->playlist.cur = j;
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 
 	  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
 	    gGui->ignore_next = 1;
@@ -2001,8 +1998,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	    for(i = j; i < gGui->playlist.num; i++)
 	      gGui->playlist.mmk[i] = gGui->playlist.mmk[i + 1];
 	    
-	    gGui->playlist.mmk = (mediamark_t **) 
-	      realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
+	    gGui->playlist.mmk = (mediamark_t **) realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
 	    
 	    gGui->playlist.mmk[gGui->playlist.num + 1] = NULL;
 	    
@@ -2014,7 +2010,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	  playlist_update_playlist();
 
 	if(gGui->playlist.num)
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	else {
 	  
 	  if(is_playback_widgets_enabled())
@@ -2023,7 +2019,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
 	    gui_stop(NULL, NULL);
 	  
-	  gui_set_current_mrl(NULL);
+	  gui_set_current_mmk(NULL);
 	}
       }
     }
@@ -2091,7 +2087,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	char buf[64];
 	int  status;
 
-	sprintf(buf, "%s", "Current status: ");
+	snprintf(buf, sizeof(buf), "%s", "Current status: ");
 	status = xine_get_status(gGui->stream);
 	
 	if(status <= XINE_STATUS_QUIT)
@@ -2106,7 +2102,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	char buf[64];
 	int  speed = -1, i;
 	
-	sprintf(buf, "%s", "Current speed: ");
+	snprintf(buf, sizeof(buf), "%s", "Current speed: ");
 	speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
 
 	for(i = 0; speeds_struct[i].name != NULL; i++) {
@@ -2131,7 +2127,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_stream,
 			    &pos_time,
 			    &length_time);
-	sprintf(buf, "Current position: %d\n",pos_time);
+	snprintf(buf, sizeof(buf), "%s: %d\n", "Current position", pos_time);
 	sock_write(client_info->socket, buf);
       }
       else if(is_arg_contain(client_info, 1, "length")) {
@@ -2143,32 +2139,32 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_stream,
 			    &pos_time,
 			    &length_time);
-	sprintf(buf, "Current length: %d\n",length_time);
+	snprintf(buf, sizeof(buf), "%s: %d\n", "Current length", length_time);
 	sock_write(client_info->socket, buf);
       }
       else if(is_arg_contain(client_info, 1, "loop")) {
 	char buf[64];
 
-	sprintf(buf, "Current loop mode is: ");
+	snprintf(buf, sizeof(buf), "%s", "Current loop mode is: ");
 
 	switch(gGui->playlist.loop) {
 	case PLAYLIST_LOOP_NO_LOOP:
-	  sprintf(buf, "%s'No Loop'", buf);
+	  sprintf(buf, "%s%s", buf, "'No Loop'");
 	  break;
 	case PLAYLIST_LOOP_LOOP:
-	  sprintf(buf, "%s'Loop'", buf);
+	  sprintf(buf, "%s%s", buf, "'Loop'");
 	  break;
 	case PLAYLIST_LOOP_REPEAT:
-	  sprintf(buf, "%s'Repeat'", buf);
+	  sprintf(buf, "%s%s", buf, "'Repeat'");
 	  break;
 	case PLAYLIST_LOOP_SHUFFLE:
-	  sprintf(buf, "%s'Shuffle'", buf);
+	  sprintf(buf, "%s%s", buf, "'Shuffle'");
 	  break;
 	case PLAYLIST_LOOP_SHUF_PLUS:	
-	  sprintf(buf, "%s'Shuffle forever'", buf);
+	  sprintf(buf, "%s%s", buf, "'Shuffle forever'");
 	  break;
 	default:
-	  sprintf(buf, "%s'!!Unknown!!'", buf);
+	  sprintf(buf, "%s%s", buf, "'!!Unknown!!'");
 	  break;
 	}
 
@@ -2572,10 +2568,10 @@ static void say_hello(client_info_t *client_info) {
   struct hostent *hp = NULL;
   
   if(!gethostname(myfqdn, 255) && (hp = gethostbyname(myfqdn)) != NULL) {
-    sprintf(buf, "%s %s %s remote server. Nice to meet you.\n", hp->h_name, PACKAGE, VERSION);
+    snprintf(buf, sizeof(buf), "%s %s %s %s\n", hp->h_name, PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
   else {
-    sprintf(buf, "%s %s remote server. Nice to meet you.\n", PACKAGE, VERSION);
+    snprintf(buf, sizeof(buf), "%s %s %s\n", PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
   sock_write(client_info->socket, buf);
   
@@ -2637,13 +2633,9 @@ static void parse_destock_remain(client_info_t *client_info) {
 	p++;
       
       if(p)
-	sprintf(remaining, "%s", (_atoa(p)));
+	snprintf(remaining, sizeof(remaining), "%s", (_atoa(p)));
       
-      if(client_info->command.line)
-	client_info->command.line = (char *) realloc(client_info->command.line, 
-						     sizeof(char *) * (strlen(c) + 1));
-      else
-	client_info->command.line = (char *) xine_xmalloc(sizeof(char *) * (strlen(c) + 1));
+      client_info->command.line = (char *) realloc(client_info->command.line, sizeof(char *) * (strlen(c) + 1));
       
       sprintf(client_info->command.line, "%s", c);
       
@@ -2673,12 +2665,8 @@ static void parse_destock_remain(client_info_t *client_info) {
       
     }
     else { /* no ';' in remain, copy AS IS remain to line */
-      if(client_info->command.line)
-	client_info->command.line = (char *) realloc(client_info->command.line, 
-						     sizeof(char *) * (strlen(client_info->command.remain) + 1));
-      else
-	client_info->command.line = (char *) xine_xmalloc(sizeof(char *) * 
-							  (strlen(client_info->command.remain) + 1));
+      client_info->command.line = (char *) realloc(client_info->command.line, 
+						   sizeof(char *) * (strlen(client_info->command.remain) + 1));
       
       sprintf(client_info->command.line, "%s", client_info->command.remain);
       
@@ -2729,11 +2717,8 @@ static void parse_command(client_info_t *client_info) {
     cmd = NULL;
 
   if(cmd) {
-    if(client_info->command.command)
-      client_info->command.command = (char *) realloc(client_info->command.command, 
-						      strlen(client_info->command.line) - strlen(cmd) + 1);
-    else
-      client_info->command.command = (char *) xine_xmalloc(strlen(client_info->command.line) - strlen(cmd) + 1);
+    client_info->command.command = (char *) realloc(client_info->command.command, 
+						    strlen(client_info->command.line) - strlen(cmd) + 1);
 
     memset(client_info->command.command, 0, sizeof(client_info->command.command));
     snprintf(client_info->command.command, 
@@ -2802,10 +2787,7 @@ static void parse_command(client_info_t *client_info) {
     }
   }
   else {
-    if(client_info->command.command)
-      client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
-    else
-      client_info->command.command = (char *) xine_xmalloc(strlen(client_info->command.line) + 1);
+    client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
     
     memset(client_info->command.command, 0, sizeof(client_info->command.command));
     sprintf(client_info->command.command, "%s", client_info->command.line);
@@ -2919,8 +2901,7 @@ static void *client_thread(void *data) {
   for(i = 0; i < 256; i++)
     SAFE_FREE(client_info->command.args[i]);
   
-  SAFE_FREE(client_info);
-
+  free(client_info);
   client_info = NULL;
 
   pthread_exit(NULL);
@@ -2972,7 +2953,7 @@ static void *server_thread(void *data) {
     char         passwdfilename[(strlen((get_homedir())) + strlen(passwdfile)) + 2];
     struct stat  st;
     
-    sprintf(passwdfilename, "%s/%s", (get_homedir()), passwdfile);
+    snprintf(passwdfilename, sizeof(passwdfilename), "%s/%s", (get_homedir()), passwdfile);
     
     if(stat(passwdfilename, &st) < 0) {
       fprintf(stderr, "ERROR: there is no password file for network access.!\n");
@@ -3001,10 +2982,13 @@ static void *server_thread(void *data) {
     
     if(client_info->socket < 0) {
       
+      free(client_info);
+      
       if(errno == EINTR)
 	continue;
       
       sock_err("accept: %s\n", strerror(errno));
+      continue;
     }
     
     close(msock);
@@ -3034,15 +3018,18 @@ static void *server_thread(void *data) {
  *
  */
 void start_remote_server(void) {
+  pthread_t thread;
 
-  if(gGui->network) {
-    pthread_create(&thread_server, NULL, server_thread, NULL);
-  }
+  if(gGui->network)
+    pthread_create(&thread, NULL, server_thread, NULL);
   
 }
 
 const char *get_homedir(void) {
-  struct passwd pwd, *pw = NULL;
+#ifdef HAVE_GETPWUID_R
+  struct passwd pwd;
+#endif
+  struct passwd *pw = NULL;
   static char homedir[BUFSIZ] = {0,};
 
   if(homedir[0])
@@ -3074,3 +3061,5 @@ const char *get_homedir(void) {
 }
 
 #endif
+
+#endif  /* HAVE_READLINE */
