@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: osd.c,v 1.25 2003/12/08 12:46:02 miguelfreitas Exp $
+ * $Id: osd.c,v 1.31 2004/05/08 21:57:52 f1rmb Exp $
  *
  */
 
@@ -257,7 +257,7 @@ void osd_stream_infos(void) {
     char        buffer[256], *p;
     int         x, y;
     int         w, h, osdw;
-    int         playedtime, totaltime, pos;
+    int         playedtime, playeddays, totaltime, pos;
     int         audiochannel, spuchannel, len;
 
     vcodec       = xine_get_meta_info(gGui->stream, XINE_META_INFO_VIDEOCODEC);
@@ -299,7 +299,7 @@ void osd_stream_infos(void) {
 
     y = x = 0;
 
-    sprintf(buffer, "%s", (gGui->is_display_mrl) ? gGui->mmk.mrl : gGui->mmk.ident);
+    snprintf(buffer, sizeof(buffer), "%s", (gGui->is_display_mrl) ? gGui->mmk.mrl : gGui->mmk.ident);
     xine_osd_get_text_size(gGui->osd.sinfo, buffer, &osdw, &h);
     p = buffer;
     while(osdw > (wwidth - 40)) {
@@ -314,7 +314,7 @@ void osd_stream_infos(void) {
     y += h;
     
     if(vcodec && vwidth && vheight) {
-      sprintf(buffer, "%s: %dX%d", vcodec, vwidth, vheight);
+      snprintf(buffer, sizeof(buffer), "%s: %dX%d", vcodec, vwidth, vheight);
       xine_osd_draw_text(gGui->osd.sinfo, x, y, buffer, XINE_OSD_TEXT1);
       xine_osd_get_text_size(gGui->osd.sinfo, buffer, &w, &h);
       if(w > osdw)
@@ -323,7 +323,7 @@ void osd_stream_infos(void) {
     }
 
     if(acodec && asrate) {
-      sprintf(buffer, "%s: %dHz", acodec, asrate);
+      snprintf(buffer, sizeof(buffer), "%s: %d%s", acodec, asrate, "Hz");
       xine_osd_draw_text(gGui->osd.sinfo, x, y, buffer, XINE_OSD_TEXT1);
       xine_osd_get_text_size(gGui->osd.sinfo, buffer, &w, &h);
       if(w > osdw)
@@ -333,15 +333,15 @@ void osd_stream_infos(void) {
     
     memset(&buffer, 0, sizeof(buffer));
 
-    sprintf(buffer, _("Audio: "));
+    snprintf(buffer, sizeof(buffer), "%s", _("Audio: "));
     len = strlen(buffer);
     switch(audiochannel) {
     case -2:
-      sprintf(buffer, "%soff", buffer);
+      sprintf(buffer, "%s%s", buffer, "off");
       break;
     case -1:
       if(!xine_get_audio_lang (gGui->stream, audiochannel, &buffer[len]))
-	sprintf(buffer, "%sauto", buffer);
+	sprintf(buffer, "%s%s", buffer, "auto");
       break;
     default:
       if(!xine_get_audio_lang (gGui->stream, audiochannel, &buffer[len]))
@@ -349,15 +349,15 @@ void osd_stream_infos(void) {
       break;
     }
 
-    sprintf(buffer, "%s, Spu: ", buffer);
+    sprintf(buffer, "%s%s", buffer, ", Spu: ");
     len = strlen(buffer);
     switch (spuchannel) {
     case -2:
-      sprintf(buffer, "%soff", buffer);
+      sprintf(buffer, "%s%s", buffer, "off");
       break;
     case -1:
       if(!xine_get_spu_lang (gGui->stream, spuchannel, &buffer[len]))
-	sprintf(buffer, "%sauto", buffer);
+	sprintf(buffer, "%s%s", buffer, "auto");
       break;
     default:
       if(!xine_get_spu_lang (gGui->stream, spuchannel, &buffer[len]))
@@ -372,19 +372,32 @@ void osd_stream_infos(void) {
     
     y += (h);
 
-    if(totaltime) {
-      sprintf(buffer, _("%d:%.2d:%.2d (%.0f%%) of %d:%.2d:%.2d"),
-	      playedtime / 3600, (playedtime % 3600) / 60, playedtime % 60,
-	      ((float)playedtime / (float)totaltime) * 100,
-	      totaltime / 3600, (totaltime % 3600) / 60, totaltime % 60);
-      xine_osd_draw_text(gGui->osd.sinfo, x, y, buffer, XINE_OSD_TEXT1);
-      xine_osd_get_text_size(gGui->osd.sinfo, buffer, &w, &h);
-      if(w > osdw)
-	osdw = w;
-
-      osd_stream_position(pos);
+    playeddays = playedtime / (3600 * 24);
+    
+    if(playeddays > 0)
+      sprintf(buffer, "%d::%02d ", playeddays, playedtime / 3600);
+    else
+      sprintf(buffer, "%d:%02d:%02d ", playedtime / 3600, (playedtime % 3600) / 60, playedtime % 60);
+    
+    if(totaltime > 0) {
+      int totaldays;
+      
+      totaldays  = totaltime / (3600 * 24);
+      sprintf(buffer, "%s(%.0f%%) %s ", buffer, ((float)playedtime / (float)totaltime) * 100, _("of"));
+      
+      if(totaldays > 0)
+	sprintf(buffer, "%s%d::%02d", buffer, totaldays, totaltime / 3600);
+      else
+	sprintf(buffer, "%s%d:%02d:%02d", buffer, totaltime / 3600, (totaltime % 3600) / 60, totaltime % 60);
     }
     
+    xine_osd_draw_text(gGui->osd.sinfo, x, y, buffer, XINE_OSD_TEXT1);
+    xine_osd_get_text_size(gGui->osd.sinfo, buffer, &w, &h);
+    if(w > osdw)
+      osdw = w;
+    
+    osd_stream_position(pos);
+    
     x = (wwidth - osdw) - 40;
     xine_osd_set_position(gGui->osd.sinfo, (x >= 0) ? x : 0, 15);
 
@@ -575,7 +588,7 @@ void osd_update_status(void) {
     switch(status) {
     case XINE_STATUS_IDLE:
     case XINE_STATUS_STOP:
-      sprintf(buffer, "%s", (_osd_get_status_sym(status)));
+      snprintf(buffer, sizeof(buffer), "%s", (_osd_get_status_sym(status)));
       break;
       
     case XINE_STATUS_PLAY:
@@ -586,11 +599,11 @@ void osd_update_status(void) {
 	if(gui_xine_get_pos_length(gGui->stream, NULL, &secs, NULL)) {
 	  secs /= 1000;
 	  
-	  sprintf(buffer, "%s %02d:%02d:%02d", (_osd_get_speed_sym(speed)), 
-		  secs / (60*60), (secs / 60) % 60, secs % 60);
+	  snprintf(buffer, sizeof(buffer), "%s %02d:%02d:%02d", (_osd_get_speed_sym(speed)), 
+		   secs / (60*60), (secs / 60) % 60, secs % 60);
 	}
 	else
-	  sprintf(buffer, "%s", (_osd_get_speed_sym(speed)));
+	  snprintf(buffer, sizeof(buffer), "%s", (_osd_get_speed_sym(speed)));
 	
       }
       break;
@@ -604,8 +617,13 @@ void osd_update_status(void) {
       xine_osd_set_font(gGui->osd.status, "cetus", UNSCALED_FONT_SIZE);
     else
       xine_osd_set_font(gGui->osd.status, "cetus", FONT_SIZE);
-    
+
+    /* set latin1 encoding (NULL) for status text with special characters,
+     * then switch back to locale encoding ("") 
+     */
+    xine_osd_set_encoding(gGui->osd.status, NULL);
     xine_osd_draw_text(gGui->osd.status, 0, 0, buffer, XINE_OSD_TEXT1);
+    xine_osd_set_encoding(gGui->osd.status, "");
     xine_osd_set_position(gGui->osd.status, 20, 10);
 
     _osd_get_output_size(&wwidth, &wheight);
@@ -621,7 +639,7 @@ void osd_update_status(void) {
 }
 
 void osd_display_spu_lang(void) {
-  char   buffer[32];
+  char   buffer[XINE_LANG_MAX+128];
   char   lang_buffer[XINE_LANG_MAX];
   char  *lang = NULL;
   int    channel;
@@ -640,17 +658,17 @@ void osd_display_spu_lang(void) {
     break;
   default:
     if(!xine_get_spu_lang(gGui->stream, channel, &lang_buffer[0]))
-      sprintf(lang_buffer, "%3d", channel);
+      snprintf(lang_buffer, sizeof(lang_buffer), "%3d", channel);
     lang = lang_buffer;
     break;
   }
   
-  sprintf(buffer, _("Subtitles: %s"), get_language_from_iso639_1(lang));
+  snprintf(buffer, sizeof(buffer), "%s%s", _("Subtitles: "), get_language_from_iso639_1(lang));
   osd_display_info(buffer);
 }
 
 void osd_display_audio_lang(void) {
-  char   buffer[32];
+  char   buffer[XINE_LANG_MAX+128];
   char   lang_buffer[XINE_LANG_MAX];
   char  *lang = NULL;
   int    channel;
@@ -669,11 +687,11 @@ void osd_display_audio_lang(void) {
     break;
   default:
     if(!xine_get_audio_lang(gGui->stream, channel, &lang_buffer[0]))
-      sprintf(lang_buffer, "%3d", channel);
+      snprintf(lang_buffer, sizeof(lang_buffer), "%3d", channel);
     lang = lang_buffer;
     break;
   }
 
-  sprintf(buffer, _("Audio Channel: %s"), get_language_from_iso639_1(lang));
+  snprintf(buffer, sizeof(buffer), "%s%s", _("Audio Channel: "), get_language_from_iso639_1(lang));
   osd_display_info(buffer);
 }
