@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.197 2003/11/18 16:48:22 f1rmb Exp $
+ * $Id: panel.c,v 1.212 2004/07/04 16:14:35 miguelfreitas Exp $
  *
  * xine panel related stuff
  *
@@ -39,7 +39,7 @@
 
 extern gGui_t     *gGui;
 
-_panel_t          *panel;
+_panel_t          *panel = NULL;
 
 void panel_update_nextprev_tips(void) {
 
@@ -132,6 +132,46 @@ static void panel_store_new_position(int x, int y, int w, int h) {
 
 }
 
+static void panel_exit(xitk_widget_t *w, void *data) {
+
+  if(panel) {
+    window_info_t wi;
+    
+    panel->visible = 0;
+
+    if((xitk_get_window_info(panel->widget_key, &wi))) {
+      config_update_num ("gui.panel_x", wi.x);
+      config_update_num ("gui.panel_y", wi.y);
+      WINDOW_INFO_ZERO(&wi);
+    }
+
+    xitk_unregister_event_handler(&panel->widget_key);
+
+    XLockDisplay(gGui->display);
+    XUnmapWindow(gGui->display, gGui->panel_window);
+    XUnlockDisplay(gGui->display);
+
+    xitk_destroy_widgets(panel->widget_list);
+
+    XLockDisplay(gGui->display);
+    XDestroyWindow(gGui->display, gGui->panel_window);
+    Imlib_destroy_image(gGui->imlib_data, panel->bg_image);
+    XUnlockDisplay(gGui->display);
+
+    gGui->panel_window = None;
+    xitk_list_free((XITK_WIDGET_LIST_LIST(panel->widget_list)));
+
+    XLockDisplay(gGui->display);
+    XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(panel->widget_list)));
+    XUnlockDisplay(gGui->display);
+
+    free(panel->widget_list);
+    
+    free(panel);
+    panel = NULL;  
+  }
+}
+
 /*
  * Change the current skin.
  */
@@ -173,14 +213,17 @@ void panel_change_skins(void) {
 
   XLockDisplay(gGui->display);
 
-  if(!gGui->use_root_window)
+  if(!gGui->use_root_window && gGui->video_display == gGui->display)
     XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
 
   Imlib_destroy_image(gGui->imlib_data, old_img);
   Imlib_apply_image(gGui->imlib_data, new_img, gGui->panel_window);
   
   XUnlockDisplay(gGui->display);
-  
+
+  if(panel_is_visible())
+    raise_window(gGui->panel_window, 1, 1);
+
   xitk_skin_unlock(gGui->skin_config);
   
   xitk_change_skins_widget_list(panel->widget_list, gGui->skin_config);
@@ -189,16 +232,17 @@ void panel_change_skins(void) {
    * Update position of dynamic buttons.
    */
   {
-    int i = 0, x, y, dir;
+    int i = 0, x, y, dir, max;
     
-    x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
-    y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
+    x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
+    y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
     dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayGUI");
+    max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayGUI");
 
     switch(dir) {
     case DIRECTION_UP:
     case DIRECTION_DOWN:
-      while(panel->autoplay_plugins[i] != NULL) {
+      while((max && ((i < max) && (panel->autoplay_plugins[i] != NULL))) || (!max && panel->autoplay_plugins[i] != NULL)) {
 	
 	(void) xitk_set_widget_pos(panel->autoplay_plugins[i], x, y);
 	
@@ -212,7 +256,7 @@ void panel_change_skins(void) {
       break;
     case DIRECTION_LEFT:
     case DIRECTION_RIGHT:
-      while(panel->autoplay_plugins[i] != NULL) {
+      while((max && ((i < max) && (panel->autoplay_plugins[i] != NULL))) || (!max && panel->autoplay_plugins[i] != NULL)) {
 	
 	(void) xitk_set_widget_pos(panel->autoplay_plugins[i], x, y);
 	
@@ -225,8 +269,15 @@ void panel_change_skins(void) {
       }
       break;
     }
-  }
 
+    if(max) {
+      while(panel->autoplay_plugins[i] != NULL) {
+	xitk_disable_and_hide_widget(panel->autoplay_plugins[i]);
+	i++;
+      }
+    }
+  }
+  
   enable_playback_controls(panel->playback_widgets.enabled);
   xitk_paint_widget_list(panel->widget_list);
 
@@ -254,25 +305,46 @@ void panel_update_runtime_display(void) {
   }
 
   if(pos || seconds) {
-    remain = (length - seconds) / 1000;
+    int days, rdays;
+    
+    remain  = (length - seconds) / 1000;
     seconds /= 1000;
-    
-    if(panel->runtime_mode == 0)
-      sprintf(timestr, "%02d:%02d:%02d", seconds / (60*60), (seconds / 60) % 60, seconds % 60);
+    days    = seconds / (60*60*24);
+    rdays   = remain / (60*60*24);
+
+    if(panel->runtime_mode == 0) {
+      if(days > 0)
+	snprintf(timestr, sizeof(timestr), "% 4d::%02d", days, seconds / (60*60));
+      else
+	snprintf(timestr, sizeof(timestr), "%02d:%02d:%02d", seconds / (60*60), (seconds / 60) % 60, seconds % 60);
+    }
     else
-      sprintf(timestr, "%02d:%02d:%02d", remain / (60*60), (remain / 60) % 60, remain % 60);
+      if(remain < 0)
+	snprintf(timestr, sizeof(timestr), "%s", "  :  :  ");
+      else {
+	if(rdays > 0)
+	  snprintf(timestr, sizeof(timestr), "% 4d::%02d", rdays, remain / (60*60));
+	else
+	  snprintf(timestr, sizeof(timestr), "%02d:%02d:%02d", remain / (60*60), (remain / 60) % 60, remain % 60);
+      }
     
   }
   else
-    sprintf(timestr, "%s", "--:--:--");
+    snprintf(timestr, sizeof(timestr), "%s", "--:--:--");
   
-  if(length) {
+  if(length > 0) {
+    int days;
+
     length /= 1000;
-    sprintf(buffer, _("Total time: %02d:%02d:%02d"), 
-	    length / (60*60), (length / 60) % 60, length % 60);
+    days = length / (60*60*24);
+    
+    if(days > 0)
+      snprintf(buffer, sizeof(buffer), "%s% 4d::%02d", _("Total time: "), days, length / (60*60));
+    else
+      snprintf(buffer, sizeof(buffer), "%s%02d:%02d:%02d", _("Total time: "), length / (60*60), (length / 60) % 60, length % 60);
   }
   else
-    sprintf(buffer, "%s", _("Total time: --:--:--"));
+    snprintf(buffer, sizeof(buffer), "%s%s", _("Total time: "), "??:??:??");
   
   xitk_set_widget_tips(panel->runtime_label, buffer);
   
@@ -343,9 +415,10 @@ static void *slider_loop(void *dummy) {
 	    panel_update_mrl_display();
 	}
       }
+      
+      if(((status == XINE_STATUS_PLAY) && (speed != XINE_SPEED_PAUSE)) && 
+	 (!xitk_is_window_iconified(gGui->video_display, gGui->video_window))) {
 
-      if((status == XINE_STATUS_PLAY) && (speed != XINE_SPEED_PAUSE)) {
-	
 	if(gGui->ssaver_timeout) {
 	  
 	  if(!(i % 2))
@@ -397,11 +470,6 @@ static void *slider_loop(void *dummy) {
 	}
       }
       
-      if(gGui->got_reference_stream) {
-	gGui->got_reference_stream--;
-	gui_play(NULL, NULL);
-      }
-
     }
     
     if(gGui->cursor_visible) {
@@ -524,7 +592,7 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
       help_toggle_visibility(NULL, NULL);
   }
 
-  if (panel->visible) {
+  if (panel->visible && gGui->video_display == gGui->display) {
     
     XLockDisplay(gGui->display);
     
@@ -542,12 +610,15 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
 	xitk_hide_widgets(panel->widget_list);
       }
     }
+    else
+      XIconifyWindow(gGui->display, gGui->panel_window, (XDefaultScreen(gGui->display)));
 
     if(gGui->cursor_grabbed)
-       XGrabPointer(gGui->display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
+      XGrabPointer(gGui->video_display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
     
     /* Give focus to video output window */
     try_to_set_input_focus(gGui->video_window);
+    
     XUnlockDisplay(gGui->display);
      
   }
@@ -560,7 +631,7 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
     
     XRaiseWindow(gGui->display, gGui->panel_window); 
     XMapWindow(gGui->display, gGui->panel_window); 
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint (gGui->display,  gGui->panel_window, gGui->video_window);
     XUnlockDisplay (gGui->display);
 
@@ -692,7 +763,7 @@ void panel_update_channel_display (void) {
 
   default:
     if(!xine_get_audio_lang (gGui->stream, channel, &buffer[0]))
-      sprintf(buffer, "%3d", channel);
+      snprintf(buffer, sizeof(buffer), "%3d", channel);
     lang = buffer;
     break;
   }
@@ -714,7 +785,7 @@ void panel_update_channel_display (void) {
 
   default:
     if(!xine_get_spu_lang (gGui->stream, channel, &buffer[0]))
-      sprintf(buffer, "%3d", channel);
+      snprintf(buffer, sizeof(buffer), "%3d", channel);
     lang = buffer;
     break;
   }
@@ -754,7 +825,12 @@ void panel_update_mrl_display (void) {
  */
 void panel_toggle_audio_mute(xitk_widget_t *w, void *data, int state) {
 
-  if(gGui->mixer.caps & MIXER_CAP_MUTE) {
+  if(gGui->mixer.method == SOFTWARE_MIXER) {
+    gGui->mixer.mute = state;
+    xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_MUTE, gGui->mixer.mute);
+    osd_display_info(_("Amp: %s"), gGui->mixer.mute ? _("Muted") : _("Unmuted"));
+  }
+  else if(gGui->mixer.caps & MIXER_CAP_MUTE) {
     gGui->mixer.mute = state;
     xine_set_param(gGui->stream, XINE_PARAM_AUDIO_MUTE, gGui->mixer.mute);
     osd_display_info(_("Audio: %s"), gGui->mixer.mute ? _("Muted") : _("Unmuted"));
@@ -812,7 +888,7 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 /*
  * Handle X events here.
  */
-void panel_handle_event(XEvent *event, void *data) {
+static void panel_handle_event(XEvent *event, void *data) {
 
   switch(event->type) {
   case ButtonPress:
@@ -846,17 +922,18 @@ void panel_handle_event(XEvent *event, void *data) {
  */
 void panel_add_autoplay_buttons(void) {
   int                        x, y, dir;
-  int                        i = 0;
+  int                        i = 0, max;
   xitk_labelbutton_widget_t  lb;
   const char *const         *autoplay_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
   const char                *autoplay_label;
   
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
 
-  x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
-  y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
+  x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayGUI");
+  y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayGUI");
   dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayGUI");
-  
+  max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayGUI");
+
   if(autoplay_plugins) {
     autoplay_label = *autoplay_plugins++;
     while(autoplay_label) {
@@ -895,7 +972,10 @@ void panel_add_autoplay_buttons(void) {
 	  x -= (xitk_get_widget_width(panel->autoplay_plugins[i]) + 1);
 	break;
       }
-
+      
+      if(max && (i >= max))
+	xitk_disable_and_hide_widget(panel->autoplay_plugins[i]);
+      
       i++;
       
       autoplay_label = *autoplay_plugins++;
@@ -957,7 +1037,13 @@ void panel_add_mixer_control(void) {
 void panel_deinit(void) {
   if(panel_is_visible())
     _panel_toggle_visibility(NULL, NULL);
-  xitk_unregister_event_handler(&panel->widget_key);
+  panel_exit(NULL, NULL);
+}
+
+void panel_reparent(void) {
+  if(panel) {
+    reparent_window(gGui->panel_window);
+  }
 }
 
 /*
@@ -1057,13 +1143,17 @@ void panel_init (void) {
 				      CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				      &attr);
 
-  XSetStandardProperties(gGui->display, gGui->panel_window, title, title,
-			 None, NULL, 0, &hint);
+  XmbSetWMProperties(gGui->display, gGui->panel_window, title, title, NULL, 0,
+                     &hint, NULL, NULL);
 
   XSelectInput(gGui->display, gGui->panel_window, INPUT_MOTION | KeymapStateMask);
-
   XUnlockDisplay(gGui->display);
-
+  
+  if(!video_window_is_visible())
+    xitk_set_wm_window_type(gGui->panel_window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(gGui->panel_window, WINDOW_TYPE_NORMAL);
+  
   if(is_layer_above())
     xitk_set_layer_above(gGui->panel_window);
   
@@ -1080,7 +1170,7 @@ void panel_init (void) {
                   PropModeReplace, (unsigned char *) &mwmhints,
                   PROP_MWM_HINTS_ELEMENTS);
   
-  if(!gGui->use_root_window)
+  if(!gGui->use_root_window && gGui->video_display == gGui->display)
     XSetTransientForHint (gGui->display, gGui->panel_window, gGui->video_window);
 
   /* 
@@ -1091,6 +1181,7 @@ void panel_init (void) {
     xclasshint->res_name = title;
     xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, gGui->panel_window, xclasshint);
+    XFree(xclasshint);
   }
 
   wm_hint = XAllocWMHints();
@@ -1390,13 +1481,17 @@ void panel_init (void) {
   /* 
    * show panel 
    */
-  panel->visible = xine_config_register_bool (gGui->xine, "gui.panel_visible", 
-					      1,
-					      _("gui panel visibility"),
-					      CONFIG_NO_HELP,
-					      CONFIG_LEVEL_DEB,
-					      CONFIG_NO_CB,
-					      CONFIG_NO_DATA);
+  {
+    int visible = xine_config_register_bool(gGui->xine, "gui.panel_visible", 
+					    1,
+					    _("gui panel visibility"),
+					    CONFIG_NO_HELP,
+					    CONFIG_LEVEL_DEB,
+					    CONFIG_NO_CB,
+					    CONFIG_NO_DATA);
+
+      panel->visible = (!gGui->no_gui) ? visible : 0;
+  }
   
   /*  The user don't want panel on startup */
   if(panel->visible && (actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_VISIBLITY))) {
@@ -1404,7 +1499,7 @@ void panel_init (void) {
     config_update_num ("gui.panel_visible", panel->visible);
   }
   
-  if(gGui->use_root_window && (!panel->visible))
+  if((gGui->use_root_window || gGui->video_display != gGui->display) && (!panel->visible))
     panel->visible = 1;
   
   if (panel->visible) {
