@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.h,v 1.20 2002/07/17 21:41:51 f1rmb Exp $
+ * $Id: panel.h,v 1.30 2004/04/13 19:55:48 f1rmb Exp $
  *
  * xine panel related stuff
  *
@@ -26,42 +26,50 @@
 #ifndef PANEL_H
 #define PANEL_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <pthread.h>
+
 #include "xitk.h"
 
 typedef struct {
   xitk_widget_list_t   *widget_list;
   
+  int                   x;
+  int                   y;
+  int                   skin_on_change;
+
   xitk_widget_t        *title_label;
   xitk_widget_t        *runtime_label;
+  int                   runtime_mode;
 
   struct {
-    int                enabled;
-    xitk_widget_t     *prev;
-    xitk_widget_t     *stop;
-    xitk_widget_t     *play;
-    xitk_widget_t     *pause;
-    xitk_widget_t     *next;
-    xitk_widget_t     *eject;
-    xitk_widget_t     *slider_play;
+    int                 enabled;
+    xitk_widget_t      *prev;
+    xitk_widget_t      *stop;
+    xitk_widget_t      *play;
+    xitk_widget_t      *pause;
+    xitk_widget_t      *next;
+    xitk_widget_t      *eject;
+    xitk_widget_t      *slider_play;
   } playback_widgets;
   
   struct {
-    xitk_widget_t        *slider;
-    xitk_widget_t        *mute;
+    xitk_widget_t      *slider;
+    xitk_widget_t      *mute;
   } mixer;
   
   struct {
-    int                   enable;
-    unsigned int          timeout;
+    int                 enable;
+    unsigned int        timeout;
   } tips;
   
   int                   visible;
   char                  runtime[20];
-  char                  audiochan[20];
   xitk_widget_t        *audiochan_label;
   xitk_widget_t        *autoplay_plugins[64];
-  char                  spuid[20];
   xitk_widget_t        *spuid_label;
   ImlibImage           *bg_image;
   xitk_register_key_t   widget_key;
@@ -78,6 +86,7 @@ void panel_show_tips(void);
 int panel_get_tips_enable(void);
 unsigned long panel_get_tips_timeout(void);
 
+void panel_deinit(void);
 void panel_init (void);
 
 void panel_change_skins(void);
@@ -100,18 +109,18 @@ void panel_check_pause(void);
 
 void panel_check_mute(void);
 
+void panel_reset_runtime_label(void);
+
 void panel_reset_slider (void);
 
 void panel_update_channel_display (void);
 
 void panel_update_runtime_display(void);
 
-void panel_set_no_mrl(void);
-
 void panel_update_mrl_display (void);
 
-void panel_layer_above(int);
-
 void panel_set_title(char*);
 
+void panel_reparent(void);
+
 #endif
