@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.c,v 1.134 2003/12/01 00:46:01 f1rmb Exp $
+ * $Id: playlist.c,v 1.154 2004/07/02 21:41:05 f1rmb Exp $
  *
  * Playlist Editor
  *
@@ -66,9 +66,11 @@ typedef struct {
   char                **playlist_mrls;
   char                **playlist_idents;
   int                   playlist_len;
+
 } _playlist_t;
 
-static _playlist_t   *playlist;
+static filebrowser_t   *load_fb = NULL, *save_fb = NULL;
+static _playlist_t     *playlist;
 
 void playlist_handle_event(XEvent *event, void *data);
 
@@ -108,11 +110,11 @@ static void _playlist_update_browser_list(int start) {
 
   if(gGui->is_display_mrl)
     xitk_browser_update_list(playlist->playlist, 
-			     (const char **)playlist->playlist_mrls, 
+			     (const char **)playlist->playlist_mrls, NULL, 
 			     playlist->playlist_len, start);
   else
     xitk_browser_update_list(playlist->playlist, 
-			     (const char **)playlist->playlist_idents, 
+			     (const char **)playlist->playlist_idents, NULL,
 			     playlist->playlist_len, start);
 
   if((sel >= 0) && ((start == -1) || (old_start == start)))
@@ -170,51 +172,22 @@ static void _playlist_handle_selection(xitk_widget_t *w, void *data, int selecte
 }
 
 static void _playlist_xine_play(void) {
-  const mediamark_t *mmk;
+  gui_set_current_mmk(mediamark_get_current_mmk());
 
-  if((mmk = mediamark_get_current_mmk()) != NULL) {
+  if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+                             gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+                             !mediamark_have_alternates(&(gGui->mmk)))) {
     
-    if((xine_get_status(gGui->stream) != XINE_STATUS_STOP)) {
-      gGui->ignore_next = 1;
-      xine_stop(gGui->stream);
-      gGui->ignore_next = 0;
-    }
-    
-    if(!is_playback_widgets_enabled())
-      enable_playback_controls(1);
-
-    if(mrl_look_like_playlist(mmk->mrl)) {
-      if(mediamark_concat_mediamarks(mmk->mrl)) {
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-	mmk = mediamark_get_current_mmk();
-	playlist_update_playlist();
-      }
-    }
-
-    if(!xine_open(gGui->stream, mmk->mrl)) {
-      gui_handle_xine_error(gGui->stream, mmk->mrl);
-      enable_playback_controls(0);
+    if(!mediamark_have_alternates(&(gGui->mmk)) ||
+        !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+  
+      if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+	gui_exit(NULL, NULL);
+      
       gui_display_logo();
-      return;
-    }
-
-    if(mmk->sub) {
-      if(xine_open(gGui->spu_stream, mmk->sub))
-	xine_stream_master_slave(gGui->stream, 
-				 gGui->spu_stream, XINE_MASTER_SLAVE_PLAY | XINE_MASTER_SLAVE_STOP);
-    }
-    else
-      xine_close(gGui->spu_stream);
-    
-    if(!gui_xine_play(gGui->stream, 0, mmk->start, 0)) {
-      enable_playback_controls(0);
-      gui_display_logo();
-      return;
     }
   }
-
 }
-
 /*
  * Start playing an MRL
  */
@@ -224,9 +197,7 @@ void playlist_play_current(xitk_widget_t *w, void *data) {
   j = xitk_browser_get_current_selected(playlist->playlist);
 
   if((j >= 0) && (gGui->playlist.mmk[j]->mrl != NULL)) {
-    
     gGui->playlist.cur = j;
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
     _playlist_xine_play();
     xitk_browser_release_all_buttons(playlist->playlist);
   }
@@ -237,9 +208,7 @@ void playlist_play_current(xitk_widget_t *w, void *data) {
 static void _playlist_play_on_dbl_click(xitk_widget_t *w, void *data, int selected) {
   
   if(gGui->playlist.mmk[selected]->mrl != NULL) {
-    
     gGui->playlist.cur = selected;
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
     _playlist_xine_play();
     xitk_browser_release_all_buttons(playlist->playlist);
   }
@@ -263,8 +232,7 @@ void playlist_delete_current(xitk_widget_t *w, void *data) {
     for(i = j; i < gGui->playlist.num; i++)
       gGui->playlist.mmk[i] = gGui->playlist.mmk[i + 1];
     
-    gGui->playlist.mmk = (mediamark_t **) 
-      realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
+    gGui->playlist.mmk = (mediamark_t **) realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
     
     gGui->playlist.mmk[gGui->playlist.num] = NULL;
     
@@ -273,7 +241,7 @@ void playlist_delete_current(xitk_widget_t *w, void *data) {
     playlist_update_playlist();
     
     if(gGui->playlist.num)
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
     else {
       
       if(is_playback_widgets_enabled())
@@ -282,7 +250,7 @@ void playlist_delete_current(xitk_widget_t *w, void *data) {
       if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
 	gui_stop(NULL, NULL);
       
-      gui_set_current_mrl(NULL);
+      gui_set_current_mmk(NULL);
       xitk_inputtext_change_text(playlist->winput, NULL);
     }
   }
@@ -304,9 +272,11 @@ void playlist_delete_all(xitk_widget_t *w, void *data) {
   
   if(playlist && playlist->winput)
     xitk_inputtext_change_text(playlist->winput, NULL);
-  
-  xitk_browser_release_all_buttons(playlist->playlist);
-  gui_set_current_mrl(NULL);
+
+  if(playlist)
+    xitk_browser_release_all_buttons(playlist->playlist);
+
+  gui_set_current_mmk(NULL);
   enable_playback_controls(0);
 }
 
@@ -369,7 +339,7 @@ static void _playlist_load_callback(filebrowser_t *fb) {
 
   if((file = filebrowser_get_full_filename(fb)) != NULL) {
     mediamark_load_mediamarks(file);
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+    gui_set_current_mmk(mediamark_get_current_mmk());
     playlist_update_playlist();
 
     if((xine_get_status(gGui->stream) == XINE_STATUS_PLAY))
@@ -380,24 +350,34 @@ static void _playlist_load_callback(filebrowser_t *fb) {
 
     free(file);
   }
+  load_fb = NULL;
   playlist_reactivate();
 }
 static void _playlist_cancel_callback(filebrowser_t *fb) {
+  if(fb == load_fb)
+    load_fb = NULL;
+  else if(fb == save_fb)
+    save_fb = NULL;
+
   playlist_reactivate();
 }
 void playlist_load_playlist(xitk_widget_t *w, void *data) {
   filebrowser_callback_button_t  cbb[2];
   char                           buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
-  sprintf(buffer, "%s/.xine/playlist.tox", xine_get_homedir());
-  
-  cbb[0].label = _("Load");
-  cbb[0].callback = _playlist_load_callback;
-  cbb[0].need_a_file = 1;
-  cbb[1].callback = _playlist_cancel_callback;
-  
-  playlist_deactivate();
-  (void *) create_filebrowser(_("Load a playlist"), buffer, &cbb[0], NULL, &cbb[1]);
+  if(load_fb)
+    filebrowser_raise_window(load_fb);
+  else {
+    snprintf(buffer, sizeof(buffer), "%s%s", xine_get_homedir(), "/.xine/playlist.tox");
+    
+    cbb[0].label = _("Load");
+    cbb[0].callback = _playlist_load_callback;
+    cbb[0].need_a_file = 1;
+    cbb[1].callback = _playlist_cancel_callback;
+    
+    playlist_deactivate();
+    load_fb = create_filebrowser(_("Load a playlist"), buffer, hidden_file_cb, &cbb[0], NULL, &cbb[1]);
+  }
 }
 
 /*
@@ -410,6 +390,7 @@ static void _playlist_save_callback(filebrowser_t *fb) {
     mediamark_save_mediamarks(file);
     free(file);
   }
+  save_fb = NULL;
   playlist_reactivate();
 }
 void playlist_save_playlist(xitk_widget_t *w, void *data) {
@@ -417,15 +398,19 @@ void playlist_save_playlist(xitk_widget_t *w, void *data) {
   char                           buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
   if(gGui->playlist.num) {
-    sprintf(buffer, "%s/.xine/playlist.tox", xine_get_homedir());
-    
-    cbb[0].label = _("Save");
-    cbb[0].callback = _playlist_save_callback;
-    cbb[0].need_a_file = 1;
-    cbb[1].callback = _playlist_cancel_callback;
-  
-    playlist_deactivate();
-    (void *) create_filebrowser(_("Save a playlist"), buffer, &cbb[0], NULL, &cbb[1]);
+    if(save_fb)
+      filebrowser_raise_window(save_fb);
+    else {
+      snprintf(buffer, sizeof(buffer), "%s%s", xine_get_homedir(), "/.xine/playlist.tox");
+      
+      cbb[0].label = _("Save");
+      cbb[0].callback = _playlist_save_callback;
+      cbb[0].need_a_file = 1;
+      cbb[1].callback = _playlist_cancel_callback;
+      
+      playlist_deactivate();
+      save_fb = create_filebrowser(_("Save a playlist"), buffer, hidden_file_cb, &cbb[0], NULL, &cbb[1]);
+    }
   }
 }
 
@@ -443,10 +428,6 @@ static void _playlist_add_input(xitk_widget_t *w, void *data, char *filename) {
  * Handle X events here.
  */
 static void _playlist_handle_event(XEvent *event, void *data) {
-  XKeyEvent      mykeyevent;
-  KeySym         mykey;
-  char           kbuf[256];
-  int            len;
 
   switch(event->type) {
 
@@ -483,51 +464,54 @@ static void _playlist_handle_event(XEvent *event, void *data) {
     break;
 
   case KeyPress:
-    
-    if(!playlist)
-      return;
-    
-    mykeyevent = event->xkey;
-    
-    XLockDisplay(gGui->display);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUnlockDisplay(gGui->display);
-    
-    switch (mykey) {
+    {
+      KeySym mkey;
       
-    case XK_Down:
-    case XK_Next: {
-      xitk_widget_t *w;
-      
-      mmk_editor_end();
-      w = xitk_get_focused_widget(playlist->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Down)
-	  xitk_browser_step_up(playlist->playlist, NULL);
-	else
-	  xitk_browser_page_up(playlist->playlist, NULL);
-      }
-    }
-      break;
-      
-    case XK_Up:
-    case XK_Prior: {
-      xitk_widget_t *w;
+      if(!playlist)
+	return;
 
-      mmk_editor_end();
-      w = xitk_get_focused_widget(playlist->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Up)
-	  xitk_browser_step_down(playlist->playlist, NULL);
-	else
-	  xitk_browser_page_down(playlist->playlist, NULL);
+      mkey = xitk_get_key_pressed(event);
+
+      switch (mkey) {
+	
+      case XK_Down:
+      case XK_Next: {
+	xitk_widget_t *w;
+	
+	mmk_editor_end();
+	w = xitk_get_focused_widget(playlist->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Down)
+	    xitk_browser_step_up(playlist->playlist, NULL);
+	  else
+	    xitk_browser_page_up(playlist->playlist, NULL);
+	}
+      }
+	break;
+	
+      case XK_Up:
+      case XK_Prior: {
+	xitk_widget_t *w;
+	
+	mmk_editor_end();
+	w = xitk_get_focused_widget(playlist->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Up)
+	    xitk_browser_step_down(playlist->playlist, NULL);
+	  else
+	    xitk_browser_page_down(playlist->playlist, NULL);
+	}
+      }
+	break;
+
+      case XK_Escape:
+	playlist_exit(NULL, NULL);
+	break;
+	
+      default:
+	gui_handle_event(event, data);
+	break;
       }
-    }
-      break;
-      
-    default:
-      gui_handle_event(event, data);
-      break;
     }
     break;
 
@@ -542,7 +526,12 @@ static void _playlist_handle_event(XEvent *event, void *data) {
 
 static void _playlist_apply_cb(void *data) {
   playlist_mrlident_toggle();
-  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+  gui_set_current_mmk(mediamark_get_current_mmk());
+}
+
+void playlist_get_input_focus(void) {
+  if(playlist)
+    try_to_set_input_focus(playlist->window);
 }
 
 void playlist_mmk_editor(void) {
@@ -555,68 +544,61 @@ void playlist_mmk_editor(void) {
   }
 }
 
+static void _scan_for_playlist_infos(xine_stream_t *stream, int n) {
+  
+  if(xine_open(stream, gGui->playlist.mmk[n]->mrl)) {
+    char  *ident;
+    
+    if((ident = stream_infos_get_ident_from_stream(stream)) != NULL) {
+      
+      if(gGui->playlist.mmk[n]->ident)
+	free(gGui->playlist.mmk[n]->ident);
+      
+      gGui->playlist.mmk[n]->ident = strdup(ident);
+      
+      if(n == gGui->playlist.cur) {
+	
+	if(gGui->mmk.ident)
+	  free(gGui->mmk.ident);
+	
+	gGui->mmk.ident = strdup(ident);
+	
+	panel_update_mrl_display();
+      }
+      
+      free(ident);
+    }
+    xine_close(stream);
+  }
+}
+
+void playlist_scan_for_infos_selected(void) {
+
+  if(gGui->playlist.num) {
+    int                 selected = xitk_browser_get_current_selected(playlist->playlist);
+    xine_stream_t      *stream;
+    
+    stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
+    _scan_for_playlist_infos(stream, selected);
+    
+    xine_dispose(stream);
+    
+    playlist_mrlident_toggle();
+  }
+}
+
 void playlist_scan_for_infos(void) {
   
   if(gGui->playlist.num) {
-    int            i;
-    char          *ident;
-    int            old_pos = 0, rerun = 0;
-    xine_stream_t *stream = gGui->playlist.scan_stream;
-    int            mute = gGui->mixer.mute;
-
-    /* 
-     * Because we share audio driver (and since we can't stat infos of
-     * audio only streams without audio driver), pausing the gGui->stream
-     * avoid some sync/crash problems.
-     */
-    if(((xine_get_status(gGui->stream)) != XINE_STATUS_STOP) && (gGui->logo_mode == 0)) {
-      if(!gui_xine_get_pos_length(gGui->stream, &old_pos, NULL, NULL))
-	old_pos = 0;
-      gui_stop(NULL, NULL);
-      rerun = 1;
-    }
-    if((!mute) && gGui->mixer.caps & MIXER_CAP_MUTE) {
-      mute = 1;
-      xine_set_param(gGui->stream, XINE_PARAM_AUDIO_MUTE, 1);
-    }
-    else
-      mute = 0;
+    int                 i;
+    xine_stream_t      *stream;
     
-    for(i = 0; i < gGui->playlist.num; i++) {
-      
-      if(xine_open(stream, gGui->playlist.mmk[i]->mrl)) {
-
-	if((ident = stream_infos_get_ident_from_stream(stream)) != NULL) {
-	  
-	  if(gGui->playlist.mmk[i]->ident)
-	    free(gGui->playlist.mmk[i]->ident);
-	  
-	  gGui->playlist.mmk[i]->ident = strdup(ident);
-	  
-	  if(i == gGui->playlist.cur) {
-	    
-	    if(gGui->mmk.ident)
-	      free(gGui->mmk.ident);
-	    
-	    gGui->mmk.ident = strdup(ident);
-	    
-	    panel_update_mrl_display();
-	  }
-	  
-	  free(ident);
-	}
-
-	xine_close(stream);
-      }
-    }
+    stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
     
-    /* Restoring previous play status */
-    if(mute)
-      xine_set_param(gGui->stream, XINE_PARAM_AUDIO_MUTE, !mute);
+    for(i = 0; i < gGui->playlist.num; i++)
+      _scan_for_playlist_infos(stream, i);
     
-    if(rerun)
-      gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, old_pos, 
-			     0, gGui->mmk.av_offset, gGui->mmk.spu_offset);
+    xine_dispose(stream);
     
     playlist_mrlident_toggle();
   }
@@ -663,6 +645,11 @@ void playlist_update_playlist(void) {
  */
 void playlist_exit(xitk_widget_t *w, void *data) {
 
+  if(load_fb)
+    filebrowser_end(load_fb);
+  if(save_fb)
+    filebrowser_end(save_fb);
+
   if(playlist) {
     window_info_t wi;
     
@@ -702,12 +689,13 @@ void playlist_exit(xitk_widget_t *w, void *data) {
     free(playlist->widget_list);
     
     free(playlist);
-    playlist = NULL;  
+    playlist = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
 
-
 /*
  * return 1 if playlist editor is ON
  */
@@ -746,13 +734,12 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
       
       if(!strcasecmp(autoplay_plugins[i], xitk_labelbutton_get_label(w))) {
 	int                num_mrls = 0;
-	char             **autoplay_mrls = 
-	  xine_get_autoplay_mrls (gGui->xine, autoplay_plugins[i], &num_mrls);
+	char             **autoplay_mrls = xine_get_autoplay_mrls(gGui->xine, autoplay_plugins[i], &num_mrls);
+	xine_stream_t      *stream;
 	
 	if(autoplay_mrls) {
-	  int            j;
-	  int            cdda_mode = 0;
-	  xine_stream_t *stream = gGui->playlist.scan_stream;
+	  int                j;
+	  int                cdda_mode = 0;
 
 	  if(!strcasecmp(autoplay_plugins[i], "cd"))
 	    cdda_mode = 1;
@@ -765,10 +752,13 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	      xitk_inputtext_change_text(playlist->winput, NULL);
 	    gGui->playlist.cur = 0;
 	  }
-
+	  
 	  if(!gGui->playlist.num)
 	    gGui->playlist.cur = 0;
 	  
+
+	  stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
+	  
 	  for (j = 0; j < num_mrls; j++) {
 	    char *ident = NULL;
 	    
@@ -776,15 +766,17 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	      ident = stream_infos_get_ident_from_stream(stream);
 	      xine_close(stream);
 	    }
-
-	    mediamark_add_entry(autoplay_mrls[j], ident ? ident : autoplay_mrls[j], NULL, 0, -1, 0, 0);
-
+	    
+	    mediamark_append_entry(autoplay_mrls[j], ident ? ident : autoplay_mrls[j], NULL, 0, -1, 0, 0);
+	    
 	    if(ident)
 	      free(ident);
 	  }
+
+	  xine_dispose(stream);
 	  
 	  if(gGui->playlist.cur == 0)
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	  
 	  /* 
 	   * If we're in newbie mode, start playback immediately
@@ -795,7 +787,6 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	      gui_stop(NULL, NULL);
 	    gui_play(NULL, NULL);
 	  }
-	  
 	}
       }
       
@@ -844,8 +835,8 @@ void playlist_update_focused_entry(void) {
   if(playlist != NULL) {
     if(playlist->window) {
       if(playlist->visible && playlist->running && playlist->playlist_len) {
-	char *pa_mrl, *pl_mrl;
-	mediamark_t *mmk = (mediamark_t *)mediamark_get_current_mmk();
+	char        *pa_mrl, *pl_mrl;
+	mediamark_t *mmk = mediamark_get_current_mmk();
 	
 	if(mmk) {
 
@@ -924,30 +915,33 @@ void playlist_change_skins(void) {
 
     XLockDisplay(gGui->display);
     
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, playlist->window, gGui->video_window);
     
     Imlib_destroy_image(gGui->imlib_data, old_img);
     Imlib_apply_image(gGui->imlib_data, new_img, playlist->window);
-    
     XUnlockDisplay(gGui->display);
-    
+
+    if(playlist_is_visible())
+      playlist_raise_window();
+
     xitk_skin_unlock(gGui->skin_config);
     
     xitk_change_skins_widget_list(playlist->widget_list, gGui->skin_config);
     
     {
       int x, y, dir;
-      int i = 0;
+      int i = 0, max;
       
-      x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
-      y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
+      x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
+      y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
       dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayBG");
-      
+      max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayBG");
+
       switch(dir) {
       case DIRECTION_UP:
       case DIRECTION_DOWN:
-	while(playlist->autoplay_plugins[i] != NULL) {
+	while((max && ((i < max) && (playlist->autoplay_plugins[i] != NULL))) || (!max && playlist->autoplay_plugins[i] != NULL)) {
 	  
 	  (void) xitk_set_widget_pos(playlist->autoplay_plugins[i], x, y);
 	  
@@ -961,7 +955,7 @@ void playlist_change_skins(void) {
 	break;
       case DIRECTION_LEFT:
       case DIRECTION_RIGHT:
-	while(playlist->autoplay_plugins[i] != NULL) {
+	while((max && ((i < max) && (playlist->autoplay_plugins[i] != NULL))) || (!max && playlist->autoplay_plugins[i] != NULL)) {
 	  
 	  (void) xitk_set_widget_pos(playlist->autoplay_plugins[i], x, y);
 	  
@@ -974,6 +968,13 @@ void playlist_change_skins(void) {
 	}
 	break;
       }
+
+      if(max) {
+	while(playlist->autoplay_plugins[i] != NULL) {
+	  xitk_disable_and_hide_widget(playlist->autoplay_plugins[i]);
+	  i++;
+	}
+      }
     }
     
     xitk_paint_widget_list(playlist->widget_list);
@@ -984,8 +985,19 @@ void playlist_deinit(void) {
   if(playlist) {
     if(playlist_is_visible())
       playlist_toggle_visibility(NULL, NULL);
-    xitk_unregister_event_handler(&playlist->widget_key);
+    playlist_exit(NULL, NULL);
   }
+  else {
+    if(load_fb)
+      filebrowser_end(load_fb);
+    if(save_fb)
+      filebrowser_end(save_fb);
+  }
+}
+
+void playlist_reparent(void) {
+  if(playlist)
+    reparent_window(playlist->window);
 }
 
 /*
@@ -1069,11 +1081,17 @@ void playlist_editor(void) {
 				    CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				    &attr);
   
-  XSetStandardProperties(gGui->display, playlist->window, title, title,
-			 None, NULL, 0, &hint);
+  XmbSetWMProperties(gGui->display, playlist->window, title, title, NULL, 0,
+                     &hint, NULL, NULL);
   
   XSelectInput(gGui->display, playlist->window, INPUT_MOTION | KeymapStateMask);
+  XUnlockDisplay (gGui->display);
   
+  if(!video_window_is_visible())
+    xitk_set_wm_window_type(playlist->window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(playlist->window, WINDOW_TYPE_NORMAL);
+
   if(is_layer_above())
     xitk_set_layer_above(playlist->window);
 
@@ -1081,6 +1099,7 @@ void playlist_editor(void) {
    * wm, no border please
    */
 
+  XLockDisplay (gGui->display);
   prop = XInternAtom(gGui->display, "_MOTIF_WM_HINTS", True);
   mwmhints.flags = MWM_HINTS_DECORATIONS;
   mwmhints.decorations = 0;
@@ -1095,13 +1114,15 @@ void playlist_editor(void) {
     xclasshint->res_name = title;
     xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, playlist->window, xclasshint);
+    XFree(xclasshint);
   }
 
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
     wm_hint->input = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags = InputHint | StateHint;
+    wm_hint->icon_pixmap   = gGui->icon;
+    wm_hint->flags = InputHint | StateHint | IconPixmapHint;
     XSetWMHints(gGui->display, playlist->window, wm_hint);
     XFree(wm_hint);
   }
@@ -1230,13 +1251,14 @@ void playlist_editor(void) {
 
   {
     int                x, y, dir;
-    int                i = 0;
+    int                i = 0, max;
     const char *const *autoplay_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
     
-    x = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
-    y = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
+    x   = xitk_skin_get_coord_x(gGui->skin_config, "AutoPlayBG");
+    y   = xitk_skin_get_coord_y(gGui->skin_config, "AutoPlayBG");
     dir = xitk_skin_get_direction(gGui->skin_config, "AutoPlayBG");
-      
+    max = xitk_skin_get_max_buttons(gGui->skin_config, "AutoPlayBG");
+
     while(autoplay_plugins[i] != NULL) {
 
       lb.skin_element_name = "AutoPlayBG";
@@ -1270,6 +1292,9 @@ void playlist_editor(void) {
 	break;
       }
 
+      if(max && (i >= max))
+	xitk_disable_and_hide_widget(playlist->autoplay_plugins[i]);
+      
       i++;
 
     }
