@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.h,v 1.19 2003/09/15 20:28:22 f1rmb Exp $
+ * $Id: playlist.h,v 1.23 2004/07/02 09:33:37 f1rmb Exp $
  *
  */
 
@@ -31,6 +31,7 @@
 #include "xitk.h"
 
 void playlist_mmk_editor(void);
+void playlist_scan_for_infos_selected(void);
 void playlist_scan_for_infos(void);
 void playlist_show_tips(int enabled, unsigned long timeout);
 void playlist_mrlident_toggle(void);
@@ -51,5 +52,6 @@ void playlist_deinit(void);
 void playlist_editor(void);
 void playlist_load_playlist(xitk_widget_t *w, void *data);
 void playlist_save_playlist(xitk_widget_t *w, void *data);
-
+void playlist_reparent(void);
+void playlist_get_input_focus(void);
 #endif
