@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: post.c,v 1.43 2003/12/08 14:58:53 miguelfreitas Exp $
+ * $Id: post.c,v 1.53 2004/07/02 09:33:37 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -25,6 +25,10 @@
 #endif
 
 #include <stdio.h>
+
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -128,24 +132,14 @@ typedef struct {
 static _pplugin_t    *pplugin = NULL;
 static char         **post_audio_plugins;
 
-static pthread_t      rewire_thread;
-
 /* Some functions prototype */
 static void _pplugin_unwire(void);
-static post_element_t **pplugin_parse_and_load(const char *pchain, int *post_elements_num);
+static post_element_t **pplugin_parse_and_load(const char *, int *);
 static void _pplugin_rewire(void);
-void _pplugin_rewire_from_post_elements(post_element_t **post_elements, int post_elements_num);
-static post_element_t **_pplugin_join_deinterlace_and_post_elements(int *post_elements_num);
+void _pplugin_rewire_from_post_elements(post_element_t **, int);
+static post_element_t **_pplugin_join_deinterlace_and_post_elements(int *);
 static void _pplugin_save_chain(void);
-
-static void *post_rewire_thread(void *data) {
-  void (*rewire)(void) = data;
-  
-  pthread_detach(pthread_self());
-  rewire();
-  pthread_exit(NULL);
-  return NULL;
-}
+static void _pplugin_close_help(xitk_widget_t *, void *);
 
 static void post_deinterlace_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
   post_element_t **posts = NULL;
@@ -175,15 +169,8 @@ static void post_deinterlace_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
 }
 
 static void post_audio_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
-  int err;
-  
-  gGui->visual_anim.post_plugin_num = cfg->num_value;
-  
-  if ((err = pthread_create(&rewire_thread,
-			    NULL, post_rewire_thread, (void *)post_rewire_visual_anim)) != 0) {
-    printf(_("%s(): can't create new thread (%s)\n"), __XINE_FUNCTION__, strerror(err));
-    abort();
-  }
+  gGui->visual_anim.post_plugin_num = cfg->num_value;  
+  post_rewire_visual_anim();
 }
 
 const char **post_get_audio_plugins_names(void) {
@@ -205,11 +192,7 @@ void post_init(void) {
       
       while(pol[num_plug]) {
 	
-	if(!num_plug)
-	  post_audio_plugins = (char **) xine_xmalloc(sizeof(char *) * 2);
-	else
-	  post_audio_plugins = (char **) realloc(post_audio_plugins, 
-						 sizeof(char *) * (num_plug + 2));
+	post_audio_plugins = (char **) realloc(post_audio_plugins, sizeof(char *) * (num_plug + 2));
 	
 	post_audio_plugins[num_plug]     = strdup(pol[num_plug]);
 	post_audio_plugins[num_plug + 1] = NULL;
@@ -279,22 +262,10 @@ int post_rewire_audio_post_to_stream(xine_stream_t *stream) {
 
 static void _pplugin_unwire(void) {
   xine_post_out_t  *vo_source;
-  int               paused = 0;
   
   vo_source = xine_get_video_source(gGui->stream);
 
-  if((paused = (xine_get_param(gGui->stream, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)))
-    xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_SLOW_4);
-  
   (void) xine_post_wire_video_port(vo_source, gGui->vo_port);
-  
-  if(paused)
-    xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
-  
-  /* Waiting a small chunk of time helps to avoid crashing */
-  xine_usec_sleep(500000);
-  
-
 }
 
 static void _pplugin_rewire(void) {
@@ -407,6 +378,7 @@ static void _pplugin_hide_obj(post_object_t *pobj) {
     DISABLE_ME(pobj->comment);
     DISABLE_ME(pobj->up);
     DISABLE_ME(pobj->down);
+    DISABLE_ME(pobj->help);
   }
 }
 
@@ -423,6 +395,9 @@ static void _pplugin_show_obj(post_object_t *pobj) {
     if(pobj->properties)
       xitk_set_widget_pos(pobj->properties, pobj->x + 200, pobj->y + 2);
     
+    if(pobj->help)
+      xitk_set_widget_pos(pobj->help, pobj->x + 390, pobj->y);
+
     if(pobj->comment)
       xitk_set_widget_pos(pobj->comment, pobj->x + 30, (pobj->y + 27));
     
@@ -441,6 +416,9 @@ static void _pplugin_show_obj(post_object_t *pobj) {
     ENABLE_ME(pobj->value);
     ENABLE_ME(pobj->comment);
 
+    if(pobj->help)
+      ENABLE_ME(pobj->help);
+    
     if((!_pplugin_is_first_filter(pobj)) && (xitk_combo_get_current_selected(pobj->plugins)))
       ENABLE_ME(pobj->up);
     
@@ -620,7 +598,7 @@ static void _pplugin_add_parameter_widget(post_object_t *pobj) {
     xitk_font_t          *fs;
 
     memset(&buffer, 0, sizeof(buffer));
-    snprintf(buffer, 2047, "%s:", (pobj->param->description) 
+    snprintf(buffer, sizeof(buffer), "%s:", (pobj->param->description) 
 	     ? pobj->param->description : _("No description available"));
 
     fs = xitk_font_load_font(gGui->display, boldfontname);
@@ -793,8 +771,7 @@ static void _pplugin_get_plugins(void) {
     pplugin->plugin_names    = (char **) xine_xmalloc(sizeof(char *) * 2);
     pplugin->plugin_names[i] = strdup(_("No Filter"));
     while(pol[i]) {
-      pplugin->plugin_names = (char **) 
-	realloc(pplugin->plugin_names, sizeof(char *) * (i + 1 + 2));
+      pplugin->plugin_names = (char **) realloc(pplugin->plugin_names, sizeof(char *) * (i + 1 + 2));
       
       pplugin->plugin_names[i + 1] = strdup(pol[i]);
       i++;
@@ -836,6 +813,14 @@ static void _pplugin_destroy_only_obj(post_object_t *pobj) {
       _pplugin_destroy_widget(pobj->value);
       pobj->value = NULL;
     }
+
+    if(pobj->api && pobj->help) {
+      if(pplugin->help_running)
+	_pplugin_close_help(NULL,NULL);
+      _pplugin_destroy_widget(pobj->help);
+      pobj->help = NULL;
+    }
+    
   }
 }
 
@@ -898,8 +883,7 @@ static void _pplugin_kill_filters_from(post_object_t *pobj) {
   _pplugin_send_expose();
 
   if(num != pplugin->object_num) {
-    pplugin->post_objects = (post_object_t **) 
-      realloc(pplugin->post_objects, sizeof(post_object_t *) * (pplugin->object_num + 2));
+    pplugin->post_objects = (post_object_t **) realloc(pplugin->post_objects, sizeof(post_object_t *) * (pplugin->object_num + 2));
     pplugin->post_objects[pplugin->object_num + 1] = NULL;
   }
 }
@@ -923,11 +907,7 @@ static int __pplugin_retrieve_parameters(post_object_t *pobj) {
     
     while(parm->type != POST_PARAM_TYPE_LAST) {
       
-      if(!pnum)
-	pobj->properties_names = (char **) xine_xmalloc(sizeof(char *) * 2);
-      else
-	pobj->properties_names = (char **) 
-	  realloc(pobj->properties_names, sizeof(char *) * (pnum + 2));
+      pobj->properties_names = (char **) realloc(pobj->properties_names, sizeof(char *) * (pnum + 2));
       
       pobj->properties_names[pnum]     = strdup(parm->name);
       pobj->properties_names[pnum + 1] = NULL;
@@ -987,12 +967,22 @@ static void _pplugin_show_help(xitk_widget_t *w, void *data) {
   xitk_labelbutton_widget_t   lb;
   xitk_browser_widget_t       br;
 
+  if(!pobj->api) {
+    if(pplugin->help_running)
+      _pplugin_close_help(NULL,NULL);
+    return;
+  }
+  
   /* create help window if needed */
+
   if( !pplugin->help_running ) {
     x = y = 100;
     pplugin->helpwin = xitk_window_create_dialog_window(gGui->imlib_data, 
   						 _("Plugin Help"), x, y,
   						 HELP_WINDOW_WIDTH, HELP_WINDOW_HEIGHT);
+
+    set_window_states_start((xitk_window_get_window(pplugin->helpwin)));
+
     XLockDisplay (gGui->display);
     gc = XCreateGC(gGui->display, 
   		 (xitk_window_get_window(pplugin->helpwin)), None, None);
@@ -1055,10 +1045,7 @@ static void _pplugin_show_help(xitk_widget_t *w, void *data) {
     p = pobj->api->get_help();
     
     do {
-      if(!lines)
-	hbuf = (char **) xine_xmalloc(sizeof(char *) * 2);
-      else
-	hbuf  = (char **) realloc(hbuf, sizeof(char *) * (lines + 2));
+      hbuf  = (char **) realloc(hbuf, sizeof(char *) * (lines + 2));
 
       hbuf[lines]    = malloc(BROWSER_LINE_WIDTH+1);
       hbuf[lines+1]  = NULL;
@@ -1078,7 +1065,7 @@ static void _pplugin_show_help(xitk_widget_t *w, void *data) {
       int    i = 0;
 
       pplugin->help_text = hbuf;
-      xitk_browser_update_list(pplugin->help_browser, (const char **)pplugin->help_text, 
+      xitk_browser_update_list(pplugin->help_browser, (const char **)pplugin->help_text, NULL,
 			       lines, 0);
       
       if(ohbuf) {
@@ -1137,21 +1124,23 @@ static void _pplugin_retrieve_parameters(post_object_t *pobj) {
     xitk_combo_set_select(pobj->properties, 0);
     xitk_combo_callback_exec(pobj->properties);
 
+    if(pobj->api && pobj->api->get_help) {
 
-    XITK_WIDGET_INIT(&lb, gGui->imlib_data);
-    lb.button_type       = CLICK_BUTTON;
-    lb.label             = _("Help");
-    lb.align             = ALIGN_CENTER;
-    lb.callback          = _pplugin_show_help; 
-    lb.state_callback    = NULL;
-    lb.userdata          = pobj;
-    lb.skin_element_name = NULL;
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pplugin->widget_list)), 
-     (pobj->help = xitk_noskin_labelbutton_create(pplugin->widget_list, 
-						  &lb, pobj->x + 390, pobj->y, 50, 20,
-						  "Black", "Black", "White", btnfontname)));
-    xitk_show_widget(pobj->help);
-    xitk_enable_widget(pobj->help);
+      XITK_WIDGET_INIT(&lb, gGui->imlib_data);
+      lb.button_type       = CLICK_BUTTON;
+      lb.label             = _("Help");
+      lb.align             = ALIGN_CENTER;
+      lb.callback          = _pplugin_show_help; 
+      lb.state_callback    = NULL;
+      lb.userdata          = pobj;
+      lb.skin_element_name = NULL;
+      xitk_list_append_content((XITK_WIDGET_LIST_LIST(pplugin->widget_list)), 
+			       (pobj->help = xitk_noskin_labelbutton_create(pplugin->widget_list, 
+									    &lb, pobj->x + 390, pobj->y, 50, 20,
+									    "Black", "Black", "White", btnfontname)));
+      xitk_show_widget(pobj->help);
+      xitk_enable_widget(pobj->help);
+    }
   }
   else {
     xitk_label_widget_t   lb;
@@ -1187,8 +1176,7 @@ static void _pplugin_select_filter(xitk_widget_t *w, void *data, int select) {
     else {
       _pplugin_destroy_base_obj(pobj);
       
-      pplugin->post_objects = (post_object_t **) 
-	realloc(pplugin->post_objects, sizeof(post_object_t *) * (pplugin->object_num + 2));
+      pplugin->post_objects = (post_object_t **) realloc(pplugin->post_objects, sizeof(post_object_t *) * (pplugin->object_num + 2));
       pplugin->post_objects[pplugin->object_num + 1] = NULL;
       
       if(!xitk_is_widget_enabled(pplugin->new_filter))
@@ -1271,12 +1259,7 @@ static void _pplugin_create_filter_object(void) {
   
   xitk_disable_widget(pplugin->new_filter);
 
-  if(!pplugin->object_num)
-    pplugin->post_objects = (post_object_t **) xine_xmalloc(sizeof(post_object_t *) * 2);
-  else
-    pplugin->post_objects = (post_object_t **) 
-      realloc(pplugin->post_objects, sizeof(post_object_t *) * (pplugin->object_num + 2));
-  
+  pplugin->post_objects = (post_object_t **) realloc(pplugin->post_objects, sizeof(post_object_t *) * (pplugin->object_num + 2));
   pplugin->post_objects[pplugin->object_num + 1] = NULL;
   
   pobj = pplugin->post_objects[pplugin->object_num] = (post_object_t *) xine_xmalloc(sizeof(post_object_t));
@@ -1449,8 +1432,7 @@ static void _pplugin_save_chain(void) {
 	  free(gGui->post_elements[j]);
 	}
 	
-	gGui->post_elements = (post_element_t **) 
-	  realloc(gGui->post_elements, sizeof(post_element_t *) * (post_num + 1));
+	gGui->post_elements = (post_element_t **) realloc(gGui->post_elements, sizeof(post_element_t *) * (post_num + 1));
 	
       }
 	
@@ -1555,6 +1537,8 @@ static void pplugin_exit(xitk_widget_t *w, void *data) {
    
     free(pplugin);
     pplugin = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -1573,24 +1557,15 @@ static void _pplugin_handle_event(XEvent *event, void *data) {
     }
     break;
 
-  case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
-    int            modifier;
-    
+  case KeyPress:
     if(xitk_is_widget_enabled(pplugin->slider)) {
-      
-      mykeyevent = event->xkey;
+      KeySym         mkey;
+      int            modifier;
       
       xitk_get_key_modifier(event, &modifier);
-      
-      XLockDisplay(gGui->display);
-      len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-      XUnlockDisplay(gGui->display);
-      
-      switch(mykey) {
+      mkey = xitk_get_key_pressed(event);
+
+      switch(mkey) {
 	
       case XK_Up:
 	if((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) {
@@ -1605,29 +1580,34 @@ static void _pplugin_handle_event(XEvent *event, void *data) {
 	  xitk_slider_callback_exec(pplugin->slider);
 	}
 	break;
-
-
-      case XK_Next: {
-	int pos, max = xitk_slider_get_max(pplugin->slider);
 	
-	pos = max - (pplugin->first_displayed + MAX_DISPLAY_FILTERS);
-	xitk_slider_set_pos(pplugin->slider, (pos >= 0) ? pos : 0);
-	xitk_slider_callback_exec(pplugin->slider);
-      }
-	break;
-
-      case XK_Prior: {
-	int pos, max = xitk_slider_get_max(pplugin->slider);
 	
-	pos = max - (pplugin->first_displayed - MAX_DISPLAY_FILTERS);
-	xitk_slider_set_pos(pplugin->slider, (pos <= max) ? pos : max);
-	xitk_slider_callback_exec(pplugin->slider);
-      }
+      case XK_Next:
+	{
+	  int pos, max = xitk_slider_get_max(pplugin->slider);
+	  
+	  pos = max - (pplugin->first_displayed + MAX_DISPLAY_FILTERS);
+	  xitk_slider_set_pos(pplugin->slider, (pos >= 0) ? pos : 0);
+	  xitk_slider_callback_exec(pplugin->slider);
+	}
 	break;
 	
+      case XK_Prior:
+	{
+	  int pos, max = xitk_slider_get_max(pplugin->slider);
+	  
+	  pos = max - (pplugin->first_displayed - MAX_DISPLAY_FILTERS);
+	  xitk_slider_set_pos(pplugin->slider, (pos <= max) ? pos : max);
+	  xitk_slider_callback_exec(pplugin->slider);
+	}
+	break;
+	
+      case XK_Escape:
+	pplugin_exit(NULL, NULL);
+	break;
+
       }
     }
-  }
     break;
 
   }
@@ -1665,10 +1645,6 @@ void _pplugin_rewire_from_post_elements(post_element_t **post_elements, int post
   if(post_elements_num) {
     xine_post_out_t   *vo_source;
     int                i = 0;
-    int                paused = 0;
-    
-    if((paused = (xine_get_param(gGui->stream, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)))
-      xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_SLOW_4);
     
     for(i = (post_elements_num - 1); i >= 0; i--) {
       const char *const *outs = xine_post_list_outputs(post_elements[i]->post);
@@ -1686,12 +1662,9 @@ void _pplugin_rewire_from_post_elements(post_element_t **post_elements, int post
     
     vo_source = xine_get_video_source(gGui->stream);
     xine_post_wire_video_port(vo_source, post_elements[0]->post->video_input[0]);
-    
-    if(paused)
-      xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
-
   }
 }
+
 void pplugin_rewire_posts(void) {
   
   _pplugin_unwire();
@@ -1744,6 +1717,11 @@ void pplugin_update_enable_button(void) {
     xitk_labelbutton_set_state(pplugin->enable, gGui->post_enable);
 }
 
+void pplugin_reparent(void) {
+  if(pplugin)
+    reparent_window((xitk_window_get_window(pplugin->xwin)));
+}
+
 void pplugin_panel(void) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
@@ -1777,6 +1755,8 @@ void pplugin_panel(void) {
 						   _("Chain Reaction"), x, y,
 						   WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(pplugin->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(pplugin->xwin)), None, None);
@@ -1945,11 +1925,7 @@ static post_element_t **pplugin_parse_and_load(const char *pchain, int *post_ele
 	if(post) {
 	  post_object_t  pobj;
 	 
-	  if(!(*post_elements_num))
-	    post_elements = (post_element_t **) xine_xmalloc(sizeof(post_element_t *) * 2);
-	  else
-	    post_elements = (post_element_t **) 
-	      realloc(post_elements, sizeof(post_element_t *) * ((*post_elements_num) + 2));
+	  post_elements = (post_element_t **) realloc(post_elements, sizeof(post_element_t *) * ((*post_elements_num) + 2));
 	  
 	  post_elements[(*post_elements_num)] = (post_element_t *) 
 	    xine_xmalloc(sizeof(post_element_t));
@@ -2081,8 +2057,7 @@ void pplugin_parse_and_store_post(const char *post_chain) {
       int i;
       int ptot = gGui->post_elements_num + num;
       
-      gGui->post_elements = (post_element_t **) realloc(gGui->post_elements, 
-							sizeof(post_element_t *) * (ptot + 1));
+      gGui->post_elements = (post_element_t **) realloc(gGui->post_elements, sizeof(post_element_t *) * (ptot + 1));
       for(i = gGui->post_elements_num; i <  ptot; i++)
 	gGui->post_elements[i] = posts[i - gGui->post_elements_num];
 
