@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: session.c,v 1.15 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: session.c,v 1.17 2004/04/27 20:20:11 f1rmb Exp $
  *
  */
 /* Largely inspired of xmms control socket stuff */
@@ -84,7 +84,7 @@ static int connect_to_session(int session) {
     euid             = geteuid();
     setuid(euid);
 
-    snprintf(saddr.sun_path, 108, "%s/.xine/session.%d", (xine_get_homedir()), session);
+    snprintf(saddr.sun_path, 108, "%s%s%d", (xine_get_homedir()), "/.xine/session.", session);
     setreuid(stored_uid, euid);
 
     if((connect(fd,(struct sockaddr *) &saddr, sizeof(saddr))) != -1)
@@ -257,7 +257,7 @@ int is_remote_running(int session) {
 
 
 #ifndef CTRL_TEST
-void *ctrlsocket_func(void *data) {
+static void *ctrlsocket_func(void *data) {
   fd_set                set;
   struct timeval        tv;
   struct sockaddr_un    saddr;
@@ -370,7 +370,7 @@ void *ctrlsocket_func(void *data) {
 	  gui_stop(NULL, NULL);
 	
 	gGui->playlist.cur = 0;
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	gui_set_current_mmk(mediamark_get_current_mmk());
 	gui_play(NULL, NULL);
       }
       send_ack(shdr);
@@ -382,7 +382,7 @@ void *ctrlsocket_func(void *data) {
 	  gui_stop(NULL, NULL);
 	
 	gGui->playlist.cur = gGui->playlist.num - 1;
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	gui_set_current_mmk(mediamark_get_current_mmk());
 	gui_play(NULL, NULL);
       }
       send_ack(shdr);
@@ -410,7 +410,7 @@ void *ctrlsocket_func(void *data) {
 
     case CMD_PLAYLIST_LOAD:
       mediamark_load_mediamarks((const char *)shdr->data);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
       playlist_update_playlist();
       if((!is_playback_widgets_enabled()) && gGui->playlist.num)
 	enable_playback_controls(1);
@@ -513,7 +513,7 @@ int init_session(void) {
     for(i = 0;; i++)	{
       saddr.sun_family = AF_UNIX;
       
-      snprintf(saddr.sun_path, 108, "%s/.xine/session.%d", (xine_get_homedir()), i);
+      snprintf(saddr.sun_path, 108, "%s%s%d", (xine_get_homedir()), "/.xine/session.", i);
       if(!is_remote_running(i)) {
 	if((unlink(saddr.sun_path) == -1) && errno != ENOENT) {
 	  fprintf(stderr, "setup_ctrlsocket(): Failed to unlink %s (Error: %s)", 
@@ -610,22 +610,19 @@ void session_handle_subopt(char *suboptarg, int *session) {
       else if(!strcasecmp(optstr, "prev"))
 	spu_prev++;
       break;
-
+      
       /* session */
     case 12:
       if((atoi(optstr)) >= 0)
 	optsess = atoi(optstr);
       break;
-
+      
       /* mrl */
     case 13:
-      if(num_mrls == 0)
-	mrls = (char **) xine_xmalloc(sizeof(char *) * 2);
-      else
-	mrls = (char **) realloc(mrls, sizeof(char *) * (num_mrls + 2));
-	
-	mrls[num_mrls++] = strdup(optstr);
-	mrls[num_mrls]   = NULL;
+      mrls = (char **) realloc(mrls, sizeof(char *) * (num_mrls + 2));
+      
+      mrls[num_mrls++] = strdup(optstr);
+      mrls[num_mrls]   = NULL;
       break;
 
       /* playlist */
