@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.c,v 1.87 2003/11/29 23:18:35 f1rmb Exp $
+ * $Id: setup.c,v 1.98 2004/07/02 09:33:38 f1rmb Exp $
  *
  *
  */
@@ -94,7 +94,8 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
     xitk_widget_t *lbl;                                                                         \
     char           _labelkey[strlen(labelkey) + 5];                                             \
                                                                                                 \
-    sprintf(_labelkey, "%s%s", labelkey, (!entry->callback) ? " (*)" : "");                     \
+    snprintf(_labelkey, sizeof(_labelkey), "%s%s", labelkey,		                        \
+      (!entry->callback_data && !entry->callback) ? " (*)" : "");                               \
                                                                                                 \
     fs = xitk_font_load_font(gGui->display, fontname);                                          \
     xitk_font_set_font(fs, (XITK_WIDGET_LIST_GC(setup->widget_list)));                          \
@@ -177,7 +178,6 @@ typedef struct {
 
 static _setup_t    *setup = NULL;
 
-static void setup_end(xitk_widget_t *, void *);
 static void setup_change_section(xitk_widget_t *, void *, int);
 
 static void add_widget_to_list(xitk_widget_t *w) {
@@ -187,7 +187,7 @@ static void add_widget_to_list(xitk_widget_t *w) {
 /*
  * Leaving setup panel, release memory.
  */
-void setup_exit(xitk_widget_t *w, void *data) {
+static void setup_exit(xitk_widget_t *w, void *data) {
 
   if(setup) {
     window_info_t wi;
@@ -220,6 +220,8 @@ void setup_exit(xitk_widget_t *w, void *data) {
     
     free(setup);
     setup = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -284,7 +286,7 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 	  char          *strval = NULL;
 	  
 	  if(!need_restart) {
-	    if(setup->wg[i]->changed && (!setup->wg[i]->cfg->callback))
+	    if(setup->wg[i]->changed && (!setup->wg[i]->cfg->callback_data && !setup->wg[i]->cfg->callback))
 	      need_restart = 1;
 	  }
 
@@ -346,7 +348,7 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 				 ALIGN_CENTER,
 				 _("You changed some configuration value which require"
 				   " to restart xine to take effect."));
-      if(!gGui->use_root_window) {
+      if(!gGui->use_root_window && gGui->video_display == gGui->display) {
 	XLockDisplay(gGui->display);
 	XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
 	XUnlockDisplay(gGui->display);
@@ -369,7 +371,7 @@ static void setup_set_cursor(int state) {
 
 static void setup_ok(xitk_widget_t *w, void *data) {
   setup_apply(w, data);
-  setup_end(w, data);
+  setup_exit(w, data);
 }
 
 /*
@@ -461,24 +463,15 @@ static void setup_handle_event(XEvent *event, void *data) {
     }
     break;
 
-  case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
-    int            modifier;
-    
+  case KeyPress:
     if(xitk_is_widget_enabled(setup->slider_wg)) {
-      
-      mykeyevent = event->xkey;
+      KeySym         mkey;
+      int            modifier;
       
       xitk_get_key_modifier(event, &modifier);
+      mkey = xitk_get_key_pressed(event);
       
-      XLockDisplay(gGui->display);
-      len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-      XUnlockDisplay(gGui->display);
-      
-      switch(mykey) {
+      switch(mkey) {
 	
       case XK_Up:
 	if((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) {
@@ -493,29 +486,32 @@ static void setup_handle_event(XEvent *event, void *data) {
 	  xitk_slider_callback_exec(setup->slider_wg);
 	}
 	break;
-
-
-      case XK_Next: {
-	int pos, max = xitk_slider_get_max(setup->slider_wg);
 	
-	pos = max - (setup->first_displayed + MAX_DISPLAY_WIDGETS);
-	xitk_slider_set_pos(setup->slider_wg, (pos >= 0) ? pos : 0);
-	xitk_slider_callback_exec(setup->slider_wg);
-      }
-	break;
-
-      case XK_Prior: {
-	int pos, max = xitk_slider_get_max(setup->slider_wg);
-	
-	pos = max - (setup->first_displayed - MAX_DISPLAY_WIDGETS);
-	xitk_slider_set_pos(setup->slider_wg, (pos <= max) ? pos : max);
-	xitk_slider_callback_exec(setup->slider_wg);
-      }
+      case XK_Next:
+	{
+	  int pos, max = xitk_slider_get_max(setup->slider_wg);
+	  
+	  pos = max - (setup->first_displayed + MAX_DISPLAY_WIDGETS);
+	  xitk_slider_set_pos(setup->slider_wg, (pos >= 0) ? pos : 0);
+	  xitk_slider_callback_exec(setup->slider_wg);
+	}
 	break;
 	
+      case XK_Prior:
+	{
+	  int pos, max = xitk_slider_get_max(setup->slider_wg);
+	  
+	  pos = max - (setup->first_displayed - MAX_DISPLAY_WIDGETS);
+	  xitk_slider_set_pos(setup->slider_wg, (pos <= max) ? pos : max);
+	  xitk_slider_callback_exec(setup->slider_wg);
+	}
+	break;
+	
+      case XK_Escape:
+	setup_exit(NULL, NULL);
+	break;
       }
     }
-  }
     break;
     
   }
@@ -740,7 +736,7 @@ static widget_triplet_t *setup_add_checkbox (const char *title, const char *labe
   xitk_list_append_content ((XITK_WIDGET_LIST_LIST(setup->widget_list)),
 			   (checkbox = 
 			    xitk_noskin_checkbox_create(setup->widget_list, &cb,
-							x, y, 10, 10)));
+							x, y, 12, 12)));
   xitk_checkbox_set_state (checkbox, entry->num_value);  
   ADD_LABEL(checkbox, label_cb, (void *) checkbox);
 
@@ -1031,7 +1027,7 @@ static void setup_sections (void) {
 /*
  *
  */
-static void setup_end(xitk_widget_t *w, void *data) {
+void setup_end(void) {
   setup_exit(NULL, NULL);
 }
 
@@ -1048,6 +1044,11 @@ static void setup_nextprev_wg(xitk_widget_t *w, void *data, int pos) {
   }
 }
 
+void setup_reparent(void) {
+  if(setup)
+    reparent_window((xitk_window_get_window(setup->xwin)));
+}
+
 /*
  * Create setup panel window
  */
@@ -1081,6 +1082,8 @@ void setup_panel(void) {
 						 _("xine setup"), 
 						 x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(setup->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(setup->xwin)), None, None);
@@ -1133,8 +1136,8 @@ void setup_panel(void) {
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)),
 			     (w = xitk_noskin_label_create(setup->widget_list, &lbl,
 							   (WINDOW_WIDTH - len) >> 1,
-							   (WINDOW_HEIGHT - (57 + 20)) + 1 +4,
-							   len + 1, 15, fontname)));
+							   (WINDOW_HEIGHT - (57 + 20)) + 1 + 4,
+							   len + 3, 15, fontname)));
     xitk_enable_and_show_widget(w);
   }
 
@@ -1171,7 +1174,7 @@ void setup_panel(void) {
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = ALIGN_CENTER;
-  lb.callback          = setup_end; 
+  lb.callback          = setup_exit; 
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
