@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.c,v 1.61 2003/12/01 21:37:24 f1rmb Exp $
+ * $Id: skins.c,v 1.75 2004/07/02 09:33:38 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,7 @@
 #include <dirent.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <string.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -106,7 +107,7 @@ static void get_available_skins_from(char *path) {
     
     while((pdirent = readdir(pdir)) != NULL) {
       memset(&fullfilename, 0, sizeof(fullfilename));
-      sprintf(fullfilename, "%s/%s", path, pdirent->d_name);
+      snprintf(fullfilename, sizeof(fullfilename), "%s/%s", path, pdirent->d_name);
       
       if((is_a_dir(fullfilename))
 	 && (!(strlen(pdirent->d_name) == 1 && pdirent->d_name[0] == '.' )
@@ -117,11 +118,10 @@ static void get_available_skins_from(char *path) {
 	 * Check if a skinconfig file exist
 	 */
 	memset(&skcfgname, 0, sizeof(skcfgname));
-	sprintf(skcfgname, "%s/%s", fullfilename, "skinconfig");
+	snprintf(skcfgname, sizeof(skcfgname), "%s/%s", fullfilename, "skinconfig");
 	if(is_a_file(skcfgname)) {
 	  
-	  skins_avail = (skins_locations_t **) realloc(skins_avail, 
-						       (skins_avail_num + 2) * sizeof(skins_locations_t*));
+	  skins_avail = (skins_locations_t **) realloc(skins_avail, (skins_avail_num + 2) * sizeof(skins_locations_t*));
 	  skins_avail[skins_avail_num] = (skins_locations_t *) xine_xmalloc(sizeof(skins_locations_t));
 	  
 	  skins_avail[skins_avail_num]->pathname = strdup(path);
@@ -156,29 +156,30 @@ static void looking_for_available_skins(void) {
   skins_avail = (skins_locations_t **) xine_xmalloc(sizeof(skins_locations_t*));
   
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s/.xine/skins", xine_get_homedir());
+  snprintf(buf, sizeof(buf), "%s%s", xine_get_homedir(), "/.xine/skins");
   
   get_available_skins_from(buf);
   get_available_skins_from(XINE_SKINDIR);
   
 }
 
+#if 0
 /*
  * Return default skindir.
  */
-char *skin_get_skindir(void) {
+static char *skin_get_skindir(void) {
   static char          tmp[2048];
   xine_cfg_entry_t     entry;
   char                *skin;
   
-  memset(&tmp, 0, 2048);
+  memset(&tmp, 0, sizeof(tmp));
   memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
   if(xine_config_lookup_entry(gGui->xine, "gui.skin", &entry))
     skin = (char *) skins_avail[entry.num_value]->skin;
   else
     skin = DEFAULT_SKIN;
   
-  snprintf(tmp, 2048, "%s/%s", XINE_SKINDIR, skin);
+  snprintf(tmp, sizeof(tmp), "%s/%s", XINE_SKINDIR, skin);
   
   return tmp;
 }
@@ -186,22 +187,23 @@ char *skin_get_skindir(void) {
 /*
  * Return the full pathname the skin (default) configfile.
  */
-char *skin_get_configfile(void) {
+static char *skin_get_configfile(void) {
   static char          tmp[2048];
   xine_cfg_entry_t  entry;
   char                *skin;
   
-  memset(&tmp, 0, 2048);
+  memset(&tmp, 0, sizeof(tmp));
   memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
   if (xine_config_lookup_entry(gGui->xine, "gui.skin",&entry)) 
     skin = (char *) skins_avail[entry.num_value]->skin;
   else
     skin = DEFAULT_SKIN;
   
-  snprintf(tmp, 2048, "%s/%s/skinconfig", XINE_SKINDIR, skin);
+  snprintf(tmp, sizeof(tmp), "%s/%s/skinconfig", XINE_SKINDIR, skin);
 
   return tmp;
 }
+#endif
 
 /*
  * Return pointer to skins_avail.
@@ -221,7 +223,7 @@ int get_available_skins_num(void) {
 /*
  *
  */
-int get_skin_offset(char *skin) {
+static int get_skin_offset(char *skin) {
   int i = 0;
   
   while(skins_avail[i] != NULL) {
@@ -236,7 +238,7 @@ int get_skin_offset(char *skin) {
  * Try to find desired skin in skins_avail, then return
  * a pointer to the right entry.
  */
-skins_locations_t *get_skin_location(char *skin) {
+static skins_locations_t *get_skin_location(char *skin) {
   int i = 0;
   
   while(skins_avail[i] != NULL) {
@@ -258,6 +260,7 @@ static int change_skin(skins_locations_t *sk) {
   int                  twice = 0, twice_load = 0;
   int                  ret = 0;
   char                *skin_anim;
+  xitk_skin_config_t  *nskin_config, *oskin_config;
 
   if(!sk)
     return -1;
@@ -266,15 +269,13 @@ static int change_skin(skins_locations_t *sk) {
 
   old_skin = DEFAULT_SKIN;
   
-  xitk_skin_unload_config(gGui->skin_config);
-
-  gGui->skin_config = xitk_skin_init_config();
+  nskin_config = xitk_skin_init_config(gGui->imlib_data);
 
  __reload_skin:
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s/%s", sks->pathname, sks->skin);
+  snprintf(buf, sizeof(buf), "%s/%s", sks->pathname, sks->skin);
 
-  if(!xitk_skin_load_config(gGui->skin_config, buf, "skinconfig")) {
+  if(!xitk_skin_load_config(nskin_config, buf, "skinconfig")) {
     skins_locations_t   *osks;
     
     ret = get_skin_offset(old_skin);
@@ -302,9 +303,9 @@ static int change_skin(skins_locations_t *sk) {
   twice_load = 0;
 
   /* Check skin version */
-  if(xitk_skin_check_version(gGui->skin_config, SKIN_IFACE_VERSION) < 1) {
-    xitk_skin_unload_config(gGui->skin_config);
-    gGui->skin_config = xitk_skin_init_config();
+  if(xitk_skin_check_version(nskin_config, SKIN_IFACE_VERSION) < 1) {
+    xitk_skin_unload_config(nskin_config);
+    nskin_config = xitk_skin_init_config(gGui->imlib_data);
     xine_error(_("Failed to load %s, wrong version. Load fallback skin '%s'.\n"), 
 	       buf, DEFAULT_SKIN);
     ret = get_skin_offset(DEFAULT_SKIN);
@@ -317,10 +318,13 @@ static int change_skin(skins_locations_t *sk) {
     free(gGui->visual_anim.mrls[gGui->visual_anim.num_mrls]);
     gGui->visual_anim.mrls[gGui->visual_anim.num_mrls--] = NULL;
   }
-  if((skin_anim = xitk_skin_get_animation(gGui->skin_config)) != NULL) {
+  if((skin_anim = xitk_skin_get_animation(nskin_config)) != NULL) {
     gGui->visual_anim.mrls[gGui->visual_anim.num_mrls++] = strdup(skin_anim);
   }
   
+  oskin_config = gGui->skin_config;
+  gGui->skin_config = nskin_config;
+
   { /* Now, change skins for each window */
     typedef struct {
       void (*change_skins)(void);
@@ -342,6 +346,8 @@ static int change_skin(skins_locations_t *sk) {
     }
   }
 
+  xitk_skin_unload_config(oskin_config);
+
   return ret;
 }
 
@@ -396,7 +402,7 @@ const char *skin_get_current_skin_dir(void) {
   (void) xine_config_lookup_entry(gGui->xine, "gui.skin", &entry);
   skin_num = entry.num_value;
   
-  sprintf(skin_dir, "%s/%s", skins_avail[skin_num]->pathname, skins_avail[skin_num]->skin);
+  snprintf(skin_dir, sizeof(skin_dir), "%s/%s", skins_avail[skin_num]->pathname, skins_avail[skin_num]->skin);
 
   return (const char *) &skin_dir;
 }
@@ -410,7 +416,7 @@ void preinit_skins_support(void) {
 
   change_config_entry = 0;
   
-  gGui->skin_config = xitk_skin_init_config();
+  gGui->skin_config = xitk_skin_init_config(gGui->imlib_data);
   
   looking_for_available_skins();
   
@@ -467,7 +473,7 @@ void init_skins_support(void) {
   
  __reload_skin:
   memset(&buf, 0, XITK_PATH_MAX + XITK_NAME_MAX);
-  sprintf(buf, "%s/%s", sk->pathname, sk->skin);
+  snprintf(buf, sizeof(buf), "%s/%s", sk->pathname, sk->skin);
   
   if(!xitk_skin_load_config(gGui->skin_config, buf, "skinconfig")) {
     if(!twice_load) {
@@ -488,12 +494,12 @@ void init_skins_support(void) {
   /* Check skin version */
   if(xitk_skin_check_version(gGui->skin_config, SKIN_IFACE_VERSION) < 1) {
     xitk_skin_unload_config(gGui->skin_config);
-    gGui->skin_config = xitk_skin_init_config();
+    gGui->skin_config = xitk_skin_init_config(gGui->imlib_data);
     fprintf(stderr, _("Failed to load %s skin, wrong version. Load fallback skin '%s'.\n"), 
 	    buf, (skins_avail[(get_skin_offset(DEFAULT_SKIN))]->skin));
     config_update_num("gui.skin", (get_skin_offset(DEFAULT_SKIN)));
     sk = get_skin_location(DEFAULT_SKIN);
-    sprintf(buf, "%s/%s", sk->pathname, sk->skin);
+    snprintf(buf, sizeof(buf), "%s/%s", sk->pathname, sk->skin);
     if(!twice) {
       twice++;
       goto __reload_skin;
@@ -619,10 +625,7 @@ static slx_entry_t **skins_get_slx_entries(char *url) {
 		printf("--\n");
 #endif
 		
-		if(entries_slx == 0)
-		  slxs = (slx_entry_t **) xine_xmalloc(sizeof(slx_entry_t *) * 2);
-		else
-		  slxs = (slx_entry_t **) realloc(slxs, sizeof(slx_entry_t *) * (entries_slx + 2));
+		slxs = (slx_entry_t **) realloc(slxs, sizeof(slx_entry_t *) * (entries_slx + 2));
 		
 		slxs[entries_slx] = (slx_entry_t *) xine_xmalloc(sizeof(slx_entry_t));
 		slxs[entries_slx]->name            = strdup(slx.name);
@@ -673,45 +676,49 @@ static slx_entry_t **skins_get_slx_entries(char *url) {
   return slxs;
 }
 
-static void download_skin_cancel(xitk_widget_t *w, void *data) {
+static void download_skin_exit(xitk_widget_t *w, void *data) {
   int i;
   
-  xitk_unregister_event_handler(&skdloader->widget_key);
-  
-  xitk_image_free_image(gGui->imlib_data, &skdloader->preview_image);
+  if(skdloader) {
+    xitk_unregister_event_handler(&skdloader->widget_key);
+    
+    xitk_image_free_image(gGui->imlib_data, &skdloader->preview_image);
+    
+    xitk_destroy_widgets(skdloader->widget_list);
+    xitk_window_destroy_window(gGui->imlib_data, skdloader->xwin);
+    
+    skdloader->xwin = NULL;
+    
+    xitk_list_free((XITK_WIDGET_LIST_LIST(skdloader->widget_list)));
+    
+    XLockDisplay(gGui->display);
+    XFreeCursor(gGui->display, skdloader->cursor[NORMAL_CURS]);
+    XFreeCursor(gGui->display, skdloader->cursor[WAIT_CURS]);
+    XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(skdloader->widget_list)));
+    XUnlockDisplay(gGui->display);
+    
+    free(skdloader->widget_list);
+    
+    for(i = 0; i < skdloader->num; i++) {
+      SAFE_FREE(skdloader->slxs[i]->name);
+      SAFE_FREE(skdloader->slxs[i]->author.name);
+      SAFE_FREE(skdloader->slxs[i]->author.email);
+      SAFE_FREE(skdloader->slxs[i]->skin.href);
+      SAFE_FREE(skdloader->slxs[i]->skin.preview);
+      free(skdloader->slxs[i]);
+      free(skdloader->entries[i]);
+    }
+    
+    free(skdloader->slxs);
+    free(skdloader->entries);
+    
+    skdloader->num = 0;
+    
+    free(skdloader);
+    skdloader = NULL;
 
-  xitk_destroy_widgets(skdloader->widget_list);
-  xitk_window_destroy_window(gGui->imlib_data, skdloader->xwin);
-
-  skdloader->xwin = NULL;
-
-  xitk_list_free((XITK_WIDGET_LIST_LIST(skdloader->widget_list)));
-  
-  XLockDisplay(gGui->display);
-  XFreeCursor(gGui->display, skdloader->cursor[NORMAL_CURS]);
-  XFreeCursor(gGui->display, skdloader->cursor[WAIT_CURS]);
-  XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(skdloader->widget_list)));
-  XUnlockDisplay(gGui->display);
-  
-  free(skdloader->widget_list);
-  
-  for(i = 0; i < skdloader->num; i++) {
-    SAFE_FREE(skdloader->slxs[i]->name);
-    SAFE_FREE(skdloader->slxs[i]->author.name);
-    SAFE_FREE(skdloader->slxs[i]->author.email);
-    SAFE_FREE(skdloader->slxs[i]->skin.href);
-    SAFE_FREE(skdloader->slxs[i]->skin.preview);
-    free(skdloader->slxs[i]);
-    free(skdloader->entries[i]);
+    try_to_set_input_focus(gGui->video_window);
   }
-  
-  free(skdloader->slxs);
-  free(skdloader->entries);
-
-  skdloader->num = 0;
-
-  free(skdloader);
-  skdloader = NULL;
 }
 
 static void download_update_blank_preview(void) {
@@ -727,38 +734,36 @@ static void download_update_blank_preview(void) {
 
 }
 
-static void download_update_preview(void) {
-
+static void redraw_preview(void) {
+  int  x, y;
+  
   download_update_blank_preview();
-
-  if(skdloader->preview_image->image && skdloader->preview_image->image->pixmap) {
-    GC   lgc;
-    int  x, y;
-    
+  
+  y = 30 + ((skdloader->preview_image->image->height - skdloader->preview_height) >> 1);
+  x = 10 + ((skdloader->preview_image->image->width - skdloader->preview_width) >> 1);
+  
+  if(skdloader->preview_image->mask && skdloader->preview_image->mask->pixmap) {
     XLockDisplay (gGui->display);
-    lgc = XCreateGC(gGui->display, (XITK_WIDGET_LIST_WINDOW(skdloader->widget_list)), None, None);
-    XCopyGC(gGui->display, 
-	    (XITK_WIDGET_LIST_GC(skdloader->widget_list)), (1 << GCLastBit) - 1, lgc);
-    XUnlockDisplay (gGui->display);
-    
-    y = 30 + ((skdloader->preview_image->image->height - skdloader->preview_height) >> 1);
-    x = 10 + ((skdloader->preview_image->image->width - skdloader->preview_width) >> 1);
-    
-    if(skdloader->preview_image->mask && skdloader->preview_image->mask->pixmap) {
-      XLockDisplay (gGui->display);
-      XSetClipOrigin(gGui->display, lgc, x, y);
-      XSetClipMask(gGui->display, lgc, skdloader->preview_image->mask->pixmap);
-      XUnlockDisplay (gGui->display);
-    }
-    
-    XLockDisplay (gGui->display);
-    XCopyArea (gGui->display, skdloader->preview_image->image->pixmap, 
-	       (XITK_WIDGET_LIST_WINDOW(skdloader->widget_list)), lgc, 0, 0,
-	       skdloader->preview_width, skdloader->preview_height, x, y);
-    XSync(gGui->display, False);
-    XFreeGC(gGui->display, lgc);
+    XSetClipOrigin(gGui->display, (XITK_WIDGET_LIST_GC(skdloader->widget_list)), x, y);
+    XSetClipMask(gGui->display, (XITK_WIDGET_LIST_GC(skdloader->widget_list)), skdloader->preview_image->mask->pixmap);
     XUnlockDisplay (gGui->display);
   }
+  
+  XLockDisplay (gGui->display);
+  XCopyArea (gGui->display, skdloader->preview_image->image->pixmap, 
+	     (XITK_WIDGET_LIST_WINDOW(skdloader->widget_list)), (XITK_WIDGET_LIST_GC(skdloader->widget_list)), 0, 0,
+	     skdloader->preview_width, skdloader->preview_height, x, y);
+  XSync(gGui->display, False);
+  XUnlockDisplay (gGui->display);
+}
+
+static void download_update_preview(void) {
+
+  if(skdloader->preview_image->image && skdloader->preview_image->image->pixmap)
+    redraw_preview();
+  else
+    download_update_blank_preview();
+
 }
 
 static void download_skin_preview(xitk_widget_t *w, void *data, int selected) {
@@ -785,7 +790,7 @@ static void download_skin_preview(xitk_widget_t *w, void *data, int selected) {
       skpname++;
     
     memset(&tmpfile, 0, sizeof(tmpfile));
-    sprintf(tmpfile, "/tmp/%d%s", (unsigned int)time(NULL), skpname);
+    snprintf(tmpfile, sizeof(tmpfile), "%s%d%s", "/tmp/", (unsigned int)time(NULL), skpname);
     
     if((fd = fopen(tmpfile, "w+b")) != NULL) {
       ImlibImage    *img = NULL;
@@ -886,7 +891,7 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
       filename++;
       
       memset(&skindir, 0, sizeof(skindir));
-      sprintf(skindir, "%s/.xine/skins", xine_get_homedir());
+      snprintf(skindir, sizeof(skindir), "%s%s", xine_get_homedir(), "/.xine/skins");
       
       /*
        * Make sure to have the directory
@@ -902,25 +907,36 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
 	FILE  *fd;
 	
 	memset(&tmpskin, 0, sizeof(tmpskin));
-	sprintf(tmpskin, "/tmp/%d%s", (unsigned int)time(NULL), filename);
+	snprintf(tmpskin, sizeof(tmpskin), "%s%d%s", "/tmp/", (unsigned int)time(NULL), filename);
 	
 	if((fd = fopen(tmpskin, "w+b")) != NULL) {
-	  char   buffer[2048];
-	  int    i, skin_found = -1;
-	  
+	  char      buffer[2048];
+	  char      fskin_path[XITK_PATH_MAX];
+	  int       i, skin_found = -1;
+
 	  fwrite(download.buf, download.size, 1, fd);
 	  fflush(fd);
 	  fclose(fd);
 
 	  memset(&buffer, 0, sizeof(buffer));
 	  
-	  sprintf(buffer, "which tar > /dev/null 2>&1 && tar -C %s -xzf %s", skindir, tmpskin);
+	  snprintf(buffer, sizeof(buffer), "%s %s %s %s %s", "which tar > /dev/null 2>&1 && cd ", skindir, " && gunzip -c ", tmpskin, " | tar xf -");
 	  xine_system(0, buffer);
 	  unlink(tmpskin);
-	  
+
 	  memset(&buffer, 0, sizeof(buffer));
 	  snprintf(buffer, ((strlen(filename) + 1) - 7), "%s", filename);
 
+	  memset(&fskin_path, 0, sizeof(fskin_path));
+	  snprintf(fskin_path, sizeof(fskin_path), "%s/%s/%s", skindir, buffer, "doinst.sh");
+	  if(is_a_file(fskin_path)) {
+	    char doinst[2048];
+
+	    memset(&doinst, 0, sizeof(doinst));
+	    snprintf(doinst, sizeof(doinst), "%s %s/%s %s", "cd", skindir, buffer, "&& ./doinst.sh");
+	    xine_system(0, doinst);
+	  }
+	  
 	  for(i = 0; i < skins_avail_num; i++) {
 	    if((!strcmp(skins_avail[i]->pathname, skindir)) 
 	       && (!strcmp(skins_avail[i]->skin, buffer))) {
@@ -930,8 +946,7 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
 	  }
 	  
 	  if(skin_found == -1) {
-	    skins_avail = (skins_locations_t **) realloc(skins_avail, 
-							 (skins_avail_num + 2) * sizeof(skins_locations_t*));
+	    skins_avail = (skins_locations_t **) realloc(skins_avail, (skins_avail_num + 2) * sizeof(skins_locations_t*));
 	    skin_names = (char **) realloc(skin_names, (skins_avail_num + 2) * sizeof(char *));
 	    
 	    skins_avail[skins_avail_num] = (skins_locations_t *) xine_xmalloc(sizeof(skins_locations_t));
@@ -979,60 +994,58 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
   if(download.error)
     free(download.error);
   
-  download_skin_cancel(w, NULL);
+  download_skin_exit(w, NULL);
 }
 
 static void download_skin_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
+
+  case Expose:
+    redraw_preview();
+    break;
     
   case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
+    KeySym         mkey;
     int            modifier;
     
-    mykeyevent = event->xkey;
-    
     xitk_get_key_modifier(event, &modifier);
+    mkey = xitk_get_key_pressed(event);
     
-    XLockDisplay(gGui->display);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUnlockDisplay(gGui->display);
-    
-    switch(mykey) {
+    switch(mkey) {
       
     case XK_Down:
-    case XK_Next: {
-      xitk_widget_t *w;
-      
-      w = xitk_get_focused_widget(skdloader->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Down)
-	  xitk_browser_step_up(skdloader->browser, NULL);
-	else
-	  xitk_browser_page_up(skdloader->browser, NULL);
+    case XK_Next:
+      {
+	xitk_widget_t *w;
+	
+	w = xitk_get_focused_widget(skdloader->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Down)
+	    xitk_browser_step_up(skdloader->browser, NULL);
+	  else
+	    xitk_browser_page_up(skdloader->browser, NULL);
+	}
       }
-    }
       break;
       
     case XK_Up:
-    case XK_Prior: {
-      xitk_widget_t *w;
-      
-      w = xitk_get_focused_widget(skdloader->widget_list);
-      if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
-	if(mykey == XK_Up)
-	  xitk_browser_step_down(skdloader->browser, NULL);
-	else
-	  xitk_browser_page_down(skdloader->browser, NULL);
+    case XK_Prior:
+      {
+	xitk_widget_t *w;
+	
+	w = xitk_get_focused_widget(skdloader->widget_list);
+	if((!w) || (w && (!((xitk_get_widget_type(w)) & WIDGET_GROUP_BROWSER)))) {
+	  if(mkey == XK_Up)
+	    xitk_browser_step_down(skdloader->browser, NULL);
+	  else
+	    xitk_browser_page_down(skdloader->browser, NULL);
+	}
       }
-    }
       break;
-     
+      
     case XK_Escape:
-      download_skin_cancel(NULL, NULL);
+      download_skin_exit(NULL, NULL);
       break;
     }
   }
@@ -1040,8 +1053,13 @@ static void download_skin_handle_event(XEvent *event, void *data) {
 
   }
 }
+
+void download_skin_end(void) {
+  download_skin_exit(NULL, NULL);
+}
     
 void download_skin(char *url) {
+#ifdef HAVE_TAR
   slx_entry_t         **slxs;
   xitk_window_t        *xwin;
   xitk_pixmap_t        *bg;
@@ -1062,7 +1080,7 @@ void download_skin(char *url) {
   while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(xwin)))
     xine_usec_sleep(5000);
   
-  if(!gGui->use_root_window) {
+  if(!gGui->use_root_window && gGui->video_display == gGui->display) {
     XLockDisplay(gGui->display);
     XSetTransientForHint (gGui->display, 
 			  xitk_window_get_window(xwin), gGui->video_window);
@@ -1071,7 +1089,7 @@ void download_skin(char *url) {
   layer_above_video(xitk_window_get_window(xwin));
   
   if((slxs = skins_get_slx_entries(url)) != NULL) {
-    char                      *fontname    = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+    char                      *fontname    = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
     char                      *btnfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
     int                        i;
     xitk_browser_widget_t      br;
@@ -1108,11 +1126,13 @@ void download_skin(char *url) {
 						       _("Choose a skin to download..."),
 						       x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
     
+    set_window_states_start((xitk_window_get_window(skdloader->xwin)));
+
     XLockDisplay(gGui->display);
     skdloader->cursor[NORMAL_CURS] = XCreateFontCursor(gGui->display, XC_left_ptr);
     skdloader->cursor[WAIT_CURS] = XCreateFontCursor(gGui->display, XC_watch);
     gc = XCreateGC(gGui->display, 
-		   (xitk_window_get_window(skdloader->xwin)), None, None);
+    		   (xitk_window_get_window(skdloader->xwin)), None, None);
     XUnlockDisplay(gGui->display);
 
     xitk_window_get_window_size(skdloader->xwin, &width, &height);
@@ -1133,10 +1153,7 @@ void download_skin(char *url) {
     i = 0;
     while(slxs[i]) {
 
-      if(i == 0)
-	skdloader->entries = (char **) xine_xmalloc(sizeof(char *) * 2);
-      else
-	skdloader->entries = (char **) realloc(skdloader->entries, sizeof(char *) * (i + 2));
+      skdloader->entries = (char **) realloc(skdloader->entries, sizeof(char *) * (i + 2));
       
       skdloader->entries[i] = strdup(slxs[i]->name);
       i++;
@@ -1168,7 +1185,7 @@ void download_skin(char *url) {
 							 WINDOW_WIDTH - (20 + 10) , 20, 12, fontname)));
     
     xitk_browser_update_list(skdloader->browser, 
-    			     (const char *const *)skdloader->entries, skdloader->num, 0);
+    			     (const char *const *)skdloader->entries, NULL, skdloader->num, 0);
     
     xitk_enable_and_show_widget(skdloader->browser);
     
@@ -1198,7 +1215,7 @@ void download_skin(char *url) {
     lb.button_type       = CLICK_BUTTON;
     lb.label             = _("Cancel");
     lb.align             = ALIGN_CENTER;
-    lb.callback          = download_skin_cancel; 
+    lb.callback          = download_skin_exit; 
     lb.state_callback    = NULL;
     lb.userdata          = NULL;
     lb.skin_element_name = NULL;
@@ -1219,7 +1236,7 @@ void download_skin(char *url) {
     XLockDisplay(gGui->display);
     XRaiseWindow(gGui->display, xitk_window_get_window(skdloader->xwin));
     XMapWindow(gGui->display, xitk_window_get_window(skdloader->xwin));
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint (gGui->display, 
 			    xitk_window_get_window(skdloader->xwin), gGui->video_window);
     XUnlockDisplay(gGui->display);
@@ -1230,5 +1247,6 @@ void download_skin(char *url) {
   }
   else
     xitk_window_dialog_destroy(xwin);
-  
+
+#endif /* HAVE_TAR */
 }
