@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: stream_infos.c,v 1.33 2003/10/29 12:32:16 f1rmb Exp $
+ * $Id: stream_infos.c,v 1.39 2004/07/02 09:33:38 f1rmb Exp $
  *
  *
  */
@@ -129,7 +129,7 @@ char *get_fourcc_string(uint32_t f) {
   fcc[4] = 0;
   
   if(f <= 0xFFFF)
-    sprintf(fcc, "0x%x", f);
+    snprintf(fcc, sizeof(fcc), "0x%x", f);
   
   if((fcc[0] == 'm') && (fcc[1] == 's')) {
     if((fcc[2] = 0x0) && (fcc[3] == 0x55)) {
@@ -144,7 +144,7 @@ static char *get_num_string(uint32_t num) {
   static char buffer[1024];
 
   memset(&buffer, 0, sizeof(buffer));
-  snprintf(buffer, 1023, "%d", num);
+  snprintf(buffer, sizeof(buffer), "%d", num);
 
   return &buffer[0];
 }
@@ -186,7 +186,7 @@ static void get_stream_video_resolution_info(void) {
   video_w = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_WIDTH);
   video_h = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
 
-  sprintf(buffer, "%d X %d", video_w, video_h);
+  snprintf(buffer, sizeof(buffer), "%d X %d", video_w, video_h);
   set_label(sinfos->infos.video_resolution, buffer);
 }
 
@@ -280,6 +280,18 @@ static void stream_infos_exit(xitk_widget_t *w, void *data) {
     
     free(sinfos);
     sinfos = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
+  }
+}
+
+static void handle_event(XEvent *event, void *data) {
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      stream_infos_exit(NULL, NULL);
+    break;
   }
 }
 
@@ -418,6 +430,11 @@ void stream_infos_update_infos(void) {
   }
 }
 
+void stream_infos_reparent(void) {
+  if(sinfos)
+    reparent_window((xitk_window_get_window(sinfos->xwin)));
+}
+
 void stream_infos_panel(void) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
@@ -448,6 +465,8 @@ void stream_infos_panel(void) {
   sinfos->xwin = xitk_window_create_dialog_window(gGui->imlib_data, _("Stream Information"), x, y,
 						  WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(sinfos->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(sinfos->xwin)), None, None);
@@ -1031,7 +1050,7 @@ void stream_infos_panel(void) {
 
   sinfos->widget_key = xitk_register_event_handler("sinfos", 
 						   (xitk_window_get_window(sinfos->xwin)),
-						   NULL,
+						   handle_event,
 						   NULL,
 						   NULL,
 						   sinfos->widget_list,
