@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.c,v 1.42 2003/12/07 23:20:41 f1rmb Exp $
+ * $Id: viewlog.c,v 1.50 2004/07/02 09:33:38 f1rmb Exp $
  *
  *
  */
@@ -69,12 +69,10 @@ typedef struct {
 
 static _viewlog_t    *viewlog = NULL;
 
-static void viewlog_end(xitk_widget_t *, void *);
-
 /*
  * Leaving setup panel, release memory.
  */
-void viewlog_exit(xitk_widget_t *w, void *data) {
+static void viewlog_exit(xitk_widget_t *w, void *data) {
 
   if(viewlog) {
     window_info_t wi;
@@ -104,6 +102,8 @@ void viewlog_exit(xitk_widget_t *w, void *data) {
     
     free(viewlog);
     viewlog = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -277,7 +277,7 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
     printf("-----------------------------------\n\n");
   }
   
-  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->real_num_entries, 0);
+  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, NULL, viewlog->real_num_entries, 0);
   
   viewlog_clear_tab();
   viewlog_paint_widgets();
@@ -342,10 +342,25 @@ static void viewlog_create_tabs(void) {
 /*
  * Leave viewlog window.
  */
-static void viewlog_end(xitk_widget_t *w, void *data) {
+void viewlog_end(void) {
   viewlog_exit(NULL, NULL);
 }
 
+static void viewlog_handle_event(XEvent *event, void *data) {
+  switch(event->type) {
+    
+  case KeyPress:
+    if(xitk_get_key_pressed(event) == XK_Escape)
+      viewlog_exit(NULL, NULL);
+    break;
+    
+  }
+}
+
+void viewlog_reparent(void) {
+  if(viewlog)
+    reparent_window((xitk_window_get_window(viewlog->xwin)));
+}
 
 /*
  * Create viewlog window
@@ -380,6 +395,8 @@ void viewlog_panel(void) {
 						   _("xine log viewer"), 
 						   x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
   
+  set_window_states_start((xitk_window_get_window(viewlog->xwin)));
+
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(viewlog->xwin)), None, None);
@@ -415,7 +432,7 @@ void viewlog_panel(void) {
   xitk_enable_and_show_widget(viewlog->browser_widget);
 
   xitk_browser_set_alignment(viewlog->browser_widget, ALIGN_LEFT);
-  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->real_num_entries, 0);
+  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, NULL, viewlog->real_num_entries, 0);
   
   viewlog_paint_widgets();
   
@@ -442,7 +459,7 @@ void viewlog_panel(void) {
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = ALIGN_CENTER;
-  lb.callback          = viewlog_end; 
+  lb.callback          = viewlog_exit; 
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
@@ -454,7 +471,7 @@ void viewlog_panel(void) {
 
   viewlog->kreg = xitk_register_event_handler("viewlog", 
 					      (xitk_window_get_window(viewlog->xwin)),
-					      NULL,
+					      viewlog_handle_event,
 					      NULL,
 					      NULL,
 					      viewlog->widget_list,
