@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: _xitk.h,v 1.70 2003/12/13 01:09:39 f1rmb Exp $
+ * $Id: _xitk.h,v 1.83 2004/07/02 13:23:31 f1rmb Exp $
  *
  *
  */
@@ -46,6 +46,10 @@
 #ifdef HAVE_XSHM_H
 #include <X11/extensions/XShm.h>
 #endif
+#ifdef WITH_XFT
+#include <ft2build.h>
+#include <X11/Xft/Xft.h>
+#endif
 
 #include "xitk/Imlib-light/Imlib.h"
 
@@ -63,7 +67,6 @@ typedef struct {
 #include "combo.h"
 #include "dnd.h"
 #include "doublebox.h"
-#include "filebrowser.h"
 #include "font.h"
 #include "image.h"
 #include "inputtext.h"
@@ -265,7 +268,11 @@ void xitk_strdupa(char *dest, char *src);
 #define DIRECTION_LEFT   3
 #define DIRECTION_RIGHT  4
 
-int xitk_x_error;
+#define CHECK_STYLE_OLD   1
+#define CHECK_STYLE_CHECK 2
+#define CHECK_STYLE_ROUND 3
+
+extern int xitk_x_error;
 
 #ifndef _XITK_C_
 
@@ -309,15 +316,20 @@ void xitk_subst_special_chars(char *, char *);
 unsigned long xitk_get_timer_label_animation(void);
 long int xitk_get_timer_dbl_click(void);
 int xitk_get_barstyle_feature(void);
+int xitk_get_checkstyle_feature(void);
 unsigned long xitk_get_warning_foreground(void);
 unsigned long xitk_get_warning_background(void);
 void xitk_modal_window(Window w);
 void xitk_unmodal_window(Window w);
 void xitk_set_current_menu(xitk_widget_t *menu);
 void xitk_unset_current_menu(void);
+int xitk_get_display_width(void);
+int xitk_get_display_height(void);
 
+int xitk_is_running(void);
 int xitk_skin_get_direction(xitk_skin_config_t *, const char *);
 int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
+int xitk_skin_get_printability(xitk_skin_config_t *, const char *);
 int xitk_skin_get_enability(xitk_skin_config_t *, const char *);
 int xitk_skin_get_coord_x(xitk_skin_config_t *, const char *);
 int xitk_skin_get_coord_y(xitk_skin_config_t *, const char *);
@@ -344,11 +356,17 @@ void xitk_skin_unlock(xitk_skin_config_t *);
 void *labelbutton_get_user_data(xitk_widget_t *w);
 void menu_auto_pop(xitk_widget_t *w);
 
+int xitk_get_bool_value(const char *val);
+
+
 struct xitk_font_s {
   Display       *display;
-#ifdef WITH_XMB
-  XFontSet       fontset;
+#ifdef WITH_XFT
+  XftFont       *font;
 #else
+# ifdef WITH_XMB
+  XFontSet       fontset;
+# endif
   XFontStruct   *font;
 #endif
   char          *name;
@@ -358,6 +376,7 @@ typedef struct xitk_dialog_s xitk_dialog_t;
 
 struct xitk_window_s {
   Window                    window;
+  Window                    win_parent;
   xitk_pixmap_t            *background;
   int                       width;
   int                       height;
@@ -375,6 +394,8 @@ struct xitk_dialog_s {
   xitk_widget_t          *wyes;
   xitk_widget_t          *wno;
   xitk_widget_t          *wcancel;
+  xitk_widget_t          *checkbox;
+  xitk_widget_t          *checkbox_label;
 
   xitk_widget_t          *default_button;
 
