@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: browser.c,v 1.59 2003/12/14 14:10:33 f1rmb Exp $
+ * $Id: browser.c,v 1.63 2004/06/01 21:07:04 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -317,9 +317,17 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
+    char         *max_shortcut = NULL;
+    int           item_width;
+    char         *label_font;
+    xitk_font_t  *fs = NULL;
+    int           old_label_offset;
     
-    private_data = (browser_private_data_t *) w->private_data;
-    
+    private_data     = (browser_private_data_t *) w->private_data;
+    item_width       = xitk_get_widget_width(private_data->item_tree[WBSTART]);
+    label_font       = xitk_labelbutton_get_fontname(private_data->item_tree[WBSTART]);
+    old_label_offset = private_data->labels_offset;
+
     xitk_browser_release_all_buttons(w);
 
     j = (private_data->list_length > (private_data->max_length - 1) ? 
@@ -329,10 +337,10 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
       max = 0;
     else if(j > (private_data->max_length - 1))
       max = j - (private_data->max_length - 1);
-    
+
     if(((start >= 0) && max) || ((start == 0) && (max == 0)))
       private_data->current_start = start;
-
+    
     private_data->need_h_slider = 0;
     xitk_disable_widget(private_data->item_tree[WBLF]);
     xitk_disable_widget(private_data->item_tree[WSLIDH]);
@@ -340,79 +348,134 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
     xitk_disable_widget(private_data->item_tree[WBUP]);
     xitk_disable_widget(private_data->item_tree[WSLID]);
     xitk_disable_widget(private_data->item_tree[WBDN]);
-
+     
     private_data->labels_offset = 0;
 
-    for(i = 0; i < private_data->max_length; i++) {
-      xitk_labelbutton_set_label_offset(private_data->item_tree[i + WBSTART], 0);
+    if(label_font) {
+      fs = xitk_font_load_font(private_data->imlibdata->x.disp, label_font);
+      xitk_font_set_font(fs, private_data->parent_wlist->gc);
+    }
+
+    if(private_data->shortcuts_content) {
+      int maxlen = 0;
+
+      for(i = 0; i < private_data->max_length; i++) {
+	char *nshortcut;
+
+	if((private_data->shortcuts_content[private_data->current_start + i]) && 
+	   (nshortcut = private_data->shortcuts_content[private_data->current_start + i])) {
+	  int nlen = strlen(nshortcut);
+
+	  if(nlen > maxlen) {
+	    maxlen = nlen;
+	    max_shortcut = nshortcut;
+	  }
+	}
+      }
+    }
+
+    if(fs) {
+      int  label_width;
+      int  j = 0;
+      int  max = 0, offset = 0, len;
+      
+      while(j < private_data->list_length) {
+	if((len = strlen(private_data->content[j])) > max) {
+	  max    = len;
+	  offset = j;
+	}
+	j++;
+      }
+      
+      label_width = (private_data->content && private_data->content[offset] && strlen(private_data->content[offset])) ? xitk_font_get_string_length(fs, private_data->content[offset]) : 0;
+      
+      if(label_width > (item_width -2)) {
+	int diff_width = (label_width - (item_width - 2));
+	
+	private_data->need_h_slider = 1;
+	
+	if(private_data->labels_offset < diff_width)
+	  private_data->labels_offset = diff_width;
+	 
+      }
+      else
+	private_data->need_h_slider = 0;
+    }
+    
+    if(!private_data->need_h_slider) {
+      for(i = 0; i < private_data->max_length; i++)
+	xitk_labelbutton_set_label_offset(private_data->item_tree[i + WBSTART], 0);
     }
     
     for(i = 0; i < private_data->max_length; i++) {
       if (((private_data->current_start + i) < private_data->list_length) 
 	  && (private_data->content[private_data->current_start + i] != NULL)) {
-	int item_width = xitk_get_widget_width(private_data->item_tree[WBSTART]);
-	char *label_font = xitk_labelbutton_get_fontname(private_data->item_tree[WBSTART]);
-	
-	xitk_labelbutton_change_label(private_data->item_tree[i + WBSTART], 
+
+	xitk_labelbutton_change_label(private_data->item_tree[i + WBSTART],
 				      private_data->content[private_data->current_start + i]);
-	
-	if(label_font) {
-	  xitk_font_t *fs = NULL;
-	  int          label_width;
-	  
-	  fs = xitk_font_load_font(private_data->imlibdata->x.disp, label_font);
-	  xitk_font_set_font(fs, private_data->parent_wlist->gc);
-	  
-	  label_width = (private_data->content[private_data->current_start + i] && 
-			 strlen(private_data->content[private_data->current_start + i])) ? 
-	    xitk_font_get_string_length(fs, private_data->content[private_data->current_start + i]) : 0;
-	  xitk_font_unload_font(fs);
-	  
-	  if(label_width > (item_width - 2)) {
-	    int diff_width = (label_width - (item_width - 2));
-	    
-	    private_data->need_h_slider = 1;
-	    
-	    if(private_data->labels_offset < diff_width)
-	      private_data->labels_offset = diff_width;
-	    
-	  }
-	  XITK_FREE(label_font);
+
+	if(private_data->shortcuts_content) {
+	  if(private_data->shortcuts_content[private_data->current_start + i])
+	    xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART],
+						   private_data->shortcuts_content[private_data->current_start + i], 
+						   item_width - (xitk_font_get_string_length(fs, private_data->shortcuts_content[private_data->current_start + i]) + 10),
+						   NULL);
+	  else
+	    xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], "", -1, NULL);
 	}
+	
       }
       else {
 	xitk_labelbutton_change_label(private_data->item_tree[i+WBSTART], "");
+	xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], "", -1, NULL);
       }
+
     }
-
+    
+    XITK_FREE(label_font);
+    
+    if(fs)
+      xitk_font_unload_font(fs);
+     
     if(private_data->need_h_slider) {
-      int align = xitk_labelbutton_get_alignment(private_data->item_tree[WBSTART]);
-
-      xitk_enable_widget(private_data->item_tree[WBLF]);
+      int npos, pos = xitk_slider_get_pos(private_data->item_tree[WSLIDH]);
+      
       xitk_enable_widget(private_data->item_tree[WSLIDH]);
+      xitk_enable_widget(private_data->item_tree[WBLF]);
       xitk_enable_widget(private_data->item_tree[WBRT]);
+      
+      if(private_data->labels_offset != old_label_offset) {
+	int align = xitk_labelbutton_get_alignment(private_data->item_tree[WBSTART]);
 
-      switch(align) {
-      case ALIGN_CENTER:
-	xitk_slider_set_min(private_data->item_tree[WSLIDH], -(private_data->labels_offset>>1));
-	xitk_slider_set_max(private_data->item_tree[WSLIDH], (private_data->labels_offset>>1));
-	break;
-      case ALIGN_LEFT:
-	xitk_slider_set_min(private_data->item_tree[WSLIDH], 0);
-	xitk_slider_set_max(private_data->item_tree[WSLIDH], private_data->labels_offset);
-	break;
-      case ALIGN_RIGHT:
-	xitk_slider_set_min(private_data->item_tree[WSLIDH], (-private_data->labels_offset) - 3);
-	xitk_slider_set_max(private_data->item_tree[WSLIDH], 0);
-	break;
+	switch(align) {
+	case ALIGN_CENTER:
+	  xitk_slider_set_min(private_data->item_tree[WSLIDH], -(private_data->labels_offset>>1));
+	  xitk_slider_set_max(private_data->item_tree[WSLIDH], (private_data->labels_offset>>1));
+	  break;
+	case ALIGN_LEFT:
+	  xitk_slider_set_min(private_data->item_tree[WSLIDH], 0);
+	  xitk_slider_set_max(private_data->item_tree[WSLIDH], private_data->labels_offset);
+	  break;
+	case ALIGN_RIGHT:
+	  xitk_slider_set_min(private_data->item_tree[WSLIDH], (-private_data->labels_offset) - 3);
+	  xitk_slider_set_max(private_data->item_tree[WSLIDH], 0);
+	  break;
+	}
       }
+
+      if((pos > (npos = xitk_slider_get_max(private_data->item_tree[WSLIDH]))) || 
+      	 (pos < (npos = xitk_slider_get_min(private_data->item_tree[WSLIDH]))))
+	pos = npos;
+
+      xitk_slider_set_pos(private_data->item_tree[WSLIDH], pos);
     }
     else {
       xitk_slider_set_max(private_data->item_tree[WSLIDH], 1);
       xitk_slider_set_min(private_data->item_tree[WSLIDH], 0);
+      xitk_slider_reset(private_data->item_tree[WSLIDH]);
     }
-    xitk_slider_reset(private_data->item_tree[WSLIDH]);
      
+
     xitk_slider_set_max(private_data->item_tree[WSLID], max);
 
     if(private_data->list_length >= private_data->max_length) {
@@ -436,7 +499,7 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
 /**
  * Update the list, and rebuild button list
  */
-void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, int len, int start) {
+void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, const char *const *shortcut, int len, int start) {
   browser_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
@@ -444,6 +507,7 @@ void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, int len
     
     private_data = (browser_private_data_t *) w->private_data;
     private_data->content = (char **)list;
+    private_data->shortcuts_content = (char **)shortcut;
     private_data->list_length = len;
     xitk_browser_rebuild_browser(w, start);
   }
@@ -493,11 +557,15 @@ static void browser_hslidmove(xitk_widget_t *w, void *data, int pos) {
 	  
 	  xitk_labelbutton_change_label(private_data->item_tree[i + WBSTART], 
 					private_data->content[private_data->current_start + i]);
+	  if(private_data->shortcuts_content)
+	    xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], 
+						   private_data->shortcuts_content[private_data->current_start + i], -1, NULL);
 	  
 	}
-	else
+	else {
 	  xitk_labelbutton_change_label(private_data->item_tree[i+WBSTART], "");
-	
+	  xitk_labelbutton_change_shortcut_label(private_data->item_tree[i + WBSTART], "", -1, NULL);
+	}
       }
     }
     else
@@ -931,6 +999,7 @@ static xitk_widget_t *_xitk_browser_create(xitk_widget_list_t *wl,
   private_data->skin_element_name    = (skin_element_name == NULL) ? NULL : strdup(br->browser.skin_element_name);
   private_data->jumped               = -1;
   private_data->content              = (char **)br->browser.entries;
+  private_data->shortcuts_content    = NULL;
   private_data->list_length          = br->browser.num_entries;
   private_data->max_length           = br->browser.max_displayed_entries;
 
