@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.59 2003/12/23 23:47:51 f1rmb Exp $
+ * $Id: inputtext.c,v 1.65 2004/06/13 11:44:18 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -73,8 +73,10 @@ static void notify_destroy(xitk_widget_t *w) {
     XFreeCursor(private_data->imlibdata->x.disp, private_data->cursor[1]);
     XUNLOCK(private_data->imlibdata->x.disp);
 
+    if(!private_data->skin_element_name)
+      xitk_image_free_image(private_data->imlibdata, &(private_data->skin));
+    
     XITK_FREE(private_data->skin_element_name);
-    xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data->text);
     XITK_FREE(private_data->normal_color);
     XITK_FREE(private_data->focused_color);
@@ -161,6 +163,32 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier) {
   return (*modifier != MODIFIER_NOMOD);
 }
 
+int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[]) {
+  int len = 0;
+  if(event) {
+    XKeyEvent  pkeyev = event->xkey;
+    
+    XLOCK(pkeyev.display);
+    len = XLookupString(&pkeyev, kbuf, sizeof(kbuf), ksym, NULL);
+    XUNLOCK(pkeyev.display);
+  }
+  return len;
+}
+
+/*
+ * Return key pressed (XK_VoidSymbol on failure)
+ */
+KeySym xitk_get_key_pressed(XEvent *event) {
+  KeySym   pkey = XK_VoidSymbol;
+  
+  if(event) {
+    char  buf[256];
+    (void) xitk_get_keysym_and_buf(event, &pkey, buf);
+  }
+
+  return pkey;
+}
+
 /*
  * Recalculate display offsets.
  */
@@ -497,7 +525,7 @@ static int notify_click_inputtext(xitk_widget_t *w, int button, int bUp, int x,
 
     if((p = private_data->text)) {
       xitk_font_t *fs = NULL;
-      int          width = 0, i = 1;
+      int          width = 0, i = 0;
       int          max_len;
       
       p += private_data->disp_offset;
@@ -515,16 +543,20 @@ static int notify_click_inputtext(xitk_widget_t *w, int button, int bUp, int x,
       
       max_len = xitk_font_get_text_width(fs, p, strlen(p));
       
-      if(pos > max_len)
-	pos = max_len + 1;
-      
-      while(width < pos) {
-	width = xitk_font_get_text_width(fs, p, i);
-	i++;
+      if(pos > max_len) {
+	/* cursor behind the last character */
+	i = strlen(p) + 1;
+      } else {
+	/* cursor in the text */
+        /* FIXME: implement multibyte string support */
+        while(width < pos) {
+          width = xitk_font_get_text_width(fs, p, i + 1);
+          i++;
+	}
       }
 
       xitk_font_unload_font(fs);
-      pos = (i - 2) + private_data->disp_offset;
+      pos = (i - 1) + private_data->disp_offset;
       
       if(pos > strlen(private_data->text))
 	pos = strlen(private_data->text);
@@ -578,9 +610,8 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
       XITK_FREE(private_data->fontname);
       private_data->fontname      = strdup(xitk_skin_get_label_fontname(skonfig, private_data->skin_element_name));
       
-      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
-      private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
-							  xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
+      private_data->skin          = xitk_skin_get_image(skonfig,
+							xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       
       private_data->max_visible   = (private_data->skin->width/2);
       
@@ -834,7 +865,6 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
   inputtext_private_data_t *private_data;
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
-    XKeyEvent   keyevent = xev->xkey;
     KeySym      key;
     char        buf[256];
     int         len;
@@ -844,10 +874,7 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
 
     private_data->have_focus = FOCUS_RECEIVED;
 
-    XLOCK(private_data->imlibdata->x.disp);
-    len = XLookupString(&keyevent, buf, sizeof(buf), &key, NULL);
-    XUNLOCK(private_data->imlibdata->x.disp);
-
+    len = xitk_get_keysym_and_buf(xev, &key, buf);
     buf[len] = '\0';
 
     (void) xitk_get_key_modifier(xev, &modifier);
@@ -1180,8 +1207,8 @@ xitk_widget_t *xitk_inputtext_create (xitk_widget_list_t *wl,
 				 (xitk_skin_get_coord_x(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_coord_y(skonfig, it->skin_element_name)),
 				 it->skin_element_name,
-				 (xitk_image_load_image(it->imlibdata, 
-							xitk_skin_get_skin_filename(skonfig, it->skin_element_name))),
+				 (xitk_skin_get_image(skonfig,
+						      xitk_skin_get_skin_filename(skonfig, it->skin_element_name))),
 				 (xitk_skin_get_label_fontname(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_label_color(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_label_color_focus(skonfig, it->skin_element_name)),
@@ -1202,6 +1229,5 @@ xitk_widget_t *xitk_noskin_inputtext_create (xitk_widget_list_t *wl,
   i = xitk_image_create_image(it->imlibdata, width * 2, height);
   draw_bevel_two_state(it->imlibdata, i);
   
-
   return _xitk_inputtext_create(wl, NULL, it, x, y, NULL, i, fontname, ncolor, fcolor, 0, 0);
 }
