@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.c,v 1.58 2003/12/14 14:10:33 f1rmb Exp $
+ * $Id: label.c,v 1.69 2004/06/13 11:44:18 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -30,6 +30,10 @@
 #include <pthread.h>
 #include <unistd.h>
 
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
+
 #include "_xitk.h"
 
 static void _create_label_pixmap(xitk_widget_t *w) {
@@ -43,7 +47,7 @@ static void _create_label_pixmap(xitk_widget_t *w) {
   private_data->anim_offset = 0;
 
   if(private_data->animation) {
-    _label = (char *) alloca((strlen(private_data->label) * 2) + 5 + 1);
+    _label = (char *) xitk_xmalloc((strlen(private_data->label) * 2) + 5 + 1);
     
     if((strlen(private_data->label)) > private_data->length)
       sprintf(_label, "%s *** %s", private_data->label, private_data->label);
@@ -52,27 +56,24 @@ static void _create_label_pixmap(xitk_widget_t *w) {
     
   }
   else
-    xitk_strdupa(_label, private_data->label);
+    _label = strdup(private_data->label);
   
-  len = strlen(_label);
-  pixwidth = private_data->char_length * 
-    ((private_data->length * ((len / private_data->length) + 1)) + 5);
-
+  len                    = strlen(_label);
+  pixwidth               = private_data->char_length * ((private_data->length * ((len / private_data->length) + 1)) + 5);
   private_data->labelpix = xitk_image_create_xitk_pixmap(private_data->imlibdata,
-							 (pixwidth) ? pixwidth : 1, 
-							 private_data->char_height);
+							 (pixwidth) ? pixwidth : 1, private_data->char_height);
 
   x_dest = 0;
-    
-  for (i = 0; i < pixwidth; i++) {
+
+  for (i = 0; i < len; i++) {
     int c = 0;
     
     if ((i < len) && (_label[i] >= 32))
       c = _label[i] - 32;
-    
+
     if (c >= 0) {
       int px, py;
-      
+         
       px = (c % 32) * private_data->char_length;
       py = (c / 32) * private_data->char_height;
       
@@ -86,7 +87,20 @@ static void _create_label_pixmap(xitk_widget_t *w) {
     
     x_dest += private_data->char_length;
   }
+  
+  /* fill gap with spaces */
+  if(len < private_data->length) {
+    XLOCK(private_data->imlibdata->x.disp);
+    for(; i < private_data->length; i++) {
+      XCopyArea(private_data->imlibdata->x.disp, font->image->pixmap, 
+		private_data->labelpix->pixmap, private_data->labelpix->gc, 0, 0,
+		private_data->char_length, private_data->char_height, x_dest, 0);
+      x_dest += private_data->char_length;
+    }
+    XUNLOCK(private_data->imlibdata->x.disp);
+  }
 
+  free(_label);
 }
 
 /*
@@ -96,8 +110,9 @@ static void notify_destroy(xitk_widget_t *w) {
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
     label_private_data_t *private_data = (label_private_data_t *) w->private_data;
-    
-    private_data->on_change = 1;
+
+    pthread_mutex_lock(&private_data->paint_mutex);
+    pthread_mutex_lock(&private_data->change_mutex);
     
     if(private_data->anim_running) {
       void *dummy;
@@ -111,14 +126,15 @@ static void notify_destroy(xitk_widget_t *w) {
       private_data->labelpix = NULL;
     }
     
+    if(!private_data->skin_element_name)
+      xitk_image_free_image(private_data->imlibdata, &(private_data->font));
+
     XITK_FREE(private_data->label);
     XITK_FREE(private_data->fontname);
     XITK_FREE(private_data->skin_element_name);
 
-    if(private_data->font)
-      xitk_image_free_image(private_data->imlibdata, &private_data->font);
-
     pthread_mutex_destroy(&private_data->paint_mutex);
+    pthread_mutex_destroy(&private_data->change_mutex);
 
     XITK_FREE(private_data);
   }
@@ -161,7 +177,10 @@ static void paint_label(xitk_widget_t *w) {
   if (w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL) && (w->visible == 1))) {
     label_private_data_t  *private_data = (label_private_data_t *) w->private_data;
     xitk_image_t          *font = (xitk_image_t *) private_data->font;
-
+    
+    if(!private_data->label_visible)
+      return;
+    
     pthread_mutex_lock(&private_data->paint_mutex);
     
     /* non skinable widget */
@@ -206,19 +225,21 @@ static void paint_label(xitk_widget_t *w) {
     else {
       int width = private_data->char_length * private_data->length;
       
-
       XLOCK(private_data->imlibdata->x.disp);
-      XCopyArea(private_data->imlibdata->x.disp, 
+      XCopyArea(private_data->imlibdata->x.disp,
 		private_data->labelpix->pixmap, w->wl->win, font->image->gc, 
 		private_data->anim_offset, 0, width, private_data->char_height, 
 		w->x, w->y);
-
-      /* We can't wait here, otherwise the rolling effect is really jerky */
-      if(private_data->animation && (private_data->on_change == 0))
-	XSync(private_data->imlibdata->x.disp, False);
       
+      /* We can't wait here, otherwise the rolling effect is really jerky */
+      if(private_data->animation) {
+	if(!pthread_mutex_trylock(&private_data->change_mutex)) {
+	  XSync(private_data->imlibdata->x.disp, False);
+	  pthread_mutex_unlock(&private_data->change_mutex);
+	}
+      }
       XUNLOCK(private_data->imlibdata->x.disp);
-
+      
     }
     
     pthread_mutex_unlock(&private_data->paint_mutex);
@@ -228,7 +249,7 @@ static void paint_label(xitk_widget_t *w) {
 /*
  *
  */
-void *xitk_label_animation_loop(void *data) {
+static void *xitk_label_animation_loop(void *data) {
   label_private_data_t *private_data = (label_private_data_t *)data;
   xitk_widget_t        *w            = private_data->lWidget;
   unsigned long         t_anim       = private_data->anim_timer;
@@ -250,8 +271,10 @@ void *xitk_label_animation_loop(void *data) {
        * Label will change sooner, don't try to paint it till the change,
        * otherwise a deadlock will happened.
        */
-      if(private_data->on_change == 0)
+      if(!pthread_mutex_trylock(&private_data->change_mutex)) {
 	paint_label(private_data->lWidget);
+	pthread_mutex_unlock(&private_data->change_mutex);
+      }
 
     }
     xitk_usec_sleep(t_anim);
@@ -270,7 +293,7 @@ static void label_setup_label(xitk_widget_t *w, char *label_) {
   
   /* Inform animation thread to not paint the label */
   pthread_mutex_lock(&private_data->paint_mutex);
-  private_data->on_change = 1;
+  pthread_mutex_lock(&private_data->change_mutex);
   
   if(private_data->anim_running) {
     void *dummy;
@@ -320,7 +343,7 @@ static void label_setup_label(xitk_widget_t *w, char *label_) {
     }
   }
 
-  private_data->on_change = 0;
+  pthread_mutex_unlock(&private_data->change_mutex);
   pthread_mutex_unlock(&private_data->paint_mutex);
 }
 
@@ -336,22 +359,23 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
       
       xitk_skin_lock(skonfig);
 
-      xitk_image_free_image(private_data->imlibdata, &private_data->font);
-      private_data->font        = xitk_image_load_image(private_data->imlibdata, 
-							xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
-      private_data->char_length = (private_data->font->width/32);
-      private_data->char_height = (private_data->font->height/3);
+      private_data->font          = xitk_skin_get_image(skonfig,
+							xitk_skin_get_label_skinfont_filename(skonfig, 
+											      private_data->skin_element_name));
+      private_data->char_length   = (private_data->font->width/32);
+      private_data->char_height   = (private_data->font->height/3);
       
-      private_data->length      = xitk_skin_get_label_length(skonfig, private_data->skin_element_name);
-      private_data->animation   = xitk_skin_get_label_animation(skonfig, private_data->skin_element_name);
-      private_data->anim_step   = xitk_skin_get_label_animation_step(skonfig, private_data->skin_element_name);
-      private_data->anim_timer  = xitk_skin_get_label_animation_timer(skonfig, private_data->skin_element_name);
-      w->x                      = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
-      w->y                      = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
-      w->width                  = private_data->char_length * private_data->length;
-      w->height                 = private_data->char_height;
-      w->visible                = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
-      w->enable                 = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+      private_data->length        = xitk_skin_get_label_length(skonfig, private_data->skin_element_name);
+      private_data->animation     = xitk_skin_get_label_animation(skonfig, private_data->skin_element_name);
+      private_data->anim_step     = xitk_skin_get_label_animation_step(skonfig, private_data->skin_element_name);
+      private_data->anim_timer    = xitk_skin_get_label_animation_timer(skonfig, private_data->skin_element_name);
+      private_data->label_visible = xitk_skin_get_label_printable(skonfig, private_data->skin_element_name);
+      w->x                        = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
+      w->y                        = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
+      w->width                    = private_data->char_length * private_data->length;
+      w->height                   = private_data->char_height;
+      w->visible                  = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      w->enable                   = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
       
       {
 	char *label;
@@ -360,11 +384,13 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
 	label_setup_label(w, label);
       }
       
-      if(private_data->on_change == 0)
+      if(!pthread_mutex_trylock(&private_data->change_mutex)) {
 	paint_label(w);
-
+	pthread_mutex_unlock(&private_data->change_mutex);
+      }
+      
       xitk_skin_unlock(skonfig);
-
+      
       xitk_set_widget_pos(w, w->x, w->y);
     }
   }
@@ -381,11 +407,10 @@ int xitk_label_change_label(xitk_widget_t *w, char *newlabel) {
     if(!newlabel || !private_data->label ||
        (newlabel && (strcmp(private_data->label, newlabel)))) {
 
-      /* Don't change label if another thread is currently doing the same thing */
-      if(!private_data->on_change) {
-	label_setup_label(w, newlabel);
-	paint_label(w);
-      }
+      pthread_mutex_lock(&private_data->change_mutex);
+      pthread_mutex_unlock(&private_data->change_mutex);
+      label_setup_label(w, newlabel);
+      paint_label(w);
     }
     return 1;
   }
@@ -423,9 +448,11 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
       if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL))) {
 	label_private_data_t  *private_data = (label_private_data_t *) w->private_data;
 
-	if(private_data->on_change == 0)
+	if(!pthread_mutex_trylock(&private_data->change_mutex)) {
 	  paint_label(w);
-
+	  pthread_mutex_unlock(&private_data->change_mutex);
+	}
+	
       }
     }
     break;
@@ -484,8 +511,8 @@ static xitk_widget_t *_xitk_label_create(xitk_widget_list_t *wl,
     private_data->length       = width;
   }
   else {
-    private_data->font         = xitk_image_load_image(private_data->imlibdata, 
-							 xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
+    private_data->font         = xitk_skin_get_image(skonfig, 
+						     xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
   
     private_data->fontname     = NULL;
     private_data->char_length  = (private_data->font->width/32);
@@ -498,9 +525,12 @@ static xitk_widget_t *_xitk_label_create(xitk_widget_list_t *wl,
   private_data->anim_running   = 0;
   private_data->anim_step      = (skin_element_name == NULL) ? 1 : xitk_skin_get_label_animation_step(skonfig, private_data->skin_element_name);
   private_data->anim_timer     = (skin_element_name == NULL) ? 0 : xitk_skin_get_label_animation_timer(skonfig, private_data->skin_element_name);
+  private_data->label_visible  = (skin_element_name == NULL) ? 1 : xitk_skin_get_label_printable(skonfig, private_data->skin_element_name);
   private_data->labelpix       = NULL;
 
+
   pthread_mutex_init(&private_data->paint_mutex, NULL);
+  pthread_mutex_init(&private_data->change_mutex, NULL);
 
   mywidget->private_data       = private_data;
 
