@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.72 2003/11/10 21:04:42 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.83 2004/04/26 21:53:34 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -32,6 +32,10 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>                                                      
+
+#ifdef HAVE_ALLOCA_H   
+#include <alloca.h>
+#endif
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -92,10 +96,8 @@ static void _duplicate_mrl_filters(mrlbrowser_private_data_t *private_data,
     for(i = 0; 
 	mrl_filters[i] && (mrl_filters[i]->name && mrl_filters[i]->ending); i++) {
 
-      private_data->mrl_filters            = (xitk_mrlbrowser_filter_t **)
-	realloc(private_data->mrl_filters, sizeof(xitk_mrlbrowser_filter_t *) * (i + 2));
-      private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) 
-	xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
+      private_data->mrl_filters            = (xitk_mrlbrowser_filter_t **) realloc(private_data->mrl_filters, sizeof(xitk_mrlbrowser_filter_t *) * (i + 2));
+      private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
       private_data->mrl_filters[i]->name   = strdup(mrl_filters[i]->name);
       private_data->mrl_filters[i]->ending = strdup(mrl_filters[i]->ending);
     }
@@ -147,10 +149,9 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
 static void update_current_origin(mrlbrowser_private_data_t *private_data) {
 
   if(private_data->mc->mrls[0] && private_data->mc->mrls[0]->origin)
-    sprintf(private_data->current_origin, "%s", 
-	    private_data->mc->mrls[0]->origin);
+    snprintf(private_data->current_origin, sizeof(private_data->current_origin), "%s", private_data->mc->mrls[0]->origin);
   else
-    sprintf(private_data->current_origin, "%s", "");
+    snprintf(private_data->current_origin, sizeof(private_data->current_origin), "%s", "");
   
   xitk_label_change_label (private_data->widget_origin, 
 			   private_data->current_origin);
@@ -275,14 +276,8 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
     
     if(private_data->mc->filtered_mrls[i]->type & XINE_MRL_TYPE_file_symlink) {
 
-      if(private_data->mc->mrls_disp[i])
-	private_data->mc->mrls_disp[i] = (char *) 
-	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 4 + 
-		  strlen(private_data->mc->filtered_mrls[i]->link) + 2);
-      else
-	private_data->mc->mrls_disp[i] = (char *) 
-	  xitk_xmalloc(strlen(p) + 4 + 
-		      strlen(private_data->mc->filtered_mrls[i]->link) + 2);
+      private_data->mc->mrls_disp[i] = (char *) realloc(private_data->mc->mrls_disp[i], 
+							strlen(p) + 4 + strlen(private_data->mc->filtered_mrls[i]->link) + 2);
       
       sprintf(private_data->mc->mrls_disp[i], "%s%c -> %s", 
 	      p, get_mrl_marker(private_data->mc->filtered_mrls[i]), 
@@ -290,11 +285,7 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
 
     }
     else {
-      if(private_data->mc->mrls_disp[i])
-	private_data->mc->mrls_disp[i] = (char *) 
-	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 2);
-      else
-	private_data->mc->mrls_disp[i] = (char *) xitk_xmalloc(strlen(p) + 2);
+      private_data->mc->mrls_disp[i] = (char *) realloc(private_data->mc->mrls_disp[i], strlen(p) + 2);
       
       sprintf(private_data->mc->mrls_disp[i], "%s%c", 
 	      p, get_mrl_marker(private_data->mc->filtered_mrls[i]));
@@ -343,11 +334,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
 
   if(lbl) {
     
-    if(private_data->last_mrl_source)
-      private_data->last_mrl_source = (char *)
-	realloc(private_data->last_mrl_source, strlen(lbl) + 1);
-    else
-      private_data->last_mrl_source = (char *) xitk_xmalloc(strlen(lbl) + 1);
+    private_data->last_mrl_source = (char *) realloc(private_data->last_mrl_source, strlen(lbl) + 1);
     
     old_old_src = strdup(private_data->last_mrl_source);
     sprintf(private_data->last_mrl_source, "%s", lbl);
@@ -358,8 +345,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
 					       private_data->last_mrl_source, 
 					       NULL, &num_mrls);
       if(!mtmp) {
-	private_data->last_mrl_source = (char *)
-	  realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
+	private_data->last_mrl_source = (char *) realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
 	sprintf(private_data->last_mrl_source, "%s", old_old_src);
 	return;
       }
@@ -370,7 +356,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
     update_current_origin(private_data);
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
-			     (const char* const*)private_data->mc->mrls_disp, 
+			     (const char* const*)private_data->mc->mrls_disp, NULL,
 			     private_data->mc->mrls_to_disp, 0);
   }
 }
@@ -660,7 +646,7 @@ void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
 /*
  * Leaving mrlbrowser.
  */
-void xitk_mrlbrowser_exit(xitk_widget_t *w, void *data) {
+static void xitk_mrlbrowser_exit(xitk_widget_t *w, void *data) {
   mrlbrowser_private_data_t *private_data = ((xitk_widget_t *)data)->private_data;
   
   if(private_data->kill_callback)
@@ -727,17 +713,19 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
     {
       int x, y;
       int i = 0;
-      int dir;
+      int dir, max;
       
-      x = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name_ip);
-      y = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name_ip);
+      x   = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name_ip);
+      y   = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name_ip);
       dir = xitk_skin_get_direction(skonfig, private_data->skin_element_name_ip);
-    
+      max = xitk_skin_get_max_buttons(skonfig, private_data->skin_element_name_ip);
+
       switch(dir) {
       case DIRECTION_UP:
       case DIRECTION_DOWN:
-	while(private_data->autodir_plugins[i] != NULL) {
-	  
+	while((max && ((i < max) && (private_data->autodir_plugins[i] != NULL))) || 
+	      (!max && private_data->autodir_plugins[i] != NULL)) {
+
 	  (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 	  
 	  if(dir == DIRECTION_DOWN)
@@ -750,7 +738,8 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
 	break;
       case DIRECTION_LEFT:
       case DIRECTION_RIGHT:
-	while(private_data->autodir_plugins[i] != NULL) {
+	while((max && ((i < max) && (private_data->autodir_plugins[i] != NULL))) || 
+	      (!max && private_data->autodir_plugins[i] != NULL)) {
 	  
 	  (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 	  
@@ -764,6 +753,12 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
 	break;
       }
 
+      if(max) {
+	while(private_data->autodir_plugins[i] != NULL) {
+	  xitk_disable_and_hide_widget(private_data->autodir_plugins[i]);
+	  i++;
+	}
+      }
     }
     
     xitk_paint_widget_list(private_data->widget_list);
@@ -780,7 +775,7 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
   char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
   memset(&buf, '\0', sizeof(buf));
-  sprintf(buf, "%s", ms->mrl);
+  snprintf(buf, sizeof(buf), "%s", ms->mrl);
   
   if((ms->type & XINE_MRL_TYPE_file) && (ms->type & XINE_MRL_TYPE_file_directory)) {
     char *filename = ms->mrl;
@@ -797,14 +792,14 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
        (filename[strlen(filename) - 2] != '.')) {
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", ms->origin);
+      snprintf(buf, sizeof(buf), "%s", ms->origin);
     }
     else if((filename[strlen(filename) - 1] == '.') &&
 	    (filename[strlen(filename) - 2] == '.')) {
       char *p;
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", ms->origin);
+      snprintf(buf, sizeof(buf), "%s", ms->origin);
       if(strlen(buf) > 1) { /* not '/' directory */
 	  
 	p = &buf[strlen(buf)-1];
@@ -833,7 +828,7 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
     update_current_origin(private_data);
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
-			     (const char* const*)private_data->mc->mrls_disp, 
+			     (const char* const*)private_data->mc->mrls_disp, NULL,
 			     private_data->mc->mrls_to_disp, 0);
     
   }
@@ -907,7 +902,7 @@ static void combo_filter_select(xitk_widget_t *w, void *data, int select) {
   private_data->filter_selected = select;
   mrlbrowser_create_enlighted_entries(private_data);
   xitk_browser_update_list(private_data->mrlb_list, 
-  			  (const char* const*) private_data->mc->mrls_disp, 
+			   (const char* const*) private_data->mc->mrls_disp, NULL,
   			   private_data->mc->mrls_to_disp, 0);
 }
 
@@ -926,21 +921,13 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
     break;
     
   case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
+    KeySym         mkey;
     int            modifier;
 
     xitk_get_key_modifier(event, &modifier);
+    mkey = xitk_get_key_pressed(event);
     
-    mykeyevent = event->xkey;
-    
-    XLOCK (private_data->imlibdata->x.disp);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUNLOCK (private_data->imlibdata->x.disp);
-    
-    switch (mykey) {
+    switch (mkey) {
 
     case XK_d: 
     case XK_D:
@@ -965,6 +952,10 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
     }
     break;
 
+    case XK_Escape:
+      xitk_mrlbrowser_exit(NULL, (void *)private_data->fbWidget);
+      break;
+      
     }
   }
   break;
@@ -1077,13 +1068,18 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 					  mb->imlibdata->x.visual,
 					  CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 					  &attr);
-  
-  XSetStandardProperties(mb->imlibdata->x.disp, private_data->window, title, title,
-			 None, NULL, 0, &hint);
+
+  XmbSetWMProperties(mb->imlibdata->x.disp, private_data->window, title, title,
+                     NULL, 0, &hint, NULL, NULL);
 
   XSelectInput(mb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
   XUNLOCK(mb->imlibdata->x.disp);
-  
+
+  if(mb->set_wm_window_normal)
+    xitk_set_wm_window_type(private_data->window, WINDOW_TYPE_NORMAL);
+  else
+    xitk_unset_wm_window_type(private_data->window, WINDOW_TYPE_NORMAL);
+
   /*
    * layer above most other things, like gnome panel
    * WIN_LAYER_ABOVE_DOCK  = 10
@@ -1122,6 +1118,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
     xclasshint->res_name  = mb->resource_name ? mb->resource_name : "xitk mrl browser";
     xclasshint->res_class = mb->resource_class ? mb->resource_class : "xitk";
     XSetClassHint(mb->imlibdata->x.disp, private_data->window, xclasshint);
+    XFree(xclasshint);
   }
   XUNLOCK(mb->imlibdata->x.disp);
   
@@ -1130,7 +1127,11 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   if (wm_hint != NULL) {
     wm_hint->input         = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags         = InputHint | StateHint;
+    if(mb->icon) {
+      wm_hint->icon_pixmap = *mb->icon;
+      wm_hint->flags       = IconPixmapHint;
+    }
+    wm_hint->flags         |= InputHint | StateHint;
     XSetWMHints(mb->imlibdata->x.disp, private_data->window, wm_hint);
     XFree(wm_hint);
   }
@@ -1202,7 +1203,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   
   memset(&private_data->current_origin, 0, strlen(private_data->current_origin));
   if(mb->origin.cur_origin)
-    sprintf(private_data->current_origin, "%s", mb->origin.cur_origin);
+    snprintf(private_data->current_origin, sizeof(private_data->current_origin), "%s", mb->origin.cur_origin);
 
   if(mb->ip_name.label.label_str) {
 
@@ -1223,13 +1224,14 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   {
     int x, y;
     int i = 0;
-    int dir;
+    int dir, max;
     
     private_data->skin_element_name_ip = strdup(mb->ip_name.button.skin_element_name);
-    x = xitk_skin_get_coord_x(skonfig, mb->ip_name.button.skin_element_name);
-    y = xitk_skin_get_coord_y(skonfig, mb->ip_name.button.skin_element_name);
+    x   = xitk_skin_get_coord_x(skonfig, mb->ip_name.button.skin_element_name);
+    y   = xitk_skin_get_coord_y(skonfig, mb->ip_name.button.skin_element_name);
     dir = xitk_skin_get_direction(skonfig, mb->ip_name.button.skin_element_name);
-    
+    max = xitk_skin_get_max_buttons(skonfig, mb->ip_name.button.skin_element_name);
+
     while(mb->ip_availables[i] != NULL) {
 
       lb.button_type    = CLICK_BUTTON;
@@ -1268,7 +1270,10 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 	  x -= (xitk_get_widget_width(private_data->autodir_plugins[i]) + 1);
 	break;
       }
-
+      
+      if(max && (i >= max))
+	xitk_disable_and_hide_widget(private_data->autodir_plugins[i]);
+      
       i++;
     }
     if(i)
@@ -1307,7 +1312,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   mywidget->tips_string        = NULL;
 
   xitk_browser_update_list(private_data->mrlb_list, 
-			   (const char* const*)private_data->mc->mrls_disp, 
+			   (const char* const*)private_data->mc->mrls_disp, NULL,
 			   private_data->mrls_num, 0);
 
   XLOCK (mb->imlibdata->x.disp);
