@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.c,v 1.25 2003/11/10 21:04:42 f1rmb Exp $
+ * $Id: skin.c,v 1.37 2004/07/02 13:23:31 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -39,9 +39,75 @@
  *                                     PRIVATES
  ************************************************************************************/
 
+static pthread_mutex_t  skin_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 /*
  *
  */
+static xitk_image_t *skin_load_to_cache(xitk_skin_config_t *skonfig, char *pixmap) {
+
+  if(skonfig && pixmap) {
+    xitk_image_t  *image  = xitk_image_load_image(skonfig->im, pixmap);
+    cache_entry_t *cache  = skonfig->cache;
+    cache_entry_t *ncache = NULL;
+
+    if(image) {
+
+      if(cache) {
+	while(cache) {
+	  if(!strcmp(cache->filename, pixmap)) {
+	    xitk_image_free_image(skonfig->im, &image);
+	    return cache->image;
+	  }
+	  cache = cache->next;
+	}
+	cache = skonfig->cache;
+      }
+
+      ncache           = (cache_entry_t *) xitk_xmalloc(sizeof(cache_entry_t));
+      ncache->image    = image;
+      ncache->filename = strdup(pixmap);
+      ncache->next     = NULL;
+    }
+    else
+      return image;
+    
+    if(!cache) {
+      skonfig->cache = ncache;
+    }
+    else {
+      while(cache->next)
+	cache = cache->next;
+      
+      cache->next  = ncache;
+    }
+
+    return image;
+  }
+
+  return NULL;
+}
+
+static void skin_free_cache(xitk_skin_config_t *skonfig) {
+  cache_entry_t *pcache, *cache  = skonfig->cache;
+  
+  while(cache) {
+    pcache = cache;
+    
+    xitk_image_free_image(skonfig->im, &(cache->image));
+    free(cache->filename);
+    
+    cache  = cache->next;
+    free(pcache);
+  }
+
+  skonfig->cache = NULL;
+}
+
+/*
+ *
+ */
+#warning FIXME
 static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
   char *p;
   char *ret = NULL;
@@ -54,7 +120,7 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
     if(strchr(cmd, '$')) {
 
       memset(&buf, 0, sizeof(buf));
-      snprintf(buf, (BUFSIZ - 1), "%s", cmd);
+      snprintf(buf, sizeof(buf), "%s", cmd);
 
       memset(&buf2, 0, sizeof(buf2));
       
@@ -80,7 +146,7 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
 		char ppath[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 		char *z;
 		
-		sprintf(ppath, "%s", skonfig->path);
+		snprintf(ppath, sizeof(ppath), "%s", skonfig->path);
 		if((z = strrchr(ppath, '/')) != NULL) {
 		  *z = '\0';
 		  sprintf(buf2, "%s%s", buf2, ppath);
@@ -149,6 +215,7 @@ static void _nullify_me(xitk_skin_element_t *s) {
   s->font            = NULL;
   s->browser_entries = -2;
   s->direction       = DIRECTION_LEFT; /* Compatibility */
+  s->max_buttons     = 0;
 }
 
 /*
@@ -241,49 +308,20 @@ static void skin_get_next_line(xitk_skin_config_t *skonfig) {
   ABORT_IF_NULL(skonfig);
   ABORT_IF_NULL(skonfig->fd);
   
- __get_next_line:
+  do {
+    skonfig->ln = fgets(skonfig->buf, 255, skonfig->fd);
+    
+    while(skonfig->ln && (*skonfig->ln == ' ' || *skonfig->ln == '\t')) ++skonfig->ln;
 
-  skonfig->ln = fgets(skonfig->buf, 255, skonfig->fd);
-
-  while(skonfig->ln && (*skonfig->ln == ' ' || *skonfig->ln == '\t')) ++skonfig->ln;
-
-  if(skonfig->ln) {
-    if((strncmp(skonfig->ln, "//", 2) == 0) ||
-       (strncmp(skonfig->ln, "/*", 2) == 0) || /**/
-       (strncmp(skonfig->ln, ";", 1) == 0) ||
-       (strncmp(skonfig->ln, "#", 1) == 0)) {
-      goto __get_next_line;
-    }
-  }
+  } while(skonfig->ln && 
+	  (!strncmp(skonfig->ln, "//", 2) ||
+	   !strncmp(skonfig->ln, "/*", 2) || /* */
+	   !strncmp(skonfig->ln, ";", 1) ||
+	   !strncmp(skonfig->ln, "#", 1)));
 
   skin_clean_eol(skonfig);
 }
 
-/*
- * Return 0/1 from char value (valids are 1/0, true/false, 
- * yes/no, on/off. Case isn't checked.
- */
-static int skin_get_bool_value(const char *val) {
-  static struct {
-    const char *str;
-    int value;
-  } bools[] = {
-    { "1",     1 }, { "true",  1 }, { "yes",   1 }, { "on",    1 },
-    { "0",     0 }, { "false", 0 }, { "no",    0 }, { "off",   0 },
-    { NULL,    0 }
-  };
-  int i;
-  
-  ABORT_IF_NULL(val);
-
-  for(i = 0; bools[i].str != NULL; i++) {
-    if(!(strcasecmp(bools[i].str, val)))
-      return bools[i].value;
-  }
-
-  return 0;
-}
-
 /*
  * Return alignement value.
  */
@@ -400,6 +438,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->pixmap_pad = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	  sprintf(skonfig->celement->pixmap_pad, "%s/%s", skonfig->path, p);
+	  skin_load_to_cache(skonfig, skonfig->celement->pixmap_pad);
 	}
 	else if(!strncasecmp(skonfig->ln, "radius", 6)) {
 	  skin_set_pos_to_value(&p);
@@ -443,7 +482,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	}
 	else if(!strncasecmp(skonfig->ln, "animation", 9)) {
 	  skin_set_pos_to_value(&p);
-	  skonfig->celement->animation = skin_get_bool_value(p);
+	  skonfig->celement->animation = xitk_get_bool_value(p);
 	}
 	else if(!strncasecmp(skonfig->ln, "length", 6)) {
 	  skin_set_pos_to_value(&p);
@@ -453,10 +492,11 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->pixmap_font = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	  sprintf(skonfig->celement->pixmap_font, "%s/%s", skonfig->path, p);
+	  skin_load_to_cache(skonfig, skonfig->celement->pixmap_font);
 	}
 	else if(!strncasecmp(skonfig->ln, "static", 6)) {
 	  skin_set_pos_to_value(&p);
-	  skonfig->celement->staticity = skin_get_bool_value(p);
+	  skonfig->celement->staticity = xitk_get_bool_value(p);
 	}
 	else if(!strncasecmp(skonfig->ln, "align", 5)) {
 	  skin_set_pos_to_value(&p);
@@ -468,7 +508,7 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	}
 	else if(!strncasecmp(skonfig->ln, "print", 5)) {
 	  skin_set_pos_to_value(&p);
-	  skonfig->celement->print = skin_get_bool_value(p);
+	  skonfig->celement->print = xitk_get_bool_value(p);
 	}
 	else if(!strncasecmp(skonfig->ln, "timer", 5)) {
 	  skin_set_pos_to_value(&p);
@@ -541,23 +581,27 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	     goto  __next_subsection;
 	  }
 	  else {
-
-	    if(!strncasecmp(skonfig->ln, "direction", 9)) {
+	    if(!strncasecmp(skonfig->ln, "max_buttons", 11)) {
+	      skin_set_pos_to_value(&p);
+	      s->max_buttons = strtol(p, &p, 10);
+	    }
+	    else if(!strncasecmp(skonfig->ln, "direction", 9)) {
 	      skin_set_pos_to_value(&p);
 	      s->direction = skin_get_direction(p);
 	    }
 	    else if(!strncasecmp(skonfig->ln, "visible", 7)) {
 	      skin_set_pos_to_value(&p);
-	      s->visible = skin_get_bool_value(p);
+	      s->visible = xitk_get_bool_value(p);
 	    }
 	    else if(!strncasecmp(skonfig->ln, "pixmap", 6)) {
 	      skin_set_pos_to_value(&p);
 	      s->pixmap = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	      sprintf(s->pixmap, "%s/%s", skonfig->path, p);
+	      skin_load_to_cache(skonfig, s->pixmap);
 	    }
 	    else if(!strncasecmp(skonfig->ln, "enable", 6)) {
 	      skin_set_pos_to_value(&p);
-	      s->enable = skin_get_bool_value(p);
+	      s->enable = xitk_get_bool_value(p);
 	    }
 	    
 	  }
@@ -660,8 +704,8 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       }
 
       if(s->browser_entries > -1)
-	printf("  browser entries = %d\n", s->browset_entries);
-
+	printf("  browser entries = %d\n", s->browser_entries);
+      
       printf("  animation   = %d\n", s->animation);
       printf("  step        = %d\n", s->animation_step);
       printf("  print       = %d\n", s->print);
@@ -672,6 +716,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       printf("  color click = '%s'\n", s->color_click);
       printf("  pixmap font = '%s'\n", s->pixmap_font);
       printf("  font        = '%s'\n", s->font);
+      printf("  max_buttons = %d\n", s->max_buttons);
       s = s->next;
     }
 
@@ -695,6 +740,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       printf("  color click = '%s'\n", s->color_click);
       printf("  pixmap font = '%s'\n", s->pixmap_font);
       printf("  font        = '%s'\n", s->font);
+      printf("  max_buttons = %d\n", s->max_buttons);
       s = s->prev;
     }
   }
@@ -726,23 +772,30 @@ static xitk_skin_element_t *skin_lookup_section(xitk_skin_config_t *skonfig, con
 /*
  * Alloc a xitk_skin_config_t* memory area, nullify pointers.
  */
-xitk_skin_config_t *xitk_skin_init_config(void) {
+xitk_skin_config_t *xitk_skin_init_config(ImlibData *im) {
   xitk_skin_config_t *skonfig;
   
   if((skonfig = (xitk_skin_config_t *) xitk_xmalloc(sizeof(xitk_skin_config_t))) == NULL) {
     XITK_DIE("xitk_xmalloc() failed: %s\n", strerror(errno));
   }
-  skonfig->version = -1;
-  skonfig->first = skonfig->last = skonfig->celement = NULL;
-  skonfig->name = skonfig->author = skonfig->date = skonfig->url = 
-    skonfig->load_command = skonfig->unload_command = skonfig->logo = 
-    skonfig->animation = NULL;
+  skonfig->im       = im;
+  skonfig->cache    = NULL;
+  skonfig->version  = -1;
+  skonfig->first    = skonfig->last 
+                    = skonfig->celement 
+                    = NULL;
+  skonfig->name     = skonfig->author 
+                    = skonfig->date 
+                    = skonfig->url
+                    = skonfig->load_command 
+                    = skonfig->unload_command 
+                    = skonfig->logo 
+                    = skonfig->animation 
+                    = NULL;
   skonfig->skinfile = skonfig->path = NULL;
 
   skonfig->ln = skonfig->buf;
   
-  pthread_mutex_init(&skonfig->mutex, NULL);
-
   return skonfig;
 }
 
@@ -754,7 +807,9 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
 
   ABORT_IF_NULL(skonfig);
 
+  pthread_mutex_lock(&skin_mutex);
   if(skonfig->celement) {
+    
     s = skonfig->last;
     
     while(s) {
@@ -785,9 +840,10 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   XITK_FREE(skonfig->unload_command);
   XITK_FREE(skonfig->skinfile);
 
-  pthread_mutex_destroy(&skonfig->mutex);
+  skin_free_cache(skonfig);
 
   XITK_FREE(skonfig);
+  pthread_mutex_unlock(&skin_mutex);
 }
 
 /*
@@ -800,10 +856,12 @@ int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filenam
   ABORT_IF_NULL(path);
   ABORT_IF_NULL(filename);
 
+  pthread_mutex_lock(&skin_mutex);
+
   skonfig->path     = strdup(path);
   skonfig->skinfile = strdup(filename);
 
-  snprintf(buf, 2048, "%s/%s", skonfig->path, skonfig->skinfile);
+  snprintf(buf, sizeof(buf), "%s/%s", skonfig->path, skonfig->skinfile);
 
   if((skonfig->fd = fopen(buf, "r")) != NULL) {
     
@@ -822,12 +880,14 @@ int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filenam
   else {
     XITK_WARNING("%s(): Unable to open '%s' file.\n", __FUNCTION__, skonfig->skinfile);
     XITK_FREE(skonfig->skinfile);
+    pthread_mutex_unlock(&skin_mutex);
     return 0;
   }
 
   if(!skonfig->first) {
     XITK_WARNING("%s(): no valid skin element found in '%s/%s'.\n",
 		 __FUNCTION__, skonfig->path, skonfig->skinfile);
+    pthread_mutex_unlock(&skin_mutex);
     return 0;
   }
 
@@ -841,6 +901,8 @@ int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filenam
   if(skonfig->load_command)
     xitk_system(0, skonfig->load_command);
 
+  pthread_mutex_unlock(&skin_mutex);
+
   return 1;
 }
 
@@ -908,6 +970,21 @@ int xitk_skin_get_visibility(xitk_skin_config_t *skonfig, const char *str) {
   return 1;
 }
 
+/*
+ *
+ */
+int xitk_skin_get_printability(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  ABORT_IF_NULL(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->print;
+
+  return 1;
+}
+
+
 /*
  *
  */
@@ -1200,17 +1277,48 @@ int xitk_skin_get_browser_entries(xitk_skin_config_t *skonfig, const char *str)
   
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->browser_entries;
-
+  
   return -1;
 }
 
+/*
+ *
+ */
+xitk_image_t *xitk_skin_get_image(xitk_skin_config_t *skonfig, const char *str) {
+  ABORT_IF_NULL(skonfig);
+
+  if(skonfig->cache) {
+    cache_entry_t *cache = skonfig->cache;
+    
+    while(cache) {
+      if(!strcmp(str, cache->filename))
+	return cache->image;
+      
+      cache = cache->next;
+    }
+  }
+
+  return (skin_load_to_cache(skonfig, (char *) str));
+  //  return (xitk_image_load_image(skonfig->im, (char *) str));
+}
+
+int xitk_skin_get_max_buttons(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  ABORT_IF_NULL(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->max_buttons;
+  
+  return 0;
+}
+
 /*
  *
  */
 void xitk_skin_lock(xitk_skin_config_t *skonfig) {
   ABORT_IF_NULL(skonfig);
-
-  pthread_mutex_lock(&skonfig->mutex);
+  pthread_mutex_lock(&skin_mutex);
 }
 
 /*
@@ -1218,6 +1326,5 @@ void xitk_skin_lock(xitk_skin_config_t *skonfig) {
  */
 void xitk_skin_unlock(xitk_skin_config_t *skonfig) {
   ABORT_IF_NULL(skonfig);
-  
-  pthread_mutex_unlock(&skonfig->mutex);
+  pthread_mutex_unlock(&skin_mutex);
 }
