@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.c,v 1.110 2003/12/13 01:09:40 f1rmb Exp $
+ * $Id: xitk.c,v 1.128 2004/07/02 13:23:31 f1rmb Exp $
  *
  */
 #define _XITK_C_
@@ -42,9 +42,14 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/time.h>
 
 #include <locale.h>
 
+#ifdef __linux__
+#include <execinfo.h>
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
@@ -111,6 +116,8 @@ typedef struct {
 
 typedef struct {
   Display                    *display;
+  int                         display_width;
+  int                         display_height;
   int                         verbosity;
   xitk_list_t                *list;
   xitk_list_t                *gfx;
@@ -129,6 +136,10 @@ typedef struct {
 
   Window                      modalw;
   xitk_widget_t              *menu;
+  
+  struct timeval              keypress;
+
+  pthread_t                  *tips_thread;
 } __xitk_t;
 
 static __xitk_t    *gXitk;
@@ -137,6 +148,15 @@ static Atom XA_WIN_LAYER = None, XA_STAYS_ON_TOP = None;
 static Atom XA_NET_WM_STATE = None, XA_NET_WM_STATE_ABOVE = None;
 static Atom XA_NET_WM_STATE_FULLSCREEN = None;
 
+static Atom XA_WM_WINDOW_TYPE = None;
+static Atom XA_WM_WINDOW_TYPE_DESKTOP = None;
+static Atom XA_WM_WINDOW_TYPE_DOCK = None;
+static Atom XA_WM_WINDOW_TYPE_TOOLBAR = None;
+static Atom XA_WM_WINDOW_TYPE_MENU = None;
+static Atom XA_WM_WINDOW_TYPE_UTILITY = None;
+static Atom XA_WM_WINDOW_TYPE_SPLASH = None;
+static Atom XA_WM_WINDOW_TYPE_DIALOG = None;
+static Atom XA_WM_WINDOW_TYPE_NORMAL = None;
 
 void widget_stop(void);
 
@@ -209,7 +229,7 @@ void xitk_usec_sleep(unsigned long usec) {
 #endif
 }
 
-int _x_error_handler(Display *display, XErrorEvent *xevent) {
+static int _x_error_handler(Display *display, XErrorEvent *xevent) {
   char buffer[2048];
   
   XGetErrorText(display, xevent->error_code, &buffer[0], 1023);
@@ -280,12 +300,34 @@ static void xitk_signal_handler(int sig) {
     }
     break;
 
-#ifndef DEBUG
   case SIGSEGV:
-    fprintf(stderr, "xiTK received SIGSEGV signal, RIP.\n");
-    abort();
-    break;
+    {
+      fprintf(stderr, "xiTK received SIGSEGV signal, RIP.\n");
+#ifndef DEBUG
+      abort();
+#else
+#ifdef __linux__
+      void    *backtrace_array[255];
+      char   **backtrace_strings;
+      int      entries, i;
+      
+      if((entries = backtrace(backtrace_array, 255))) {
+	if((backtrace_strings = backtrace_symbols(backtrace_array, entries))) {
+	  printf("Backtrace:\n");
+	  
+	  for(i = 0; i < entries; i++) {
+	    printf("  [%d] %s\n", i, backtrace_strings[i]);
+	  }
+	  
+	  free(backtrace_strings);
+	  printf("--\n");
+	}
+      }
 #endif
+#endif
+    }
+    break;
+
 
   }
 }
@@ -519,11 +561,21 @@ static uint32_t xitk_check_wm(Display *display) {
   }
 
   if(type & WM_TYPE_EWMH_COMP) {
-    XA_WIN_LAYER          = XInternAtom(display, "_NET_WM_STATE", False);
-    XA_STAYS_ON_TOP       = XInternAtom(display, "_NET_WM_STATE_STAYS_ON_TOP", False);
-    XA_NET_WM_STATE       = XInternAtom(display, "_NET_WM_STATE", False);
-    XA_NET_WM_STATE_ABOVE = XInternAtom(display, "_NET_WM_STATE_ABOVE", False);
+    XA_WIN_LAYER               = XInternAtom(display, "_NET_WM_STATE", False);
+    XA_STAYS_ON_TOP            = XInternAtom(display, "_NET_WM_STATE_STAYS_ON_TOP", False);
+    XA_NET_WM_STATE            = XInternAtom(display, "_NET_WM_STATE", False);
+    XA_NET_WM_STATE_ABOVE      = XInternAtom(display, "_NET_WM_STATE_ABOVE", False);
     XA_NET_WM_STATE_FULLSCREEN = XInternAtom(display, "_NET_WM_STATE_FULLSCREEN", False);
+
+    XA_WM_WINDOW_TYPE          = XInternAtom(display, "_NET_WM_WINDOW_TYPE", False);
+    XA_WM_WINDOW_TYPE_DESKTOP  = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DESKTOP", False);
+    XA_WM_WINDOW_TYPE_DOCK     = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DOCK", False);
+    XA_WM_WINDOW_TYPE_TOOLBAR  = XInternAtom(display, "_NET_WM_WINDOW_TYPE_TOOLBAR", False);
+    XA_WM_WINDOW_TYPE_MENU     = XInternAtom(display, "_NET_WM_WINDOW_TYPE_MENU", False);
+    XA_WM_WINDOW_TYPE_UTILITY  = XInternAtom(display, "_NET_WM_WINDOW_TYPE_UTILITY", False);
+    XA_WM_WINDOW_TYPE_SPLASH   = XInternAtom(display, "_NET_WM_WINDOW_TYPE_SPLASH", False);
+    XA_WM_WINDOW_TYPE_DIALOG   = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DIALOG", False);
+    XA_WM_WINDOW_TYPE_NORMAL   = XInternAtom(display, "_NET_WM_WINDOW_TYPE_NORMAL", False);
   }
   
   switch(type & WM_TYPE_COMP_MASK) {
@@ -670,18 +722,18 @@ void xitk_set_layer_above(Window window) {
       XEvent xev;
       
       XLockDisplay(gXitk->display);
-      
-      xev.type                 = ClientMessage;
+      xev.xclient.type         = ClientMessage;
+      xev.xclient.display      = gXitk->display;
       xev.xclient.window       = window;
       xev.xclient.message_type = XA_NET_WM_STATE;
       xev.xclient.format       = 32;
       xev.xclient.data.l[0]    = 1;
       xev.xclient.data.l[1]    = XA_STAYS_ON_TOP;
-      xev.xclient.data.l[2]    = None;
+      xev.xclient.data.l[2]    = 0l;
+      xev.xclient.data.l[3]    = 0l;
+      xev.xclient.data.l[4]    = 0l;
       
-      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), 
-		 False, SubstructureRedirectMask | SubstructureNotifyMask, (XEvent*) &xev);
-
+      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), True, SubstructureRedirectMask, &xev);
       XUnlockDisplay(gXitk->display);
     }
     else {
@@ -770,29 +822,106 @@ void xitk_set_window_layer(Window window, int layer) {
   XUnlockDisplay(gXitk->display);
 }
 
-void xitk_set_ewmh_fullscreen(Window window) {
+static void _set_ewmh_state(Window window, Atom atom, int enable) {
   XEvent xev;
   
+  if((window == None) || (atom == None))
+    return;
+
+  xev.xclient.type         = ClientMessage;
+  xev.xclient.message_type = XA_NET_WM_STATE;
+  xev.xclient.display      = gXitk->display;
+  xev.xclient.window       = window;
+  xev.xclient.format       = 32;
+  xev.xclient.data.l[0]    = (enable == 1) ? 1 : 0;
+  xev.xclient.data.l[1]    = atom;
+  xev.xclient.data.l[2]    = 0l;
+  xev.xclient.data.l[3]    = 0l;
+  xev.xclient.data.l[4]    = 0l;
+
+  XLockDisplay(gXitk->display);
+  XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), True, SubstructureRedirectMask, &xev);
+  XUnlockDisplay(gXitk->display);
+}
+
+void xitk_set_ewmh_fullscreen(Window window) {
+  
   if(!(gXitk->wm_type & WM_TYPE_EWMH_COMP) || (window == None))
     return;
   
-  XLockDisplay(gXitk->display);
-  
-  xev.xclient.type         = ClientMessage;
-  xev.xclient.send_event   = True;
-  xev.xclient.display      = gXitk->display;
-  xev.xclient.window       = window;
-  xev.xclient.message_type = XA_NET_WM_STATE;
-  xev.xclient.format       = 32;
-  xev.xclient.data.l[0]    = (long) 1;
-  xev.xclient.data.l[1]    = (long) XA_NET_WM_STATE_FULLSCREEN;
-  xev.xclient.data.l[2]    = (long) None;
-  
-  XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), 
-	     False, SubstructureRedirectMask | SubstructureNotifyMask, (XEvent*) &xev);
-  
-  XUnlockDisplay(gXitk->display);
+  _set_ewmh_state(window, XA_NET_WM_STATE_FULLSCREEN, 1);
+  _set_ewmh_state(window, XA_STAYS_ON_TOP, 1);
+}
 
+void xitk_unset_ewmh_fullscreen(Window window) {
+  
+  if(!(gXitk->wm_type & WM_TYPE_EWMH_COMP) || (window == None))
+    return;
+  
+  _set_ewmh_state(window, XA_NET_WM_STATE_FULLSCREEN, 0);
+  _set_ewmh_state(window, XA_STAYS_ON_TOP, 0);
+}
+
+static void _set_wm_window_type(Window window, xitk_wm_window_type_t type, int value) {
+  if(window && (gXitk->wm_type & WM_TYPE_EWMH_COMP)) {
+    Atom *atom = NULL;
+    
+    switch(type) {
+    case WINDOW_TYPE_DESKTOP:
+      atom = &XA_WM_WINDOW_TYPE_DESKTOP;
+      break;
+    case WINDOW_TYPE_DOCK:
+      atom = &XA_WM_WINDOW_TYPE_DOCK;
+      break;
+    case WINDOW_TYPE_TOOLBAR:
+      atom = &XA_WM_WINDOW_TYPE_TOOLBAR;
+      break;
+    case WINDOW_TYPE_MENU:
+      atom = &XA_WM_WINDOW_TYPE_MENU;
+      break;
+    case WINDOW_TYPE_UTILITY:
+      atom = &XA_WM_WINDOW_TYPE_UTILITY;
+      break;
+    case WINDOW_TYPE_SPLASH:
+      atom = &XA_WM_WINDOW_TYPE_SPLASH;
+      break;
+    case WINDOW_TYPE_DIALOG:
+      atom = &XA_WM_WINDOW_TYPE_DIALOG;
+      break;
+    case WINDOW_TYPE_NORMAL:
+      atom = &XA_WM_WINDOW_TYPE_NORMAL;
+      break;
+    }
+    
+    if(atom) {
+      XEvent xev;
+
+      xev.xclient.type         = ClientMessage;
+      xev.xclient.display      = gXitk->display;
+      xev.xclient.window       = window;
+      xev.xclient.message_type = XA_WIN_LAYER;
+      xev.xclient.format       = 32;
+      xev.xclient.data.l[0]    = *atom;
+      xev.xclient.data.l[1]    = 0l;
+      xev.xclient.data.l[2]    = 0l;
+      xev.xclient.data.l[3]    = 0l;
+      xev.xclient.data.l[4]    = 0l;
+  
+      XLOCK(gXitk->display);
+      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), False, SubstructureNotifyMask, &xev);
+
+      XRaiseWindow(gXitk->display, window);
+      XUNLOCK(gXitk->display);
+    }
+  }
+}
+
+void xitk_unset_wm_window_type(Window window, xitk_wm_window_type_t type) {
+  _set_wm_window_type(window, type, 0);
+}
+
+void xitk_set_wm_window_type(Window window, xitk_wm_window_type_t type) {
+  _set_wm_window_type(window, type, 1);
 }
 
 /*
@@ -972,7 +1101,6 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
   
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
-  
   while(fx) {
     
     if(fx->key == *key) {
@@ -983,10 +1111,15 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
 	free(fx->xdnd);
       }
 
+      fx->xevent_callback = NULL;
+      fx->newpos_callback = NULL;
+      fx->user_data       = NULL;
+
       xitk_list_delete_current(gXitk->gfx); 
 
       free(fx->name);
       free(fx);
+      fx = NULL;
       
       MUTUNLOCK();
       return;
@@ -1009,7 +1142,7 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
     
   while(fx) {
-
+    
     if((fx->key == key) && (fx->window != None)) {
       Window c;
       
@@ -1061,6 +1194,9 @@ void xitk_xevent_notify(XEvent *event) {
   }
   
   while(fx) {
+
+    if(event->type == KeyRelease)
+      gettimeofday(&gXitk->keypress, 0);
     
     if(fx->window != None) {
       
@@ -1093,13 +1229,9 @@ void xitk_xevent_notify(XEvent *event) {
 	  len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
 	  XUNLOCK(gXitk->display);
 
-	  if(fx->widget_list && fx->widget_list->widget_under_mouse)
-	    xitk_tips_tips_kill(fx->widget_list->widget_under_mouse);
+	  if(fx->widget_list && (fx->widget_list->widget_under_mouse || fx->widget_list->widget_focused))
+	    xitk_tips_hide_tips();
 	  
-	  if(fx->widget_list && fx->widget_list->widget_focused)
-	    xitk_tips_tips_kill(fx->widget_list->widget_focused);
-
-
 	  if(fx->widget_list && fx->widget_list->widget_focused) {
 	    w = fx->widget_list->widget_focused;
 	  }
@@ -1512,6 +1644,8 @@ void xitk_init(Display *display, int verbosity) {
 
   gXitk = (__xitk_t *) xitk_xmalloc(sizeof(__xitk_t));
 
+  gXitk->display_width   = DisplayWidth(display, DefaultScreen(display));
+  gXitk->display_height  = DisplayHeight(display, DefaultScreen(display));
   gXitk->verbosity       = verbosity;
   gXitk->list            = xitk_list_new();
   gXitk->gfx             = xitk_list_new();
@@ -1525,10 +1659,11 @@ void xitk_init(Display *display, int verbosity) {
   gXitk->x_error_handler = NULL;
   gXitk->modalw          = None;
 
+  memset(&gXitk->keypress, 0, sizeof(gXitk->keypress));
+
   pthread_mutex_init (&gXitk->mutex, NULL);
   
-  sprintf(buffer, "-[ xiTK version %d.%d.%d ", 
-	  XITK_MAJOR_VERSION, XITK_MINOR_VERSION, XITK_SUB_VERSION);
+  snprintf(buffer, sizeof(buffer), "-[ xiTK version %d.%d.%d ", XITK_MAJOR_VERSION, XITK_MINOR_VERSION, XITK_SUB_VERSION);
   
   /* Check if SHM is working */
 #ifdef HAVE_SHM
@@ -1552,7 +1687,6 @@ void xitk_init(Display *display, int verbosity) {
 	shminfo.shmaddr = xim->data =  shmat(shminfo.shmid, 0, 0);
 	if(shminfo.shmaddr == (char *) -1) {
 	  XDestroyImage(xim);
-	  shmctl(shminfo.shmid, IPC_RMID, 0);
 	  gXitk->use_xshm = 0;
 	}
 	else {
@@ -1572,17 +1706,19 @@ void xitk_init(Display *display, int verbosity) {
 	  
 	  XDestroyImage(xim);
 	  shmdt(shminfo.shmaddr);
-	  shmctl(shminfo.shmid, IPC_RMID, 0);
 
 	  xitk_uninstall_x_error_handler();
 	  xitk_x_error = 0;
 	}
+	shmctl(shminfo.shmid, IPC_RMID, 0);
       }
     }
   }
 #endif
 
-#ifdef WITH_XMB
+#ifdef WITH_XFT
+  sprintf(buffer, "%s%s", buffer, "[XFT]");
+#elif defined(WITH_XMB)
   sprintf(buffer, "%s%s", buffer, "[XMB]");
 #endif
   
@@ -1595,6 +1731,8 @@ void xitk_init(Display *display, int verbosity) {
   
   /* init font caching */
   xitk_font_cache_init();
+
+  xitk_tips_init(display);
 }
 
 /* Return True is event type isn't completion */
@@ -1731,20 +1869,22 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
 
   }
 
+  /* destroy font caching */
+  xitk_font_cache_done();
+  
   xitk_list_free(gXitk->list);
   xitk_list_free(gXitk->gfx);
   xitk_config_deinit(gXitk->config);
   
   XITK_FREE(gXitk);
 
-  /* destroy font caching */
-  xitk_font_cache_done();
 }
 
 /*
  * Stop the wait xevent loop
  */
 void xitk_stop(void) {
+  xitk_tips_deinit();
   gXitk->running = 0;
 }
  
@@ -1754,6 +1894,12 @@ char *xitk_get_system_font(void) {
 char *xitk_get_default_font(void) {
   return xitk_config_get_default_font(gXitk->config);
 }
+int xitk_get_xmb_enability(void) {
+  return xitk_config_get_xmb_enability(gXitk->config);
+}
+void xitk_set_xmb_enability(int value) {
+  xitk_config_set_xmb_enability(gXitk->config, value);
+}
 int xitk_get_black_color(void) {
   return xitk_config_get_black_color(gXitk->config);
 }
@@ -1784,6 +1930,19 @@ long int xitk_get_timer_dbl_click(void) {
 int xitk_get_barstyle_feature(void) {
   return xitk_config_get_barstyle_feature(gXitk->config);
 }
+int xitk_get_checkstyle_feature(void) {
+  return xitk_config_get_checkstyle_feature(gXitk->config);
+}
+int xitk_get_menu_shortcuts_enability(void) {
+  return xitk_config_get_menu_shortcuts_enability(gXitk->config);
+}
+
+int xitk_get_display_width(void) {
+  return gXitk->display_width;
+}
+int xitk_get_display_height(void) {
+  return gXitk->display_height;
+}
 
 /*
  * copy src to dest and substitute special chars. dest should have 
@@ -1855,11 +2014,26 @@ char *xitk_set_locale(void) {
   return cur_locale;
 }
 
+
+/*
+ *
+ */
+long int xitk_get_last_keypressed_time(void) {
+  struct timeval tm, tm_diff;
+  
+  gettimeofday(&tm, NULL);
+  timersub(&tm, &gXitk->keypress, &tm_diff);
+  return tm_diff.tv_sec;
+}
+
 /*
  * Return home directory.
  */
 const char *xitk_get_homedir(void) {
-  struct passwd  pwd, *pw = NULL;
+#ifdef HAVE_GETPWUID_R
+  struct passwd  pwd;
+#endif
+  struct passwd *pw = NULL;
   static char    homedir[BUFSIZ] = {0,};
 
   if(homedir[0])
@@ -1889,3 +2063,28 @@ const char *xitk_get_homedir(void) {
 
   return homedir;
 }
+
+/*
+ * Return 0/1 from char value (valids are 1/0, true/false, 
+ * yes/no, on/off. Case isn't checked.
+ */
+int xitk_get_bool_value(const char *val) {
+  static struct {
+    const char *str;
+    int value;
+  } bools[] = {
+    { "1",     1 }, { "true",  1 }, { "yes",   1 }, { "on",    1 },
+    { "0",     0 }, { "false", 0 }, { "no",    0 }, { "off",   0 },
+    { NULL,    0 }
+  };
+  int i;
+  
+  ABORT_IF_NULL(val);
+
+  for(i = 0; bools[i].str != NULL; i++) {
+    if(!(strcasecmp(bools[i].str, val)))
+      return bools[i].value;
+  }
+
+  return 0;
+}
