@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.h,v 1.111 2003/12/07 23:20:41 f1rmb Exp $
+ * $Id: xitk.h,v 1.130 2004/07/02 22:26:35 f1rmb Exp $
  *
  * Unique public header file for xitk TK.
  *
@@ -37,14 +37,18 @@
 #include <stdarg.h>
 #include "xitk/Imlib-light/Imlib.h"
 
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
+
 #ifdef NEED_MRLBROWSER
 #include <xine.h>
 #endif
 
 #define XITK_MAJOR_VERSION          (0)
 #define XITK_MINOR_VERSION          (10)
-#define XITK_SUB_VERSION            (6)
-#define XITK_VERSION                "0.10.6"
+#define XITK_SUB_VERSION            (7)
+#define XITK_VERSION                "0.10.7"
 
 #define XITK_CHECK_VERSION(major, minor, sub)                          \
                                     (XITK_MAJOR_VERSION > (major) ||   \
@@ -228,13 +232,12 @@ typedef struct {
 /* Grouped widgets */
 #define WIDGET_GROUP_MASK           0x03FFE000
 #define WIDGET_GROUP_BROWSER        0x00002000
-#define WIDGET_GROUP_FILEBROWSER    0x00004000
-#define WIDGET_GROUP_MRLBROWSER     0x00008000
-#define WIDGET_GROUP_COMBO          0x00010000
-#define WIDGET_GROUP_TABS           0x00020000
-#define WIDGET_GROUP_INTBOX         0x00040000
-#define WIDGET_GROUP_DOUBLEBOX      0x00080000
-#define WIDGET_GROUP_MENU           0x00100000
+#define WIDGET_GROUP_MRLBROWSER     0x00004000
+#define WIDGET_GROUP_COMBO          0x00008000
+#define WIDGET_GROUP_TABS           0x00010000
+#define WIDGET_GROUP_INTBOX         0x00020000
+#define WIDGET_GROUP_DOUBLEBOX      0x00040000
+#define WIDGET_GROUP_MENU           0x00080000
 
 /* Real widgets. */
 #define WIDGET_TYPE_MASK            0x00001FFF
@@ -315,6 +318,20 @@ typedef struct {
 #define WM_TYPE_LARSWM              0x0000000B
 #define WM_TYPE_DTWM                0x0000000C
 
+typedef enum {
+  WINDOW_TYPE_DESKTOP = 1,
+  WINDOW_TYPE_DOCK,
+  WINDOW_TYPE_TOOLBAR,
+  WINDOW_TYPE_MENU,
+  WINDOW_TYPE_UTILITY,
+  WINDOW_TYPE_SPLASH,
+  WINDOW_TYPE_DIALOG,
+  WINDOW_TYPE_NORMAL
+} xitk_wm_window_type_t;
+
+void xitk_set_wm_window_type(Window window, xitk_wm_window_type_t type);
+void xitk_unset_wm_window_type(Window window, xitk_wm_window_type_t type);
+
 /* 
  * See xitk_widget_list_[set/get]()
  */
@@ -435,58 +452,6 @@ typedef struct {
   xitk_widget_list_t               *parent_wlist;
 } xitk_browser_widget_t;
 
-typedef struct {
-  int                               magic;
-  ImlibData                        *imlibdata;
-
-  char                             *skin_element_name;
-  Window                            window_trans;
-  int                               layer_above;
-
-  int                               x;
-  int                               y;
-  char                             *window_title;
-  char                             *resource_name;
-  char                             *resource_class;
-
-  struct {
-    char                           *skin_element_name;
-  } sort_default;
-
-  struct {
-    char                           *skin_element_name;
-  } sort_reverse;
-
-  struct {
-    char                           *cur_directory;
-    char                           *skin_element_name;
-  } current_dir;
-  
-  xitk_dnd_callback_t               dndcallback;
-
-  struct {
-    char                           *caption;
-    char                           *skin_element_name;
-  } homedir;
-
-  struct {
-    char                           *caption;
-    char                           *skin_element_name;
-    xitk_string_callback_t          callback;
-  } select;
-
-  struct {
-    char                           *caption;
-    char                           *skin_element_name;
-  } dismiss;
-
-  struct {
-    xitk_simple_callback_t          callback;
-  } kill;
- 
-  xitk_browser_widget_t             browser;
-} xitk_filebrowser_widget_t;
-
 #ifdef NEED_MRLBROWSER
 
 typedef struct {
@@ -500,6 +465,8 @@ typedef struct {
   char                             *skin_element_name;
   Window                            window_trans;
   int                               layer_above;
+  Pixmap                           *icon;
+  int                               set_wm_window_normal;
 
   int                               x;
   int                               y;
@@ -628,7 +595,8 @@ typedef struct {
 
 struct xitk_menu_entry_s {
   char                             *menu;
-  char                             *type; /* NULL, <separator>, <branch>, <check>, <checked> */
+  char                             *shortcut; /* displayed (can be NULL) */
+  char                             *type;     /* NULL, <separator>, <branch>, <check>, <checked> */
   xitk_menu_callback_t              cb;
   void                             *user_data;
 };
@@ -701,6 +669,11 @@ void xitk_init(Display *display, int verbosity);
  */
 char *xitk_set_locale(void);
 
+/*
+ *
+ */
+long int xitk_get_last_keypressed_time(void);
+
 /*
  *
  */
@@ -725,6 +698,7 @@ void xitk_set_window_layer(Window window, int layer);
  *
  */
 void xitk_set_ewmh_fullscreen(Window window);
+void xitk_unset_ewmh_fullscreen(Window window);
 
 /*
  * This function start the widget live. It's a block function,
@@ -744,6 +718,8 @@ void xitk_stop(void);
  */
 char *xitk_get_system_font(void);
 char *xitk_get_default_font(void);
+int xitk_get_xmb_enability(void);
+void xitk_set_xmb_enability(int value);
 int xitk_get_black_color(void);
 int xitk_get_white_color(void);
 int xitk_get_background_color(void);
@@ -755,6 +731,7 @@ long int xitk_get_timer_dbl_click(void);
 int xitk_get_barstyle_feature(void);
 unsigned long xitk_get_warning_foreground(void);
 unsigned long xitk_get_warning_background(void);
+int xitk_get_menu_shortcuts_enability(void);
 
 
 /*
@@ -1164,11 +1141,13 @@ xitk_widget_t *xitk_noskin_labelbutton_create (xitk_widget_list_t *wl,
  * Change label of button 'widget'.
  */
 int xitk_labelbutton_change_label(xitk_widget_t *, char *);
+int xitk_labelbutton_change_shortcut_label(xitk_widget_t *, char *, int, char *);
 
 /**
  * Return label of button 'widget'.
  */
 char *xitk_labelbutton_get_label(xitk_widget_t *);
+char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *);
 
 /**
  * Get state of button 'widget'.
@@ -1337,7 +1316,7 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start);
 /**
  * Update the list, and rebuild button list
  */
-void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, int len, int start);
+void xitk_browser_update_list(xitk_widget_t *w, const char *const *list, const char *const *shortcut, int len, int start);
 
 /**
  * slide up.
@@ -1406,65 +1385,6 @@ void xitk_browser_warp_jump(xitk_widget_t *w, char *key, int modifier);
 
 xitk_widget_t *xitk_browser_get_browser(xitk_widget_t *w);
 
-/*
- * Filebrowser
- */
-/**
- *
- */
-xitk_widget_t *xitk_filebrowser_create(xitk_widget_list_t *wl,
-				       xitk_skin_config_t *skonfig, xitk_filebrowser_widget_t *fb);
-
-/**
- *
- */
-void xitk_filebrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig);
-
-/**
- *
- */
-int xitk_filebrowser_is_running(xitk_widget_t *w);
-
-/**
- *
- */
-int xitk_filebrowser_is_visible(xitk_widget_t *w);
-
-/**
- *
- */
-void xitk_filebrowser_hide(xitk_widget_t *w);
-
-/**
- *
- */
-void xitk_filebrowser_show(xitk_widget_t *w);
-
-/**
- *
- */
-void xitk_filebrowser_set_transient(xitk_widget_t *w, Window window);
-
-/**
- *
- */
-void xitk_filebrowser_destroy(xitk_widget_t *w);
-
-/**
- *
- */
-char *xitk_filebrowser_get_current_dir(xitk_widget_t *w);
-
-/**
- *
- */
-int xitk_filebrowser_get_window_info(xitk_widget_t *w, window_info_t *inf);
-
-/**
- *
- */
-Window xitk_filebrowser_get_window_id(xitk_widget_t *w);
-
 #ifdef NEED_MRLBROWSER
 /**
  *
@@ -1524,6 +1444,18 @@ void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned lo
 
 #endif
 
+/*
+ * return len of keystring (stored in kbuf)
+ * define ksym
+ * kbuf should be large enought (256 usually)
+ */
+int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[]);
+
+/**
+ * return keypressed
+ */
+KeySym xitk_get_key_pressed(XEvent *event);
+
 /**
  * All states of modifiers (see xitk_get_key_modifier() bellow).
  */
@@ -1564,7 +1496,7 @@ void xitk_inputtext_change_text(xitk_widget_t *it, char *text);
 /*
  * Alloc a xitk_skin_config_t* memory area, nullify pointers.
  */
-xitk_skin_config_t *xitk_skin_init_config(void);
+xitk_skin_config_t *xitk_skin_init_config(ImlibData *);
 
 /*
  * Release all allocated memory of a xitk_skin_config_t* variable (element chained struct too).
@@ -1600,6 +1532,11 @@ int xitk_skin_get_direction(xitk_skin_config_t *skonfig, const char *str);
  */
 int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
 
+/*
+ *
+ */
+int xitk_skin_get_printability(xitk_skin_config_t *, const char *);
+
 /*
  *
  */
@@ -1705,6 +1642,16 @@ char *xitk_skin_get_animation(xitk_skin_config_t *);
  */
 int xitk_skin_get_browser_entries(xitk_skin_config_t *, const char *);
 
+/*
+ *
+ */
+xitk_image_t *xitk_skin_get_image(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+int xitk_skin_get_max_buttons(xitk_skin_config_t *, const char *);
+
 /*
  *
  */
@@ -1793,6 +1740,37 @@ int xitk_font_get_descent(xitk_font_t *xtfs, const char *c);
  */
 void xitk_font_set_font(xitk_font_t *xtfs, GC gc);
 
+typedef struct {
+#ifdef HAVE_ICONV
+  iconv_t    id;
+#else
+  int        id;
+#endif
+} xitk_recode_t;
+
+/**
+ * get current locale encoding 
+ */
+char *xitk_get_system_encoding(void);
+
+/**
+ * prepare recoding
+ *   - when one of encoding is NULL, is prepared no conversion, strings will
+ *     be strdup()ed
+ *   - when encoding is "", it's got from system
+ */
+xitk_recode_t *xitk_recode_init(const char *src_encoding, const char *dst_encoding);
+
+/**
+ * destroy recoding
+ */
+void xitk_recode_done(xitk_recode_t *xr);
+
+/**
+ * recode string 'src' into 'dst' according to prepared 'xr'
+ */
+char *xitk_recode(xitk_recode_t *xr, const char *src);
+
 
 /**
  *
@@ -1906,8 +1884,7 @@ xitk_image_t *xitk_image_create_image_from_string(ImlibData *im,
 						  int width, int align, char *str);
 xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height);
 
-xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, 
-							int width, int height, int depth);
+xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, int width, int height, int depth);
 
 xitk_pixmap_t *xitk_image_create_xitk_pixmap(ImlibData *im, int width, int height);
 
@@ -1919,7 +1896,6 @@ void xitk_image_destroy_xitk_pixmap(xitk_pixmap_t *p);
  * Free an image object.
  */
 void xitk_image_free_image(ImlibData *im, xitk_image_t **src);
-
 /**
  *
  */
@@ -1945,6 +1921,10 @@ void draw_bevel_three_state(ImlibData *im, xitk_image_t *p);
  */
 void draw_bevel_two_state(ImlibData *im, xitk_image_t *p);
 
+void draw_three_state_round_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked);
+void draw_three_state_check_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked);
+
+
 void draw_paddle_three_state_vertical(ImlibData *im, xitk_image_t *p);
 void draw_paddle_three_state_horizontal(ImlibData *im, xitk_image_t *p);
 
@@ -2116,6 +2096,11 @@ void xitk_set_window_title(Display *display, Window window, char *title);
  */
 void xitk_window_set_window_title(ImlibData *im, xitk_window_t *w, char *title);
 
+/*
+ *
+ */
+int xitk_is_window_iconified(Display *display, Window window);
+
 /*
  *
  */
@@ -2216,13 +2201,23 @@ xitk_window_t *xitk_window_dialog_yesnocancel(ImlibData *im, char *title,
 /*
  *
  */
-xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_checkbox_two_buttons_with_width(ImlibData *im, char *title,
 							 char *button1_label, char *button2_label,
 							 xitk_state_callback_t cb1, 
 							 xitk_state_callback_t cb2, 
+                                                         char *checkbox_label, int checkbox_state,
+                                                         xitk_state_callback_t cb3,
 							 void *userdata, 
 							 int window_width, int align, char *message, ...);
 
+#ifdef __GNUC__
+#define xitk_window_dialog_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, userdata, window_width, align, message, args...) \
+  xitk_window_dialog_checkbox_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, NULL, 0, NULL, userdata, window_width, align, message, ##args)
+#else
+#define xitk_window_dialog_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, userdata, window_width, align, ...) \
+  xitk_window_dialog_checkbox_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, NULL, 0, NULL, userdata, window_width, align, __VA_ARGS__)
+#endif
+
 /**
  *
  */
@@ -2247,6 +2242,8 @@ xitk_window_t *xitk_window_dialog_yesno(ImlibData *im, char *title,
   xitk_window_dialog_yesno_with_width(im, title, ycb, ncb, userdata, 400, align, __VA_ARGS__)
 #endif
 
+void xitk_window_set_parent_window(xitk_window_t *xwin, Window parent);
+
 void xitk_window_set_modal(xitk_window_t *w);
 void xitk_window_dialog_set_modal(xitk_window_t *w);
 
